#ifndef GeometricMeanResidualsTupleProcessor_h
#define GeometricMeanResidualsTupleProcessor_h 1

//C++
#include <string>

//Marlin
#include "marlin/Processor.h"

//Aida
#include <AIDA/AIDA.h>
#include <marlin/AIDAProcessor.h>
#include <AIDA/ITupleFactory.h>
#include <AIDA/ITuple.h>

//LCIO
#include "lcio.h"

using namespace lcio ;
using namespace marlin ;

namespace marlintpc
{

  /**  Processor for outputting inclusive/exclusive residual data to useful Root tuples.
   *   The idea is that this processor performs no cuts, but saves a variety of data
   *   so that different cuts can be made easily from the same data.
   *   
   *   NOTE: At this time, this processor is optimized for Micromegas, and may not work for other data/geometries.
   * 
   *   <h4>Input - Prerequisites</h4>
   *   Needs a collection of Tracks.
   *
   *   <h4>Output</h4> 
   *   A tuple containing the residuals, as well as information about the track and hit 
   *   that went in to each entry (e.g., track parameters, hit module, row, column, etc.)
   * 
   *   @param TrackCollection Name of the input Track Collection
   * 
   *   @author P. Hayman, Carleton University
   *   @version $Id: GeometricMeanResidualsTupleProcessor.h$ 
   */

  class TrackFitterFactory;

  class GeometricMeanResidualsTupleProcessor : public Processor 
  {

    public:

      virtual Processor*  newProcessor() { return new GeometricMeanResidualsTupleProcessor ; }


      GeometricMeanResidualsTupleProcessor() ;
      ~GeometricMeanResidualsTupleProcessor() ;

      /** Called at the begin of the job before anything is read.
       * Used to initialize the processor, e.g. book histograms.
       */
      virtual void init() ;

      /** Called for every run.
       */
      virtual void processRunHeader( LCRunHeader* run ) ;

      /** Called for every event - the working horse.
       */
      virtual void processEvent( LCEvent * evt ) ; 


      virtual void check( LCEvent * evt ) ; 


      /** Called after data processing for clean up.
       */
      virtual void end() ;


    protected:

      /** Input collection name.
       */
      lcio::StringVec _input_track_collection_names ;

      EVENT::LCParameters * _trackCollectionParameters;
      int _nRun ;
      int _nEvt ;

      int _track_type ;
      int _track_fitter_type ;
      int _kNlayers;

      /** Tuple for track parameters
       */
      AIDA::ITuple* _output_tuple ;

  } ;

}
#endif



