/*
 *
 *   Created on: September 30, 2013
 *      Author: amuennich
 */

#ifndef HISTOGRAMWIDTHEXTRACTIONTOOLS
#define HISTOGRAMWIDTHEXTRACTIONTOOLS

#include "TH1.h"

namespace marlintpc
{
  /**
   * A collection of tools to evaluate the width of a distribution in a histogram.
   * Available methods:
   * 1. RMS90
   * 2. Gaussian fit within range of 3 sigma
   * 3. rms after 5 sigma iterative cut of tails
   */
  
  class HistogramWidthExtractionTools
  {
  public:
    ///constructor
    HistogramWidthExtractionTools();
    
    ///destructor
    virtual ~HistogramWidthExtractionTools();

    /**Function to calculate width and mean and errors of the given histogram to be evaluated using the chosen method.
     * method=1: RMS90
     * method=2: Gaussian fit within range of 3 sigma
     * method=3: rms after 5 sigma iterative cut of tails
     * method=4: Gaussian fit within range of 3 sigma only on the rising slope for asymmetric shapes
     */
    void Calculate(TH1* histogram, int method=1);
    ///Function giving back the width as a measure for the resolution for the chosen method
    double GetWidth(){return _width;};
    ///Function giving back the error on the width for the chosen method
    double GetWidthError(){return _width_err;};
    ///Function giving back the mean for the chosen method
    double GetMean(){return _mean;};
    ///Function giving back the error on the mean for the chosen method
    double GetMeanError(){return _mean_err;};
    ///Function giving back the height for the chosen method
    double GetHeight(){return _height;};
    ///Function giving back the error on the height for the chosen method
    double GetHeightError(){return _height_err;};

    ///Function giving back the range used in the method
    std::pair<double, double> GetRange()  { return _range; };
    
    
  private:
    
    ///Function calculating the rms90 and mean90
    void CalculateRMS90(TH1* histogram);
    
    ///Function carrying out the 3 sigma range fit
    void Fit3SigmaRange(TH1* histogram);

    void FitLowerHalf3SigmaRange(TH1* histogram);
    
    ///Function carrying out the 5 sigma iterative cut
    void CalculateRMS5SigmaRange(TH1* histogram);
    
    ///Variables to hold results
    double _width;
    double _width_err;
    double _mean;
    double _mean_err;
    double _height;
    double _height_err;

    std::pair<double,double> _range;

       
  };


}
#endif
