#ifndef HITCORRELATIONPROCESSOR_H
#define HITCORRELATIONPROCESSOR_H

#include <marlin/Processor.h>
#include "marlin/Global.h"

#include <lcio.h>
#include <string>
#include "TrackFitterBase.h"
#include <EVENT/TrackerHit.h>

//MarlinTPC
#include "ModuleDistortionCalculator.h"

//GEAR
#include "gear/TPCParameters.h"

//ROOT
#include "TH2.h"
#include "TNtuple.h"


// In case you want AIDA histograms

namespace marlintpc
{

/**  Hit Correlation calculator
 *
 *   Determines the Hit correlations of a selectable row with all other rows
 *
 *
 * @author Felix Mueller, DESY
 *
 */

class HitCorrelationProcessor : public marlin::Processor
{
public:

	virtual Processor*  newProcessor() { return new HitCorrelationProcessor; }

	HitCorrelationProcessor();

	virtual void init();

	virtual void processRunHeader(lcio::LCRunHeader* run);

	virtual void processEvent(lcio::LCEvent* evt);

	virtual void check(lcio::LCEvent* evt);

	virtual void end();



protected:
	/* the place for protected and private member data and functions */
	std::string _inputColName; ///< Name of the input collection

private:
	double _bfieldScaleFactor; ///< scale factor for magnetic field (default: 1.0)
	int _xyResolutionBins; ///< Number of bins in the xy-residual histogram
	float _xyResolutionRange; ///< Range of the xy-residual histogram
	int _zResolutionBins; ///< Number of bins in the z-residual histogram
	float _zResolutionRange; ///< Range of the z-residual histogram
	int _observedHitRow; ///< Row with which the correlation is calculated
	int _maxRow; ///< maximum number of rows on the endplate

	std::map< int, TH2*> _zResidualCorrelatedRow;
	std::map< int, TH2*> _xyResidualCorrelatedRow;
//	TH2 * _zResidualCorrelatedRows;
//	TH2 * _xyResidualCorrelatedRows;
	TH2 * _zResidualUncorrelatedRows;
	TH2 * _xyResidualUncorrelatedRows;

	TrackFitterBase * _trackFitter;

	ModuleDistortionCalculator * _myModuleDistortionCalculator;
	const gear::TPCParameters* _gearParameters;

};
} // namespace marlintpc
#endif // HITCORRELATIONPROCESSOR_H
