#ifndef HITQUALITYPROCESSOR_H
#define HITQUALITYPROCESSOR_H

#include <marlin/Processor.h>
#include <lcio.h>
#include <string>

//ROOT
#include "TH1D.h"
#include "TNtuple.h"

// Aida
#ifdef MARLIN_USE_AIDA
//AIDA
#include <marlin/AIDAProcessor.h>

#endif

namespace marlintpc
{
/**  This processor creates histograms for the number of hits in over range and with dead channel
 *
 * 
 * @author F. Mueller, DESY
 *
 */

class HitQualityProcessor : public marlin::Processor
{
 public:
  
  virtual Processor*  newProcessor() { return new HitQualityProcessor; }
  
  HitQualityProcessor();
  
  virtual void init();
  
  virtual void processRunHeader(lcio::LCRunHeader* run);
  
  virtual void processEvent(lcio::LCEvent* evt);
  
  virtual void check(lcio::LCEvent* evt);
  
  virtual void end();
  
  
 protected:
  /* the place for protected and private member data and functions */
  std::string _inputColName; ///< Name of the input collection

  TH1D * _overRangeHitsHisto; ///< histogram for the hits in over range
  TH1D * _deadChannelsHitsHisto; ///< histogram for the hits with a dead channel

  TNtuple * _overRangeTuple; ///< tuple for the hits in over range
  TNtuple * _deadChannelsTuple; ///< tuple for the hits with a dead channel

  float _overRangeHistoMin; ///< Minimum of the histogram for the hits in over range
  float _overRangeHistoMax; ///< Maximum of the histogram for the hits in over range

  float _deadChannelHistoMin; ///< Minimum of the histogram for the hits with dead channel
  float _deadChannelHistoMax; ///< Maximum of the histogram for the hits with dead channel
};
} // namespace marlintpc
#endif // HITQUALITYPROCESSOR_H
