#ifndef PULSEPERHITPROCESSOR_H
#define PULSEPERHITPROCESSOR_H

#include <marlin/Processor.h>
#include <lcio.h>
#include <string>
#include <map>
#include <utility>
#include <cstdlib>

//GEAR
#include "gear/TPCParameters.h"

// Aida
#ifdef MARLIN_USE_AIDA
//AIDA
#include <marlin/AIDAProcessor.h>
#endif

//ROOT
#include "TH1.h"
#include "TH1D.h"

namespace marlintpc
{
/**
 * This processor creates histograms for the number of pulses per hit and for the charge of the pulses. It also creates a graph for the number of pulses per hit vs row.
 * @author P. Malek, DESY
 *
 * \par Credits:
 * The processor skeleton was generated by the script \c createProcessor.py
 *
 */

class PulsePerHitProcessor : public marlin::Processor
{
 public:

  virtual Processor*  newProcessor() { return new PulsePerHitProcessor; }

  PulsePerHitProcessor();

  virtual void init();

  virtual void processRunHeader(lcio::LCRunHeader* run);

  virtual void processEvent(lcio::LCEvent* evt);

  virtual void check(lcio::LCEvent* evt);

  virtual void end();


 protected:
  /* the place for protected and private member data and functions */
  std::string _inputColName; ///< Name of the input collection

  float _pulsesPerHitHistoMin; ///< Minimum of the histogram for the pulses per hit
  float _pulsesPerHitHistoMax; ///< Maximum of the histogram for the pulses per hit

  std::map< std::pair<int,int>, TH1*> _pulsesPerHitPerRowHistos;///< histograms for pulses per hit in each row

  TH1D* _pulsesPerHitHisto; ///< histogram for the pulses per hit

  const gear::TPCParameters* _gearParameters;

};
} // namespace marlintpc
#endif // PULSEPERHITPROCESSOR_H
