#ifndef TIMEPIX_OCCUPANCY_PROCESSOR_H
#define TIMEPIX_OCCUPANCY_PROCESSOR_H 1

//LCIO
#include <lcio.h>
#include <IMPL/LCCollectionVec.h>
#include <EVENT/TrackerData.h>

//MARLIN
#include <marlin/Processor.h>
#include <marlin/Global.h>

//C++
#include <map>
#include <iostream>

//root+AIDA
#include <TH2I.h>
//#ifdef MARLIN_USE_AIDA // the whole processor does not make sense without aida
#include <AIDA/AIDA.h>
#include <marlin/AIDAProcessor.h>
#include <AIDA/IHistogramFactory.h>
#include <AIDA/IHistogram2D.h>
// endif


namespace AIDA{
  class IHistogram2D;
}


namespace marlintpc
{

    class TrackFitterFactory;
//    class TrackFitterBase;

    /** Count how many times a pixel has been hit on the TimePix chip
     *
     *  <h4>Input - Prerequisites</h4>
     *  \li A Collection of TimePixRawData or TimePixZeroSuppressedRawData
     *
     *  <h4>Output</h4>
     *  \li One 2D Histogram with the number of hits per pixel for each chip
     *
     *  @param InputColName Name of input collection with lcio::TrackerRawData (default: TimePixRawData)
     *  @param NColumns Number of colmns on a chip (default: 256)
     *  @param NRows Number of rows on a chip (default: 256)
     *
     *  @author Martin Killenberg, University of Bonn
     */

    class TimePixOccupancyProcessor : public marlin::Processor{
			      
    public:
    
	virtual Processor*  newProcessor() { return new TimePixOccupancyProcessor ; }
    
	TimePixOccupancyProcessor();

	~TimePixOccupancyProcessor();

	virtual void init();

	virtual void processRunHeader(lcio::LCRunHeader* run ) ;

	virtual void processEvent(lcio::LCEvent * evt ) ; 

        virtual void end();
    
    protected:

     std::string _inputColName ;    ///< the name of the input collection
     
//     TrackFitterBase *trackFitter;
//     TrackFitterFactory *_fitterFactory; ///< The fitter factory

     ///< Map with the occupancy histograms, one per chip
     std::map<int,     AIDA::IHistogram2D *>  _timePixOccupancyHistos;

     int _nColumns;       ///< Number of colmns on the chip
     int _nRows;          ///< Number of rows on the chip
     int _nColumnModules; ///< Number of modules/chips in a row
     int _nRowModules;    ///< Number of modules/chips in a row

     std::vector<int> _chipDirections; ///< Vector containing the position of the zero on the chip
     std::vector<int> _chipIDs;        ///< ID number(s) of the chip(s)

     TH2I* _allChipsOccupancy;         ///< Histo showing the occupancy for all chips

    };
}

#endif // TIMEPIX_OCCUPANCY_PROCESSOR_H
