#ifndef TRACKFINDEREVALUATEPROCESSOR_H
#define TRACKFINDEREVALUATEPROCESSOR_H

#include <marlin/Processor.h>
#include <lcio.h>
#include <string>

#include "TH1D.h"
#include "TH2D.h"
// In case you want AIDA histograms


namespace marlintpc
{
/**  Processor to evaluate the performance of the track finding algorithm.
 * It plots the number of track versus the number of of hits on a track.
 * @author F. Mueller, DESY
 *
 */

class TrackFinderEvaluateProcessor : public marlin::Processor
{
 public:
  
  virtual Processor*  newProcessor() { return new TrackFinderEvaluateProcessor; }
  
  TrackFinderEvaluateProcessor();
  
  virtual void init();
  
  virtual void processRunHeader(lcio::LCRunHeader* run);
  
  virtual void processEvent(lcio::LCEvent* evt);
  
  virtual void check(lcio::LCEvent* evt);
  
  virtual void end();
  
  
 protected:
  /* the place for protected and private member data and functions */
  std::string _inputColName; ///< Name of the input collection

  TH2D* hitsTrackNumber;
  TH2D* hitsAddedTrackNumber;
  TH1D* tracksPerEvent;
  TH1D* hitsPerTrack;

};
} // namespace marlintpc
#endif // TRACKFINDEREVALUATEPROCESSOR_H
