#ifndef WriteXrayObjectsToROOTTreeProcessor_h
#define WriteXrayObjectsToROOTTreeProcessor_h 1

#include "GridPixXrayObject.h"
#include "XrayGeometryService.h"

//c++
#include <string>
#include <sstream>
#include <map>
#include <vector>

//marlin
#include <marlin/Global.h>
#include <marlin/Processor.h>

//lcio
#include <lcio.h>
#include <EVENT/LCCollection.h>
#include <IMPL/LCCollectionVec.h>
#include <EVENT/LCRelation.h>
#include <EVENT/TrackerHit.h>
#include <EVENT/TrackerData.h>

//gear
#include <gear/GEAR.h>
#include <gear/TPCModule.h>
#include <gear/TPCParameters.h>
#include <gear/PadRowLayout2D.h>

//root
#include "TFile.h"
#include "TTree.h"

namespace marlintpc {

class WriteXrayObjectsToROOTTreeProcessor;

class WriteXrayObjectsToROOTTreeProcessor: public marlin::Processor {

public:

	virtual Processor* newProcessor() {return new WriteXrayObjectsToROOTTreeProcessor;}

	WriteXrayObjectsToROOTTreeProcessor();

	~WriteXrayObjectsToROOTTreeProcessor();

	virtual void init();

	virtual void processRunHeader(lcio::LCRunHeader *run);

	virtual void processEvent(lcio::LCEvent *evt);

	virtual void end();

protected:

	void writeHitToMap(std::map<int,double> *map, lcio::TrackerHit *hit);

	void writeHitToVectors(std::vector<int> *xVec, std::vector<int> *yVec, std::vector<double> *valVec, lcio::TrackerHit *hit);

	std::string _inputXrayObjectCollectionName;

	std::string _rootFileName;
	std::string _rootFileDescription;

	TFile* _rootFile;

	std::string _rootTreeDescription;

	TTree* _rootTree;

	XrayGeometryService* _geometryService;

	int _eventCounter;
	int _runCounter;

	//branches for the tree
	int* _runNumber;
	int* _runType;
	int* _eventNumber;
	ULong64_t* _timestamp;

	int* _numberOfPixels;
	int* _totalCharge;

	double* _energyFromCharge;
	double* _energyFromPixels;

	double* _positionX;
	double* _positionY;

	double* _rotationAngle;

	double* _rmsTransverse;
	double* _rmsLongitudinal;

	double* _excentricity;

	double* _radius;

	double* _width;
	double* _length;

	double* _skewnessTransverse;
	double* _skewnessLongitudinal;

	double* _kurtosisTransverse;
	double* _kurtosisLongitudinal;

	double* _likelihoodMarlin;

	double* _fractionWithinRmsTransverse;

	std::map<int,double>* _pixelMap;
	std::vector<int>* _xVector;
	std::vector<int>* _yVector;
	std::vector<double>* _valueVector;

	//to be continued


};//class WriteXrayObjectsToROOTTreeProcessor

}//namespace marlintpc

#endif
