#ifndef XrayAnalysisProcessor_h
#define XrayAnalysisProcessor_h 1

#include "GridPixXrayObject.h"

//c++
#include <string>
#include <vector>

//marlin
#include <marlin/Processor.h>

//root
#include <TH1D.h>
#include <TH2D.h>

namespace marlintpc{
  
  class XrayAnalysisProcessor;

  class XrayAnalysisProcessor : public marlin::Processor{
    
    public:
      
      virtual Processor* newProcessor() { return new XrayAnalysisProcessor; }
      
      XrayAnalysisProcessor();
      
      ~XrayAnalysisProcessor();
      
      virtual void init();
      
      //virtual void processRunHeader( lcio::LCRunheader* run );
      
      virtual void processEvent( lcio::LCEvent* evt );
      
      virtual void end();
      
    protected:
      
      std::string _inputCol;
      
      TH1D* _pixelsHisto;
      TH1D* _chargeHisto;
      TH1D* _energyHisto;
      TH1D* _likelihoodHisto;
      
      TH2D* _centerMap;
      
      bool _energySplit;
      
      std::vector<int> _blindRangeTimestamps;
      int _nBlinded;

      double _split;
      
      std::vector<TH1D*> _likelihoodHistoVec;
      std::vector<TH2D*> _centerMapVec;
      
      int _binsPixels;
      double _minPixels;
      double _maxPixels;
      
      int _binsCharge;
      double _minCharge;
      double _maxCharge;
      
      int _binsEnergy;
      double _minEnergy;
      double _maxEnergy;
      
      int _binsLikelihood;
      double _minLikelihood;
      double _maxLikelihood;
      
      TH1D* _histoXraysPerEvent;    
    
  };//class XrayAnalysisProcessor
  
}//namespace marlintpc
  
#endif
