#ifndef XrayCutApplicationProcessor_h
#define XrayCutApplicationProcessor_h 1

#include "GridPixXrayObject.h"
#include "XrayGeometryService.h"

//c++
#include <string>

//lcio
#include <lcio.h>
#include <EVENT/TrackerHit.h>
#include <EVENT/LCRelation.h>
#include <EVENT/LCCollection.h>
#include <IMPL/LCCollectionVec.h>

//marlin
#include <marlin/Processor.h>

namespace marlintpc{
  
  class XrayCutApplicationProcessor;
  
  class XrayCutApplicationProcessor : public marlin::Processor{
    
    public:
      
      virtual Processor* newProcessor() { return new XrayCutApplicationProcessor; }
      
      XrayCutApplicationProcessor();
      
      virtual void init();
      
      virtual void processRunHeader( lcio::LCRunHeader* run );
      
      virtual void processEvent( lcio::LCEvent * evt ); 
      
      virtual void end();
      
    protected:
      
      std::string _inputCol;
      //std::string _inputHits;
      
      std::string _outputCol;
      //std::string _outputHits;
      
      int _outputIsTransient;
      
      double _centerX;
      double _centerY;
      double _cutRadius;
      
      double _lowCutX;
      double _highCutX;

      double _lowCutY;
      double _highCutY;

      double _lowCutExcentricity;
      double _highCutExcentricity;
      
      double _lowCutSkewnessT;
      double _highCutSkewnessT;
      
      double _lowCutKurtosisT;
      double _highCutKurtosisT;
      
      double _lowCutKurtosisL;
      double _highCutKurtosisL;

      double _lowCutRmsY;
      double _highCutRmsY;
      
      double _lowCutLength;
      double _highCutLength;      
      
      int _lowCutPixels;
      int _highCutPixels;
      
      int _lowCutCharge;
      int _highCutCharge;
      
      double _lowCutEnergy;
      double _highCutEnergy;
      
      double _lowCutLikelihood;
      double _highCutLikelihood;
      
      int _highCutNumberXrays;
      
      XrayGeometryService* _geoService;   
      
      int _numberBefore;
      int _numberAfter;
      
  };//class XrayCutApplicationProcessor 
  
}//namespace marlintpc

#endif
