#ifndef XrayEnergyCalibrationProcessor_h
#define XrayEnergyCalibrationProcessor_h 1

#include "GridPixXrayObject.h"

//c++
#include <string>
#include <vector>
#include <cmath>

//lcio
#include <lcio.h>
#include <EVENT/LCCollection.h>
#include <IMPL/LCCollectionVec.h>
#include <EVENT/LCRelation.h>
#include <IMPL/LCRelationImpl.h>
#include <EVENT/TrackerHit.h>

//marlin
#include <marlin/Processor.h>
#include <marlin/Global.h>

//root
#include "TH1D.h"

//aida
#include <AIDA/AIDA.h>
#include <marlin/AIDAProcessor.h>

namespace marlintpc{
  
  class XrayEnergyCalibrationProcessor;
  
  class XrayEnergyCalibrationProcessor : public marlin::Processor{
    
    public:
      
      virtual Processor* newProcessor() {return new XrayEnergyCalibrationProcessor;}
      
      XrayEnergyCalibrationProcessor();
      
      ~XrayEnergyCalibrationProcessor();
      
      virtual void init();
      
      virtual void processRunHeader( lcio::LCRunHeader* run );
      
      virtual void processEvent( LCEvent * evt );
      
      virtual void end();
      
    protected:
      
      double energy( int value, std::vector<float>* parameters);

      std::string _inputCollection;
      
      std::string _outputXrays;

      std::string _outputCollection;
      
      int _outputIsTransient;
      
      std::vector<float> _energyFromChargeParameterVector;
      std::vector<float> _energyFromPixelsParameterVector;
      
      TH1D* _energyFromChargeHistogram;
      TH1D* _energyFromPixelsHistogram;
  
  };//class XrayEnergyCalibrationProcessor
  
}//namespace marlintpc

#endif
