#include <vector>
#include <iostream>
#include <fstream>
#include "sstream"
#include <utility>
#include <cstdarg>
#include <cmath>

#include "TLatex.h"
#include "TCanvas.h"
#include "TAxis.h"
#include "THStack.h"
#include "TGraphErrors.h"
#include "TMultiGraph.h"
#include "TFile.h"
#include "TNtuple.h"
#include "TChain.h"
#include "TStyle.h"
#include "TROOT.h"
#include "TFrame.h"
#include "TColor.h"
#include "TMath.h"

/*How to call this:
.L $MARLINTPC/analysis/rootscripts/PulsePerHit.C++
PulsePerHit("path for output","path to the aida file", number of runs in the
following list, list of the runs)
e.g PulsePerHit("../myplots","../AidaFiles/",3,18850,18851,18860)
 */
void GEMChargeVoltage(char* pathToOutput, char* pathToAida, int nRuns, ...)
{

	std::vector<double> x;
	std::vector<double> xerr;
	xerr.push_back(0);
	xerr.push_back(0);
	xerr.push_back(0);
	xerr.push_back(0);
	xerr.push_back(0);

	std::vector<double> y;
	std::vector<double> yerr;





    // loading the style
    gROOT->ProcessLine(".L ${MARLINTPC}/analysis/rootscripts/LCTPCStyle.C");
    gROOT->ProcessLine("LCTPCStyle()");

    // open a root file to store the graphs in
      TFile* outfile = new TFile(Form("%s%s", pathToOutput, "GEMChargeVoltage.root"), "RECREATE");
  	TChain* tupleChain = new TChain("MyGEMPulsingChargeEventAnalysisProcessor/chargeCalibrationTuple");
  	int numberOfChannels=0;
	// initialize the input list
	  va_list argList;
	  va_start(argList, nRuns);

	int runNumber;

	//create colors
			const int BINNING=nRuns;
			Int_t colors[BINNING];

			const Int_t NRGBs = 5;

			Double_t stops[NRGBs] = { 0.00, 0.34, 0.61, 0.84, 1.00 };
			Double_t red[NRGBs]   = { 0.00, 0.00, 0.87, 1.00, 0.51 };
			Double_t green[NRGBs] = { 0.00, 0.81, 1.00, 0.20, 0.00 };
			Double_t blue[NRGBs]  = { 0.51, 1.00, 0.12, 0.00, 0.00 };
			Int_t FI = TColor::CreateGradientColorTable(NRGBs, stops, red, green, blue, BINNING);

			for (int i=0;i<BINNING;i++)
				colors[i] = FI+i;

			// will contain the results from the processors
					THStack* AveragePerEventDist = new THStack();
						AveragePerEventDist->SetTitle("ChargeAverage per Event; charge average per event [adc];# entries");

			// loop over the given runs
			for(int i = 0; i < nRuns; i++)
			{
				// get run number from input
				runNumber = va_arg(argList, int);

				// create stringstream containing the aida file name
				std::stringstream runNumberString;
				std::cout << "run number: " << runNumber << std::endl;
				runNumberString << pathToAida << "aida_run" << runNumber << ".root";

				// open the aida file
				TFile* file = TFile::Open(runNumberString.str().c_str(), "Read");

				// check that the file exists
				if(file == 0)
					{
						std::cout << "This run will be skipped. The aida file does not "
									 "exist. Please check this!" << std::endl;
						continue;
					}
				// cd out of the file to prevent root from automatically deleting the
				// following, when closing the file
				gROOT->cd();


				TH1D* eventChargeAverages = (TH1D*)file->Get("MyGEMPulsingChargeEventAnalysisProcessor/event_ChargeAverages")-> Clone();

				if(eventChargeAverages==0)
					{
						std::cout<<"MyGEMPulsingTimeEventAnalysis: his run will be skipped. One of the necessary root graph does not exist. Please check that you ran the corresponding processors and that the histogram is in the aida file!"<<std::endl;
						continue;
					}

				double ChargeMean = eventChargeAverages -> GetMean();
				double ChargeMeanErr = eventChargeAverages -> GetRMS();

				y.push_back(ChargeMean);
				yerr.push_back(ChargeMeanErr);

				eventChargeAverages->SetLineColor(colors[i]);
				eventChargeAverages->SetMarkerColor(colors[i]);
				eventChargeAverages->Rebin(5);
				std::stringstream name;
				name << "average"<< runNumber;
				eventChargeAverages->SetTitle(name.str().c_str());
				eventChargeAverages->SetName(name.str().c_str());
				AveragePerEventDist->Add(eventChargeAverages);



				// read in Tuple
				TNtuple * chargeCalibrationTuple=(TNtuple*)file->Get("MyGEMPulsingChargeEventAnalysisProcessor/chargeCalibrationTuple");

				if (chargeCalibrationTuple == 0 )
				{
					std::cout<<"This run will be skipped. One of the necessary root tuples does not exist. Please check that you ran the corresponding processors and that the tuple is in the aida file!"<<std::endl;
					continue;
				}
				std::cout << "chargeCalibrationTuple="<<chargeCalibrationTuple->GetEntries()<< std::endl;
				for (int ii=0; ii<1; ii++)
				{
					chargeCalibrationTuple->GetEntry(ii);

					double voltage = chargeCalibrationTuple -> GetArgs()[4];
					x.push_back(voltage);

//						std::cout << " module: " << chargeCalibrationTuple -> GetArgs()[0] <<
//								" pad: " << chargeCalibrationTuple -> GetArgs()[1] <<
//								" MeanCharge: " << chargeCalibrationTuple -> GetArgs()[2] <<
//								" MeanChargeError: " << chargeCalibrationTuple -> GetArgs()[3] <<
//								" Voltage: " << chargeCalibrationTuple -> GetArgs()[4] << std::endl;
				}

				if (i==0)
				{
					numberOfChannels = chargeCalibrationTuple-> GetEntries();

				}
				tupleChain -> Add(runNumberString.str().c_str());



//				file->Close();
//				delete file;
				std::cout << "finished run number: " << runNumber << std::endl;
			}
			// save the results
			outfile->cd();



			for(int i=0; i<5; ++i)
			{
				std::cout << "Voltage: " << x[i] << "; Charge: " << y[i] << std::endl;
//				std::cout << "VoltageErr: " << xerr[i] << "; ChargeErr: " << yerr[i] << std::endl;
			}

			TCanvas* c1 = new TCanvas("c1", "c1");
//	        	gPad->BuildLegend(0.85, 0.2, 0.99, 0.99, "[mm]");

		    // will contain the results from the processors
				TGraphErrors* ChargeVSVoltage = new TGraphErrors(x.size(),&x[0], &y[0], &xerr[0], &yerr[0]);
				ChargeVSVoltage->Draw("A*");
				ChargeVSVoltage->SetTitle("ChargeVSVoltage; Voltage [V]; Charge [adc]");

				// Fit
				TF1 * straightLine = new TF1("straight","pol1",0,5.5);
				straightLine->SetParameters(0,1);
				ChargeVSVoltage->Fit("straight","MFR");
				c1->SaveAs(Form("%s%s", pathToOutput, "ChargeVSVoltage.pdf"));
				c1->SaveAs(Form("%s%s", pathToOutput, "ChargeVSVoltage.png"));
				c1->SaveAs(Form("%s%s", pathToOutput, "ChargeVSVoltage.C"));
				c1->Write();
				std::vector<double> diff,ediff;
				for(int i=0; i<5; ++i)
				{
					diff.push_back(straightLine->Eval(x[i])-y[i]);
					ediff.push_back(sqrt((yerr[i])*(yerr[i]) + pow(straightLine->GetParError(0),2) + pow(x[i]*straightLine->GetParError(1),2) ));

				}

			TCanvas* c2 = new TCanvas("c2", "c2");
			TGraphErrors* ChargeVSVoltageErr = new TGraphErrors(x.size(),&x[0], &diff[0], &xerr[0], &ediff[0]);
					ChargeVSVoltageErr->Draw("A*");
					ChargeVSVoltageErr->SetTitle("ChargeVSVoltageErr; Voltage [V]; Fit-Data");
					c2->SaveAs(Form("%s%s", pathToOutput, "ChargeVSVoltageErr.pdf"));
					c2->SaveAs(Form("%s%s", pathToOutput, "ChargeVSVoltageErr.png"));
					c2->Write();


//			TCanvas* c3 = new TCanvas("c3", "c3");
//							AveragePerEventDist->Draw("nostack");
//							TLegend* legend_AveragePerEventDist = gPad->BuildLegend();



			TCanvas* c4 = new TCanvas("c4", "c4");
//			std::cout<< "tupleChain -> GetEntries(): " << tupleChain -> GetEntries() << std::endl;

			TH1F* fitOffset = new TH1F("fitParameterOffset", "fitParameterOffset", 100, 0, 0);
			fitOffset ->GetXaxis()->SetTitle("Offset");
			fitOffset ->GetYaxis()->SetTitle("#entries");

			TH1F* fitSlope = new TH1F("fitParameterSlope", "fitParameterSlope", 100, 0, 0);
			fitSlope ->GetXaxis()->SetTitle("Slope");
			fitSlope ->GetYaxis()->SetTitle("#entries");

			TH1F* fitChi2 = new TH1F("fitChi2", "fitChi2", 100, 0, 0);
			fitChi2 ->GetXaxis()->SetTitle("%chi^2");
			fitChi2 ->GetYaxis()->SetTitle("#entries");

			std::string _filename = "Fitparameter.txt";
			std::ofstream outputFile;
			outputFile.open(_filename.c_str());

			float pad,module=0;
			tupleChain->SetBranchAddress("module",&module);
			tupleChain->SetBranchAddress("pad",&pad);
				for (int i=0; i<10; ++i)
				{
					tupleChain -> GetEntry(i);
					float temppad=pad;
					float tempmodule=module;

					int n =tupleChain -> Draw("chargeMean:voltage:chargeMeanError",Form("module==%f && pad == %f",module,pad),"goff");
					std::vector<double> voltageError;
//					std::vector<double> chargeMeanErrorScale;
//					for (int iter =0; iter<n; ++iter)
//						{
//							voltageError.push_back(0);
//							chargeMeanErrorScale.push_back(3.5*tupleChain->GetV3()[iter]);
//
//						}
//					TGraph *g = new TGraph(n,tupleChain->GetV2(),tupleChain->GetV1());
//					TGraphErrors *gerr = new TGraphErrors(n,tupleChain->GetV2(),tupleChain->GetV1(), &voltageError[0], &chargeMeanErrorScale[0]);
					TGraphErrors *gerr = new TGraphErrors(n,tupleChain->GetV2(),tupleChain->GetV1(), &voltageError[0], tupleChain->GetV3());
					gerr->Draw("a*");

					// Fit
					TF1 * straightLine1 = new TF1("straight","pol1");
					straightLine1->SetParameters(0,1);
					gerr->Fit("straight","0");
					float slope = straightLine1->GetParameter(1);
					float offset = -(straightLine1->GetParameter(0))/slope;
					float slopeError = straightLine1 -> GetParError(1);
					float offseterr = straightLine1 -> GetParError(0);
					float offsetError = sqrt(pow(offseterr/slope,2)+pow(offset*slopeError/(slope*slope),2));
					fitChi2-> Fill(straightLine1 -> GetChisquare());
					fitOffset -> Fill(offset);
					fitSlope -> Fill(slope);
//					std::cout<< "offset: " << offset << std::endl;
//					std::cout<< "slope: " << slope << std::endl;
//					std::cout<< "temppad: " << temppad << std::endl;

					float timeoffset = 0;
					float deadNoisy = 0;

					outputFile << Form("%f %f %f %f %f %f %f %f \n",tempmodule,temppad,offset,slope, offsetError, slopeError, timeoffset, deadNoisy);
				}

				outputFile.close();




				outfile->Write();
				outfile->Close();



	        //	        gStyle->SetPadRightMargin(0.04);

}
