#include <vector>
#include <iostream>
#include <fstream>
#include "sstream"
#include <utility>
#include <cstdarg>

#include "TLatex.h"
#include "TCanvas.h"
#include "TH2I.h"
#include "TGraphErrors.h"
#include "TFile.h"
#include "TNtuple.h"
#include "TStyle.h"
#include "TROOT.h"
#include "TFrame.h"
#include "TColor.h"
#include "TMath.h"

using namespace std;

/*How to call this:
.L $MARLINTPC/analysis/rootscripts/PulsePerHit.C++
PulsePerHit("path for output","path to the aida file", number of runs in the following list, list of the runs)
e.g PulsePerHit("../myplots","../AidaFiles/",3,18850,18851,18860)
 */
void NoiseHitsAndPulses(char* pathToOutput, char* pathToAida, int nRuns, ...){

	//loading the style
	gROOT->ProcessLine(".L ${MARLINTPC}/analysis/rootscripts/LCTPCStyle.C");
	gROOT->ProcessLine("LCTPCStyle()");

	//open a root file to store the graphs in
	TFile* outfile = new TFile(Form("%s%s",pathToOutput,"NoiseAndPulseCounter.root"),"RECREATE");

	//initialize the input list
	va_list argList;
	va_start(argList,nRuns);

	int runNumber;
	// will contain the results from the processors
	TH2I* noisePulseDistribution = NULL;
	TH2I* notInHitPulseDistribution = NULL;
	TNtuple* zPosTuple = NULL;
	TNtuple* NoiseHitsTuple = NULL;
	TNtuple* NoiseHitsNormTuple = NULL;
	TNtuple* NoisePulsesTuple = NULL;
	TNtuple* NoisePulsesNormTuple = NULL;
	TNtuple* NotInHitPulsesTuple = NULL;
	TNtuple* NotInHitPulsesNormTuple = NULL;
	TNtuple* TrackHitsTuple = NULL;
	TNtuple* TrackHitsNormTuple = NULL;
	TNtuple* TrackPulsesTuple = NULL;
	TNtuple* TrackPulsesNormTuple = NULL;
	TH2I* tmpNoisePulseDist = NULL;
	TH2I* tmpNotInHitPulseDist = NULL;

	vector<double> runZPositions;
	vector<double> runZPositionsError;

	vector<double> runNoiseHitsMean;
	vector<double> runNoiseHitsMeanErr;
	vector<double> runNoiseHitsRMS;
	vector<double> runNoiseHitsRMSErr;

	vector<double> runNoiseHitsNormMean;
	vector<double> runNoiseHitsNormMeanErr;
	vector<double> runNoiseHitsNormRMS;
	vector<double> runNoiseHitsNormRMSErr;

	vector<double> runNoisePulsesMean;
	vector<double> runNoisePulsesMeanErr;
	vector<double> runNoisePulsesRMS;
	vector<double> runNoisePulsesRMSErr;

	vector<double> runNoisePulsesNormMean;
	vector<double> runNoisePulsesNormMeanErr;
	vector<double> runNoisePulsesNormRMS;
	vector<double> runNoisePulsesNormRMSErr;

	vector<double> runNotInHitPulsesMean;
	vector<double> runNotInHitPulsesMeanErr;
	vector<double> runNotInHitPulsesRMS;
	vector<double> runNotInHitPulsesRMSErr;

	vector<double> runNotInHitPulsesNormMean;
	vector<double> runNotInHitPulsesNormMeanErr;
	vector<double> runNotInHitPulsesNormRMS;
	vector<double> runNotInHitPulsesNormRMSErr;

	vector<double> runTrackHitsMean;
	vector<double> runTrackHitsMeanErr;
	vector<double> runTrackHitsRMS;
	vector<double> runTrackHitsRMSErr;

	vector<double> runTrackHitsNormMean;
	vector<double> runTrackHitsNormMeanErr;
	vector<double> runTrackHitsNormRMS;
	vector<double> runTrackHitsNormRMSErr;

	vector<double> runTrackPulsesMean;
	vector<double> runTrackPulsesMeanErr;
	vector<double> runTrackPulsesRMS;
	vector<double> runTrackPulsesRMSErr;

	vector<double> runTrackPulsesNormMean;
	vector<double> runTrackPulsesNormMeanErr;
	vector<double> runTrackPulsesNormRMS;
	vector<double> runTrackPulsesNormRMSErr;

/*	//create colors
 *	const int BINNING=nRuns;
 *	Int_t colors[BINNING];
 *
 *	const Int_t NRGBs = 5;
 *
 *	Double_t stops[NRGBs] = { 0.00, 0.34, 0.61, 0.84, 1.00 };
 *	Double_t red[NRGBs]   = { 0.00, 0.00, 0.87, 1.00, 0.51 };
 *	Double_t green[NRGBs] = { 0.00, 0.81, 1.00, 0.20, 0.00 };
 *	Double_t blue[NRGBs]  = { 0.51, 1.00, 0.12, 0.00, 0.00 };
 *	Int_t FI = TColor::CreateGradientColorTable(NRGBs, stops, red, green, blue, BINNING);
 *
 *	for(int i=0 ; i<BINNING ; i++) 
 *		colors[i] = FI+i;
 */
	//loop over the given runs
	for(int i=0 ; i<nRuns ; i++)
	{
		//get run number from input
		runNumber=va_arg(argList,int);

		//create stringstream containing the aida file name
		stringstream runNumberString;
		cout << "run number: " << runNumber << endl;
		runNumberString << pathToAida << "aida_run" << runNumber << ".root";

		//open the aida file
		TFile* file  = TFile::Open(runNumberString.str().c_str(),"READ");
		//check that the file exists
		if( file==NULL )
		{
			cout << "This run will be skipped. The aida file does not exist. Please check this!" << endl;
			continue;
		}

		//get the data from the file
		zPosTuple = (TNtuple*)file->Get("MyAnalysisCutSelectorProcessor/RunZPositions");
		NoiseHitsTuple = (TNtuple*)file->Get("MyNoiseHitAndPulseCounterProcessor/NoiseHitsTuple");
		NoiseHitsNormTuple = (TNtuple*)file->Get("MyNoiseHitAndPulseCounterProcessor/NoiseHitsNormTuple");
		NoisePulsesTuple = (TNtuple*)file->Get("MyNoiseHitAndPulseCounterProcessor/NoisePulsesTuple");
		NoisePulsesNormTuple = (TNtuple*)file->Get("MyNoiseHitAndPulseCounterProcessor/NoisePulsesNormTuple");
		NotInHitPulsesTuple = (TNtuple*)file->Get("MyNoiseHitAndPulseCounterProcessor/NotInHitPulsesTuple");
		NotInHitPulsesNormTuple = (TNtuple*)file->Get("MyNoiseHitAndPulseCounterProcessor/NotInHitPulsesNormTuple");
		TrackHitsTuple = (TNtuple*)file->Get("MyNoiseHitAndPulseCounterProcessor/TrackHitsTuple");
		TrackHitsNormTuple = (TNtuple*)file->Get("MyNoiseHitAndPulseCounterProcessor/TrackHitsNormTuple");
		TrackPulsesTuple = (TNtuple*)file->Get("MyNoiseHitAndPulseCounterProcessor/TrackPulsesTuple");
		TrackPulsesNormTuple = (TNtuple*)file->Get("MyNoiseHitAndPulseCounterProcessor/TrackPulsesNormTuple");

		tmpNoisePulseDist = (TH2I*)file->Get("MyNoiseHitAndPulseCounterProcessor/NoisePulsesSpatialDistribution");
		tmpNotInHitPulseDist = (TH2I*)file->Get("MyNoiseHitAndPulseCounterProcessor/NotInHitPulsesSpatialDistribution");

		if( zPosTuple==NULL || NoiseHitsTuple==NULL || NoiseHitsNormTuple==NULL || NoisePulsesTuple==NULL || NoisePulsesNormTuple==NULL || NotInHitPulsesTuple==NULL || NotInHitPulsesNormTuple==NULL || TrackHitsTuple==NULL || TrackHitsNormTuple==NULL || TrackPulsesTuple==NULL || TrackPulsesNormTuple==NULL )
		{
			cout << "This run will be skipped. One of the necessary root tuples does not exist. Please check that you ran the corresponding processors and that the tuple is in the aida file!" << endl;
			continue;
		}
		if( tmpNoisePulseDist==NULL || tmpNotInHitPulseDist==NULL )
		{
			cout << "This run will be skipped. One of the necessary root histograms does not exist. Please check that you ran the corresponding processors and that the graph is in the aida file!" << endl;
			continue;
		}

		//open the first entry of the tuples
		zPosTuple->GetEntry(0);
		NoiseHitsTuple->GetEntry(0);
		NoiseHitsNormTuple->GetEntry(0);
		NoisePulsesTuple->GetEntry(0);
		NoisePulsesNormTuple->GetEntry(0);
		NotInHitPulsesTuple->GetEntry(0);
		NotInHitPulsesNormTuple->GetEntry(0);
		TrackHitsTuple->GetEntry(0);
		TrackHitsNormTuple->GetEntry(0);
		TrackPulsesTuple->GetEntry(0);
		TrackPulsesNormTuple->GetEntry(0);

		//567.6-hitposition is the conversion into drift distance
		runZPositions.push_back(567.6-zPosTuple->GetArgs()[0]);
		runZPositionsError.push_back(zPosTuple->GetArgs()[2]);

		runNoiseHitsMean.push_back(NoiseHitsTuple->GetArgs()[0]);
		runNoiseHitsMeanErr.push_back(NoiseHitsTuple->GetArgs()[1]);
		runNoiseHitsRMS.push_back(NoiseHitsTuple->GetArgs()[2]);
		runNoiseHitsRMSErr.push_back(NoiseHitsTuple->GetArgs()[3]);

		runNoiseHitsNormMean.push_back(NoiseHitsNormTuple->GetArgs()[0]);
		runNoiseHitsNormMeanErr.push_back(NoiseHitsNormTuple->GetArgs()[1]);
		runNoiseHitsNormRMS.push_back(NoiseHitsNormTuple->GetArgs()[2]);
		runNoiseHitsNormRMSErr.push_back(NoiseHitsNormTuple->GetArgs()[3]);

		runNoisePulsesMean.push_back(NoisePulsesTuple->GetArgs()[0]);
		runNoisePulsesMeanErr.push_back(NoisePulsesTuple->GetArgs()[1]);
		runNoisePulsesRMS.push_back(NoisePulsesTuple->GetArgs()[2]);
		runNoisePulsesRMSErr.push_back(NoisePulsesTuple->GetArgs()[3]);

		runNoisePulsesNormMean.push_back(NoisePulsesNormTuple->GetArgs()[0]);
		runNoisePulsesNormMeanErr.push_back(NoisePulsesNormTuple->GetArgs()[1]);
		runNoisePulsesNormRMS.push_back(NoisePulsesNormTuple->GetArgs()[2]);
		runNoisePulsesNormRMSErr.push_back(NoisePulsesNormTuple->GetArgs()[3]);

		runNotInHitPulsesMean.push_back(NotInHitPulsesTuple->GetArgs()[0]);
		runNotInHitPulsesMeanErr.push_back(NotInHitPulsesTuple->GetArgs()[1]);
		runNotInHitPulsesRMS.push_back(NotInHitPulsesTuple->GetArgs()[2]);
		runNotInHitPulsesRMSErr.push_back(NotInHitPulsesTuple->GetArgs()[3]);

		runNotInHitPulsesNormMean.push_back(NotInHitPulsesNormTuple->GetArgs()[0]);
		runNotInHitPulsesNormMeanErr.push_back(NotInHitPulsesNormTuple->GetArgs()[1]);
		runNotInHitPulsesNormRMS.push_back(NotInHitPulsesNormTuple->GetArgs()[2]);
		runNotInHitPulsesNormRMSErr.push_back(NotInHitPulsesNormTuple->GetArgs()[3]);

		runTrackHitsMean.push_back(TrackHitsTuple->GetArgs()[0]);
		runTrackHitsMeanErr.push_back(TrackHitsTuple->GetArgs()[1]);
		runTrackHitsRMS.push_back(TrackHitsTuple->GetArgs()[2]);
		runTrackHitsRMSErr.push_back(TrackHitsTuple->GetArgs()[3]);

		runTrackHitsNormMean.push_back(TrackHitsNormTuple->GetArgs()[0]);
		runTrackHitsNormMeanErr.push_back(TrackHitsNormTuple->GetArgs()[1]);
		runTrackHitsNormRMS.push_back(TrackHitsNormTuple->GetArgs()[2]);
		runTrackHitsNormRMSErr.push_back(TrackHitsNormTuple->GetArgs()[3]);

		runTrackPulsesMean.push_back(TrackPulsesTuple->GetArgs()[0]);
		runTrackPulsesMeanErr.push_back(TrackPulsesTuple->GetArgs()[1]);
		runTrackPulsesRMS.push_back(TrackPulsesTuple->GetArgs()[2]);
		runTrackPulsesRMSErr.push_back(TrackPulsesTuple->GetArgs()[3]);

		runTrackPulsesNormMean.push_back(TrackPulsesNormTuple->GetArgs()[0]);
		runTrackPulsesNormMeanErr.push_back(TrackPulsesNormTuple->GetArgs()[1]);
		runTrackPulsesNormRMS.push_back(TrackPulsesNormTuple->GetArgs()[2]);
		runTrackPulsesNormRMSErr.push_back(TrackPulsesNormTuple->GetArgs()[3]);

		//cd out of the file to prevent root from deleting the following, when closing the file
		gROOT->cd();

		//add up the histograms
		if(noisePulseDistribution == NULL){ noisePulseDistribution = (TH2I*)tmpNoisePulseDist->Clone(); }
		else{ noisePulseDistribution->Add(tmpNoisePulseDist); }

		if(notInHitPulseDistribution == NULL){ notInHitPulseDistribution = (TH2I*)tmpNotInHitPulseDist->Clone(); }
		else{ notInHitPulseDistribution->Add(tmpNotInHitPulseDist); }

		file->Close();
		delete file;
	}

	va_end(argList);

	//save the results
	outfile->cd();

	gStyle->SetPadRightMargin(0.18);
	gStyle->SetOptStat(0);

	TCanvas* c0 = new TCanvas("NoisePulsesDistribution","Noise Pulse Distribution");
	noisePulseDistribution->GetZaxis()->SetTitleOffset(1.5);
	noisePulseDistribution->SetContour(50);
	noisePulseDistribution->Draw("COLZ");
	c0->SetLogz();
	c0->SaveAs(Form("%s%s",pathToOutput,"NoisePulseDistribution.png"));
	c0->SaveAs(Form("%s%s",pathToOutput,"NoisePulseDistribution.pdf"));
	noisePulseDistribution->Write();

	TCanvas* c1 = new TCanvas("NotInHitPulsesDistribution","Not in Hit Pulse Distribution");
	notInHitPulseDistribution->GetZaxis()->SetTitleOffset(1.5);
	notInHitPulseDistribution->SetContour(50);
	notInHitPulseDistribution->Draw("COLZ");
	c1->SetLogz();
	c1->SaveAs(Form("%s%s",pathToOutput,"NotInHitPulseDistribution.png"));
	c1->SaveAs(Form("%s%s",pathToOutput,"NotInHitPulseDistribution.pdf"));
	notInHitPulseDistribution->Write();

	gStyle->SetPadRightMargin(0.04);

	TCanvas* c2 = new TCanvas("NoiseHits","Noise Hits");
	TGraphErrors* runNoiseHitsMeanGraph = new TGraphErrors( runZPositions.size(), &runZPositions[0], &runNoiseHitsMean[0], &runZPositionsError[0], &runNoiseHitsMeanErr[0] );
	runNoiseHitsMeanGraph->SetNameTitle("NoiseHitsMeanGraph","Noise Hits Mean");
	runNoiseHitsMeanGraph->GetXaxis()->SetTitle("drift distance [mm]");
	runNoiseHitsMeanGraph->GetYaxis()->SetTitle("Mean #Noise Hits");
	runNoiseHitsMeanGraph->Draw("AP");
	c2->Update();
	c2->SaveAs(Form("%s%s",pathToOutput,"NoiseHits.png"));
	c2->SaveAs(Form("%s%s",pathToOutput,"NoiseHits.pdf"));
	TCanvas* c2_2 = new TCanvas("NoiseHits_RMS","Noise Hits RMS");
	TGraphErrors* runNoiseHitsRMSGraph = new TGraphErrors( runZPositions.size(), &runZPositions[0], &runNoiseHitsRMS[0], &runZPositionsError[0], &runNoiseHitsRMSErr[0] );
	runNoiseHitsRMSGraph->SetNameTitle("NoiseHitsRMSGraph","Noise Hits RMS");
	runNoiseHitsRMSGraph->GetXaxis()->SetTitle("drift distance [mm]");
	runNoiseHitsRMSGraph->GetYaxis()->SetTitle("RMS #Noise Hits");
	runNoiseHitsRMSGraph->Draw("AP");
	c2_2->Update();
	c2_2->SaveAs(Form("%s%s",pathToOutput,"NoiseHitsRMS.png"));
	c2_2->SaveAs(Form("%s%s",pathToOutput,"NoiseHitsRMS.pdf"));
	runNoiseHitsMeanGraph->Write();
	runNoiseHitsRMSGraph->Write();

	TCanvas* c3 = new TCanvas("NoiseHitsNorm","Noise HitsNorm");
	TGraphErrors* runNoiseHitsNormMeanGraph = new TGraphErrors( runZPositions.size(), &runZPositions[0], &runNoiseHitsNormMean[0], &runZPositionsError[0], &runNoiseHitsNormMeanErr[0] );
	runNoiseHitsNormMeanGraph->SetNameTitle("NoiseHitsNormMeanGraph","Noise HitsNorm Mean");
	runNoiseHitsNormMeanGraph->GetXaxis()->SetTitle("drift distance [mm]");
	runNoiseHitsNormMeanGraph->GetYaxis()->SetTitle("Mean #Noise Hits/#Hits");
	runNoiseHitsNormMeanGraph->Draw("AP");
	c3->Update();
	c3->SaveAs(Form("%s%s",pathToOutput,"NoiseHitsNorm.png"));
	c3->SaveAs(Form("%s%s",pathToOutput,"NoiseHitsNorm.pdf"));
	TCanvas* c3_2 = new TCanvas("NoiseHitsNorm_RMS","Noise HitsNorm RMS");
	TGraphErrors* runNoiseHitsNormRMSGraph = new TGraphErrors( runZPositions.size(), &runZPositions[0], &runNoiseHitsNormRMS[0], &runZPositionsError[0], &runNoiseHitsNormRMSErr[0] );
	runNoiseHitsNormRMSGraph->SetNameTitle("NoiseHitsNormRMSGraph","Noise HitsNorm RMS");
	runNoiseHitsNormRMSGraph->GetXaxis()->SetTitle("drift distance [mm]");
	runNoiseHitsNormRMSGraph->GetYaxis()->SetTitle("RMS #Noise Hits/#Hits");
	runNoiseHitsNormRMSGraph->Draw("AP");
	c3_2->Update();
	c3_2->SaveAs(Form("%s%s",pathToOutput,"NoiseHitsNormRMS.png"));
	c3_2->SaveAs(Form("%s%s",pathToOutput,"NoiseHitsNormRMS.pdf"));
	runNoiseHitsNormMeanGraph->Write();
	runNoiseHitsNormRMSGraph->Write();

	TCanvas* c4 = new TCanvas("NoisePulses","Noise Pulses");
	TGraphErrors* runNoisePulsesMeanGraph = new TGraphErrors( runZPositions.size(), &runZPositions[0], &runNoisePulsesMean[0], &runZPositionsError[0], &runNoisePulsesMeanErr[0] );
	runNoisePulsesMeanGraph->SetNameTitle("NoisePulsesMeanGraph","Noise Pulses Mean");
	runNoisePulsesMeanGraph->GetXaxis()->SetTitle("drift distance [mm]");
	runNoisePulsesMeanGraph->GetYaxis()->SetTitle("Mean #Noise Pulses");
	runNoisePulsesMeanGraph->Draw("AP");
	c4->Update();
	c4->SaveAs(Form("%s%s",pathToOutput,"NoisePulses.png"));
	c4->SaveAs(Form("%s%s",pathToOutput,"NoisePulses.pdf"));
	TCanvas* c4_2 = new TCanvas("NoisePulses_RMS","Noise Pulses RMS");
	TGraphErrors* runNoisePulsesRMSGraph = new TGraphErrors( runZPositions.size(), &runZPositions[0], &runNoisePulsesRMS[0], &runZPositionsError[0], &runNoisePulsesRMSErr[0] );
	runNoisePulsesRMSGraph->SetNameTitle("NoisePulsesRMSGraph","Noise Pulses RMS");
	runNoisePulsesRMSGraph->GetXaxis()->SetTitle("drift distance [mm]");
	runNoisePulsesRMSGraph->GetYaxis()->SetTitle("RMS #Noise Pulses");
	runNoisePulsesRMSGraph->Draw("AP");
	c4_2->Update();
	c4_2->SaveAs(Form("%s%s",pathToOutput,"NoisePulsesRMS.png"));
	c4_2->SaveAs(Form("%s%s",pathToOutput,"NoisePulsesRMS.pdf"));
	runNoisePulsesMeanGraph->Write();
	runNoisePulsesRMSGraph->Write();

	TCanvas* c5 = new TCanvas("NoisePulsesNorm","Noise PulsesNorm");
	TGraphErrors* runNoisePulsesNormMeanGraph = new TGraphErrors( runZPositions.size(), &runZPositions[0], &runNoisePulsesNormMean[0], &runZPositionsError[0], &runNoisePulsesNormMeanErr[0] );
	runNoisePulsesNormMeanGraph->SetNameTitle("NoisePulsesNormMeanGraph","Noise PulsesNorm Mean");
	runNoisePulsesNormMeanGraph->GetXaxis()->SetTitle("drift distance [mm]");
	runNoisePulsesNormMeanGraph->GetYaxis()->SetTitle("Mean #Noise Pulses/#Pulses");
	runNoisePulsesNormMeanGraph->Draw("AP");
	c5->Update();
	c5->SaveAs(Form("%s%s",pathToOutput,"NoisePulsesNorm.png"));
	c5->SaveAs(Form("%s%s",pathToOutput,"NoisePulsesNorm.pdf"));
	TCanvas* c5_2 = new TCanvas("NoisePulsesNorm_RMS","Noise PulsesNorm RMS");
	TGraphErrors* runNoisePulsesNormRMSGraph = new TGraphErrors( runZPositions.size(), &runZPositions[0], &runNoisePulsesNormRMS[0], &runZPositionsError[0], &runNoisePulsesNormRMSErr[0] );
	runNoisePulsesNormRMSGraph->SetNameTitle("NoisePulsesNormRMSGraph","Noise PulsesNorm RMS");
	runNoisePulsesNormRMSGraph->GetXaxis()->SetTitle("drift distance [mm]");
	runNoisePulsesNormRMSGraph->GetYaxis()->SetTitle("RMS #Noise Pulses/#Pulses");
	runNoisePulsesNormRMSGraph->Draw("AP");
	c5_2->Update();
	c5_2->SaveAs(Form("%s%s",pathToOutput,"NoisePulsesNormRMS.png"));
	c5_2->SaveAs(Form("%s%s",pathToOutput,"NoisePulsesNormRMS.pdf"));
	runNoisePulsesNormMeanGraph->Write();
	runNoisePulsesNormRMSGraph->Write();

	TCanvas* c6 = new TCanvas("NotInHitPulses","Pulses not in Hits");
	TGraphErrors* runNotInHitPulsesMeanGraph = new TGraphErrors( runZPositions.size(), &runZPositions[0], &runNotInHitPulsesMean[0], &runZPositionsError[0], &runNotInHitPulsesMeanErr[0] );
	runNotInHitPulsesMeanGraph->SetNameTitle("NotInHitPulsesMeanGraph","Pulses not in Hits Mean");
	runNotInHitPulsesMeanGraph->GetXaxis()->SetTitle("drift distance [mm]");
	runNotInHitPulsesMeanGraph->GetYaxis()->SetTitle("Mean #Pulses w/o Hit");
	runNotInHitPulsesMeanGraph->Draw("AP");
	c6->Update();
	c6->SaveAs(Form("%s%s",pathToOutput,"NotInHitPulses.png"));
	c6->SaveAs(Form("%s%s",pathToOutput,"NotInHitPulses.pdf"));
	TCanvas* c6_2 = new TCanvas("NotInHitPulses_RMS","Pulses not in Hits RMS");
	TGraphErrors* runNotInHitPulsesRMSGraph = new TGraphErrors( runZPositions.size(), &runZPositions[0], &runNotInHitPulsesRMS[0], &runZPositionsError[0], &runNotInHitPulsesRMSErr[0] );
	runNotInHitPulsesRMSGraph->SetNameTitle("NotInHitPulsesRMSGraph","Pulses not in Hits RMS");
	runNotInHitPulsesRMSGraph->GetXaxis()->SetTitle("drift distance [mm]");
	runNotInHitPulsesRMSGraph->GetYaxis()->SetTitle("RMS #Pulses w/o Hit");
	runNotInHitPulsesRMSGraph->Draw("AP");
	c6_2->Update();
	c6_2->SaveAs(Form("%s%s",pathToOutput,"NotInHitPulsesRMS.png"));
	c6_2->SaveAs(Form("%s%s",pathToOutput,"NotInHitPulsesRMS.pdf"));
	runNotInHitPulsesMeanGraph->Write();
	runNotInHitPulsesRMSGraph->Write();

	TCanvas* c7 = new TCanvas("NotInHitPulsesNorm","PulsesNorm not in Hits");
	TGraphErrors* runNotInHitPulsesNormMeanGraph = new TGraphErrors( runZPositions.size(), &runZPositions[0], &runNotInHitPulsesNormMean[0], &runZPositionsError[0], &runNotInHitPulsesNormMeanErr[0] );
	runNotInHitPulsesNormMeanGraph->SetNameTitle("NotInHitPulsesNormMeanGraph","PulsesNorm not in Hits Mean");
	runNotInHitPulsesNormMeanGraph->GetXaxis()->SetTitle("drift distance [mm]");
	runNotInHitPulsesNormMeanGraph->GetYaxis()->SetTitle("Mean #Pulses w/o Hit/#Pulses\t");
	runNotInHitPulsesNormMeanGraph->Draw("AP");
	c7->Update();
	c7->SaveAs(Form("%s%s",pathToOutput,"NotInHitPulsesNorm.png"));
	c7->SaveAs(Form("%s%s",pathToOutput,"NotInHitPulsesNorm.pdf"));
	TCanvas* c7_2 = new TCanvas("NotInHitPulsesNorm_RMS","PulsesNorm not in Hits RMS");
	TGraphErrors* runNotInHitPulsesNormRMSGraph = new TGraphErrors( runZPositions.size(), &runZPositions[0], &runNotInHitPulsesNormRMS[0], &runZPositionsError[0], &runNotInHitPulsesNormRMSErr[0] );
	runNotInHitPulsesNormRMSGraph->SetNameTitle("NotInHitPulsesNormRMSGraph","PulsesNorm not in Hits RMS");
	runNotInHitPulsesNormRMSGraph->GetXaxis()->SetTitle("drift distance [mm]");
	runNotInHitPulsesNormRMSGraph->GetYaxis()->SetTitle("RMS #Pulses w/o Hit/#Pulses\t");
	runNotInHitPulsesNormRMSGraph->Draw("AP");
	c7_2->Update();
	c7_2->SaveAs(Form("%s%s",pathToOutput,"NotInHitPulsesNormRMS.png"));
	c7_2->SaveAs(Form("%s%s",pathToOutput,"NotInHitPulsesNormRMS.pdf"));
	runNotInHitPulsesNormMeanGraph->Write();
	runNotInHitPulsesNormRMSGraph->Write();

	TCanvas* c8 = new TCanvas("TrackHits","Track Hits");
	TGraphErrors* runTrackHitsMeanGraph = new TGraphErrors( runZPositions.size(), &runZPositions[0], &runTrackHitsMean[0], &runZPositionsError[0], &runTrackHitsMeanErr[0] );
	runTrackHitsMeanGraph->SetNameTitle("TrackHitsMeanGraph","Track Hits Mean");
	runTrackHitsMeanGraph->GetXaxis()->SetTitle("drift distance [mm]");
	runTrackHitsMeanGraph->GetYaxis()->SetTitle("Mean #Track Hits");
	runTrackHitsMeanGraph->Draw("AP");
	c8->Update();
	c8->SaveAs(Form("%s%s",pathToOutput,"TrackHits.png"));
	c8->SaveAs(Form("%s%s",pathToOutput,"TrackHits.pdf"));
	TCanvas* c8_2 = new TCanvas("TrackHits RMS","Track Hits RMS");
	TGraphErrors* runTrackHitsRMSGraph = new TGraphErrors( runZPositions.size(), &runZPositions[0], &runTrackHitsRMS[0], &runZPositionsError[0], &runTrackHitsRMSErr[0] );
	runTrackHitsRMSGraph->SetNameTitle("TrackHitsRMSGraph","Track Hits RMS");
	runTrackHitsRMSGraph->GetXaxis()->SetTitle("drift distance [mm]");
	runTrackHitsRMSGraph->GetYaxis()->SetTitle("RMS #Track Hits");
	runTrackHitsRMSGraph->Draw("AP");
	c8_2->Update();
	c8_2->SaveAs(Form("%s%s",pathToOutput,"TrackHitsRMS.png"));
	c8_2->SaveAs(Form("%s%s",pathToOutput,"TrackHitsRMS.pdf"));
	runTrackHitsMeanGraph->Write();
	runTrackHitsRMSGraph->Write();

	TCanvas* c9 = new TCanvas("TrackHitsNorm","Track HitsNorm");
	TGraphErrors* runTrackHitsNormMeanGraph = new TGraphErrors( runZPositions.size(), &runZPositions[0], &runTrackHitsNormMean[0], &runZPositionsError[0], &runTrackHitsNormMeanErr[0] );
	runTrackHitsNormMeanGraph->SetNameTitle("TrackHitsNormMeanGraph","Track HitsNorm Mean");
	runTrackHitsNormMeanGraph->GetXaxis()->SetTitle("drift distance [mm]");
	runTrackHitsNormMeanGraph->GetYaxis()->SetTitle("Mean #Track Hits/#Hits");
	runTrackHitsNormMeanGraph->Draw("AP");
	c9->Update();
	c9->SaveAs(Form("%s%s",pathToOutput,"TrackHitsNorm.png"));
	c9->SaveAs(Form("%s%s",pathToOutput,"TrackHitsNorm.pdf"));
	TCanvas* c9_2 = new TCanvas("TrackHitsNorm_RMS","Track HitsNorm RMS");
	TGraphErrors* runTrackHitsNormRMSGraph = new TGraphErrors( runZPositions.size(), &runZPositions[0], &runTrackHitsNormRMS[0], &runZPositionsError[0], &runTrackHitsNormRMSErr[0] );
	runTrackHitsNormRMSGraph->SetNameTitle("TrackHitsNormRMSGraph","Track HitsNorm RMS");
	runTrackHitsNormRMSGraph->GetXaxis()->SetTitle("drift distance [mm]");
	runTrackHitsNormRMSGraph->GetYaxis()->SetTitle("RMS #Track Hits/#Hits");
	runTrackHitsNormRMSGraph->Draw("AP");
	c9_2->Update();
	c9_2->SaveAs(Form("%s%s",pathToOutput,"TrackHitsNormRMS.png"));
	c9_2->SaveAs(Form("%s%s",pathToOutput,"TrackHitsNormRMS.pdf"));
	runTrackHitsNormMeanGraph->Write();
	runTrackHitsNormRMSGraph->Write();

	TCanvas* c10 = new TCanvas("TrackPulses","Track Pulses");
	TGraphErrors* runTrackPulsesMeanGraph = new TGraphErrors( runZPositions.size(), &runZPositions[0], &runTrackPulsesMean[0], &runZPositionsError[0], &runTrackPulsesMeanErr[0] );
	runTrackPulsesMeanGraph->SetNameTitle("TrackPulsesMeanGraph","Track Pulses Mean");
	runTrackPulsesMeanGraph->GetXaxis()->SetTitle("drift distance [mm]");
	runTrackPulsesMeanGraph->GetYaxis()->SetTitle("Mean #Track Pulses");
	runTrackPulsesMeanGraph->Draw("AP");
	c10->Update();
	c10->SaveAs(Form("%s%s",pathToOutput,"TrackPulses.png"));
	c10->SaveAs(Form("%s%s",pathToOutput,"TrackPulses.pdf"));
	TCanvas* c10_2 = new TCanvas("TrackPulses_RMS","Track Pulses RMS");
	TGraphErrors* runTrackPulsesRMSGraph = new TGraphErrors( runZPositions.size(), &runZPositions[0], &runTrackPulsesRMS[0], &runZPositionsError[0], &runTrackPulsesRMSErr[0] );
	runTrackPulsesRMSGraph->SetNameTitle("TrackPulsesRMSGraph","Track Pulses RMS");
	runTrackPulsesRMSGraph->GetXaxis()->SetTitle("drift distance [mm]");
	runTrackPulsesRMSGraph->GetYaxis()->SetTitle("RMS #Track Pulses");
	runTrackPulsesRMSGraph->Draw("AP");
	c10_2->Update();
	c10_2->SaveAs(Form("%s%s",pathToOutput,"TrackPulsesRMS.png"));
	c10_2->SaveAs(Form("%s%s",pathToOutput,"TrackPulsesRMS.pdf"));
	runTrackPulsesMeanGraph->Write();
	runTrackPulsesRMSGraph->Write();

	TCanvas* c11 = new TCanvas("TrackPulsesNorm","Track PulsesNorm");
	TGraphErrors* runTrackPulsesNormMeanGraph = new TGraphErrors( runZPositions.size(), &runZPositions[0], &runTrackPulsesNormMean[0], &runZPositionsError[0], &runTrackPulsesNormMeanErr[0] );
	runTrackPulsesNormMeanGraph->SetNameTitle("TrackPulsesNormMeanGraph","Track PulsesNorm Mean");
	runTrackPulsesNormMeanGraph->GetXaxis()->SetTitle("drift distance [mm]");
	runTrackPulsesNormMeanGraph->GetYaxis()->SetTitle("Mean #Track Pulses/#Pulses");
	runTrackPulsesNormMeanGraph->Draw("AP");
	c11->Update();
	c11->SaveAs(Form("%s%s",pathToOutput,"TrackPulsesNorm.png"));
	c11->SaveAs(Form("%s%s",pathToOutput,"TrackPulsesNorm.pdf"));
	TCanvas* c11_2 = new TCanvas("TrackPulsesNorm_RMS","Track PulsesNorm RMS");
	TGraphErrors* runTrackPulsesNormRMSGraph = new TGraphErrors( runZPositions.size(), &runZPositions[0], &runTrackPulsesNormRMS[0], &runZPositionsError[0], &runTrackPulsesNormRMSErr[0] );
	runTrackPulsesNormRMSGraph->SetNameTitle("TrackPulsesNormRMSGraph","Track PulsesNorm RMS");
	runTrackPulsesNormRMSGraph->GetXaxis()->SetTitle("drift distance [mm]");
	runTrackPulsesNormRMSGraph->GetYaxis()->SetTitle("RMS #Track Pulses/#Pulses");
	runTrackPulsesNormRMSGraph->Draw("AP");
	c11_2->Update();
	c11_2->SaveAs(Form("%s%s",pathToOutput,"TrackPulsesNormRMS.png"));
	c11_2->SaveAs(Form("%s%s",pathToOutput,"TrackPulsesNormRMS.pdf"));
	runTrackPulsesNormMeanGraph->Write();
	runTrackPulsesNormRMSGraph->Write();

	outfile->cd();

	c1->Write(); //Saving these to save plot details for future use.  --Peter
	c2->Write();
	c2_2->Write();
	c3->Write();
	c3_2->Write();
	c4->Write();
	c4_2->Write();
	c5->Write();
	c5_2->Write();
	c6->Write();
	c6_2->Write();
	c7->Write();
	c7_2->Write();
	c8->Write();
	c8_2->Write();
	c9->Write();
	c9_2->Write();
	c10->Write();
	c10_2->Write();
	c11->Write();
	c11_2->Write();

	outfile->Flush();
	outfile->Write();
	outfile->Close();

//	delete c1;
//	delete c2;
//	delete c3;
//	delete c4;
//	delete c5;
//	delete c6;
//	delete c7;
//	delete c8;
//	delete c9;
//	delete c10;
//	delete c11;
}
