#include <vector>
#include <iostream>
#include "TLatex.h"
#include "TCanvas.h"
#include "TH1.h"
#include "TGraphErrors.h"
#include "TMultiGraph.h"
#include "TFile.h"
#include "TNtuple.h"
#include "TStyle.h"
#include "TROOT.h"
#include "TFrame.h"
#include "sstream"
#include <utility>
#include <cstdarg>
#include "TColor.h"
#include "TMath.h"


/*How to call this:
.L $MARLINTPC/analysis/rootscripts/PRFPerformance.C++
PRFPerformance("path for output","path to the aida file", number of runs in the following list, list of the runs)
e.g PRFPerformance("../myplots","../AidaFiles/",3,18850,18851,18860)
 */
void PRFPerformance(char * pathToOutput,char * pathToAida, int nRuns, ...){

	//loading the style
	gROOT->ProcessLine(".L ${MARLINTPC}/analysis/rootscripts/LCTPCStyle.C");
	gROOT->ProcessLine("LCTPCStyle()");

	//open a root file to store the graphs in
	TFile *outfile=new TFile(Form("%s%s",pathToOutput,"PRFPerformance.root"),"RECREATE");

	//initialize the input list
	va_list argList;
	va_start(argList,nRuns);

	int runNumber;
	int nRows=0;
	// will contain the results from the processors
	TMultiGraph *runPRFGraph=new TMultiGraph();
	TMultiGraph *runPRFPadCorGraph=new TMultiGraph();
	//	runResZGraph->SetTitle("z Resolution;r_{row} [mm];#sigma_{z} [mm]");
	//	runResRPhiGraph->SetTitle("r phi Resolution;r_{row} [mm];#sigma_{PRF} [mm]");
	runPRFGraph->SetTitle("PRF width;r_{row} [mm];#sigma_{prf} [mm]");
	runPRFPadCorGraph->SetTitle("PRF width pad corrected;r_{row} [mm];#sigma_{prf} [mm]");

	std::vector<double> runZPositions;
	std::vector<double> runZPositionsError;
	std::vector<double> runZPositionsSingleRow;
	std::vector<double> runZPositionsSingleRowError;

	std::vector<double> runSinglePRF;
	std::vector<double> runSinglePRFError;
	std::vector<double> runBestPRF;
	std::vector<double> runBestPRFError;
	std::vector<double> runAllPRF;
	std::vector<double> runAllPRFError;
	std::vector<double> runSinglePRFMean;
	std::vector<double> runSinglePRFMeanError;

	std::vector<double> runSinglePRFPadCor;
	std::vector<double> runSinglePRFErrorPadCor;
	std::vector<double> runBestPRFPadCor;
	std::vector<double> runBestPRFErrorPadCor;
	std::vector<double> runSinglePRFMeanPadCor;
	std::vector<double> runSinglePRFMeanErrorPadCor;


	std::pair<int,int> bestRow;

	//create colors
	const int BINNING=nRuns;
	Int_t colors[BINNING];

	const Int_t NRGBs = 5;

	Double_t stops[NRGBs] = { 0.00, 0.34, 0.61, 0.84, 1.00 };
	Double_t red[NRGBs]   = { 0.00, 0.00, 0.87, 1.00, 0.51 };
	Double_t green[NRGBs] = { 0.00, 0.81, 1.00, 0.20, 0.00 };
	Double_t blue[NRGBs]  = { 0.51, 1.00, 0.12, 0.00, 0.00 };
	Int_t FI = TColor::CreateGradientColorTable(NRGBs, stops, red, green, blue, BINNING);

	for (int i=0;i<BINNING;i++)
		colors[i] = FI+i;

	//loop over the given runs
	for (int i=0;i<nRuns;i++)
	{
		//get run number from input
		runNumber=va_arg(argList,int);

		//create stringstream containing the aida file name
		std::stringstream runNumberString;
		std::cout<<"run number: "<<runNumber<<std::endl;
		runNumberString <<pathToAida<<"aida_run" <<runNumber<< ".root";

		//open the aida file
		TFile *file  = TFile::Open(runNumberString.str().c_str(),"Read");
		//check that the file exists
		if (file==0)
		{
			std::cout<<"PRFPerformance: This run will be skipped. The aida file does not exist. Please check this!"<<std::endl;
			continue;
		}

		//get the tuples from the file
		TNtuple * zPosTuple=(TNtuple*)file->Get("MyAnalysisCutSelectorProcessor/RunZPositions");
		TNtuple * allRowTuple=(TNtuple*)file->Get("MyPRFPerformanceProcessor/allRowsPRF");
		TNtuple * bestRowTuple=(TNtuple*)file->Get("MyPRFPerformanceProcessor/bestRowPRF");
		TNtuple * singleRowTuple=(TNtuple*)file->Get("MyPRFPerformanceProcessor/singleRowsPRF");
		TNtuple * bestRowPadCorTuple=(TNtuple*)file->Get("MyPRFPerformanceProcessor/bestRowPadCorPRF");
		TNtuple * singleRowPadCorTuple=(TNtuple*)file->Get("MyPRFPerformanceProcessor/singleRowsPadCorPRF");

		if (zPosTuple==0 || allRowTuple==0 || bestRowTuple==0 || singleRowTuple==0 || bestRowPadCorTuple==0 || singleRowPadCorTuple==0 )
		{
			std::cout<<"PRFPerformance: This run will be skipped. One of the necessary root tuples does not exist. Please check that you ran the corresponding processors and that the tuple is in the aida file!"<<std::endl;
			continue;
		}
		//open the first and only entry of the tuples
		zPosTuple->GetEntry(0);
		allRowTuple->GetEntry(0);
		bestRowTuple->GetEntry(0);
		bestRowPadCorTuple->GetEntry(0);


		runZPositions.push_back(567.6-zPosTuple->GetArgs()[0]);
		runZPositionsError.push_back(zPosTuple->GetArgs()[2]);
		runBestPRF.push_back(bestRowTuple->GetArgs()[2]);
		runBestPRFError.push_back(bestRowTuple->GetArgs()[3]);
		runAllPRF.push_back(allRowTuple->GetArgs()[0]);
		runAllPRFError.push_back(allRowTuple->GetArgs()[1]);
		runBestPRFPadCor.push_back(bestRowPadCorTuple->GetArgs()[2]);
		runBestPRFErrorPadCor.push_back(bestRowPadCorTuple->GetArgs()[3]);

		bestRow = std::make_pair(static_cast<int>(bestRowTuple->GetArgs()[0]),static_cast<int>(bestRowTuple->GetArgs()[1]));

		std::vector<double> xy,wxy;
		double sumwxy=0;

		nRows = singleRowTuple->GetEntries();
		for (int RowIter=0;RowIter<nRows;RowIter++)
		{
			runZPositionsSingleRow.push_back(567.6-zPosTuple->GetArgs()[0]);
			runZPositionsSingleRowError.push_back(zPosTuple->GetArgs()[2]);
			singleRowTuple->GetEntry(RowIter);
			runSinglePRF.push_back(singleRowTuple->GetArgs()[2]);
			runSinglePRFError.push_back(singleRowTuple->GetArgs()[3]);

			wxy.push_back(1/(singleRowTuple->GetArgs()[3]*singleRowTuple->GetArgs()[3]));
			xy.push_back(singleRowTuple->GetArgs()[2]);

			sumwxy+=wxy[RowIter];
		}


		std::vector<double> xyPadCor,wxyPadCor;
		double sumwxyPadCor=0;

		nRows = singleRowPadCorTuple->GetEntries();
		for (int RowIter=0;RowIter<nRows;RowIter++)
		{
			runZPositionsSingleRow.push_back(567.6-zPosTuple->GetArgs()[0]);
			runZPositionsSingleRowError.push_back(zPosTuple->GetArgs()[2]);
			singleRowPadCorTuple->GetEntry(RowIter);
			runSinglePRFPadCor.push_back(singleRowPadCorTuple->GetArgs()[2]);
			runSinglePRFErrorPadCor.push_back(singleRowPadCorTuple->GetArgs()[3]);

			wxyPadCor.push_back(1/(singleRowPadCorTuple->GetArgs()[3]*singleRowPadCorTuple->GetArgs()[3]));
			xyPadCor.push_back(singleRowPadCorTuple->GetArgs()[2]);
			sumwxyPadCor+=wxyPadCor[RowIter];
		}

		runSinglePRFMean.push_back(TMath::Mean(xy.size(),&xy[0],&wxy[0]));
		runSinglePRFMeanError.push_back(sqrt(1/sumwxy));

		runSinglePRFMeanPadCor.push_back(TMath::Mean(xyPadCor.size(),&xyPadCor[0],&wxyPadCor[0]));
		runSinglePRFMeanErrorPadCor.push_back(sqrt(1/sumwxyPadCor));

		//get the graphs from the file
		TGraphErrors *g_prf=(TGraphErrors*)file->Get("MyPRFPerformanceProcessor/PRFRow");
		TGraphErrors *g_prfPadCor=(TGraphErrors*)file->Get("MyPRFPerformanceProcessor/PRFRowPadCor");
		//check that the graph exists
		if(g_prf==0 || g_prfPadCor==0)
		{
			std::cout<<"PRFPerformance: This run will be skipped. One of the necessary root graph does not exist. Please check that you ran the corresponding processors and that the graph is in the aida file!"<<std::endl;
			continue;
		}

		//fill the vectors with the values from the tuples ([0]=mean,[2]=rms)
		//567.6-hitposition is the conversion into drift distance
		int zpos= int(567.6-zPosTuple->GetArgs()[0]) ;
		std::ostringstream legEntry;
		//legEntry << "z = "<<zpos<<" [mm]";
		legEntry << "z = "<<zpos;
		//assign color
		g_prf->SetLineColor(colors[i]);
		g_prf->SetMarkerColor(colors[i]);
		g_prf->SetTitle(legEntry.str().c_str());
		g_prfPadCor->SetLineColor(colors[i]);
		g_prfPadCor->SetMarkerColor(colors[i]);
		g_prfPadCor->SetTitle(legEntry.str().c_str());


		runPRFGraph->Add(g_prf,"P");
		runPRFPadCorGraph->Add(g_prfPadCor,"P");

		file->Close();
		delete file;
	}

	va_end(argList);

	//save the results
	outfile->cd();
	TCanvas *c2 = new TCanvas("PRF best row","PRF best row");
	TGraphErrors *runBestPRFGraph = new TGraphErrors( runZPositions.size(), &runZPositions[0], &runBestPRF[0], &runZPositionsError[0], &runBestPRFError[0] );
	runBestPRFGraph->SetTitle(Form("module_%i row_%i",bestRow.first,bestRow.second));
	runBestPRFGraph->GetXaxis()->SetTitle("drift distance [mm]");
	runBestPRFGraph->GetYaxis()->SetTitle("#sigma_{PRF} [mm]");
	runBestPRFGraph->SetLineColor(colors[0]);
	runBestPRFGraph->SetFillColor(kWhite);
	runBestPRFGraph->Draw("AP");
	gPad->Update();
	runBestPRFGraph->GetYaxis()->SetRangeUser(0,gPad->GetFrame()->GetY2()*1.5);
	c2->SaveAs(Form("%s%s",pathToOutput,"PRFBestRow.png"));
	c2->SaveAs(Form("%s%s",pathToOutput,"PRFBestRow.pdf"));
	runBestPRFGraph->SetName("PRF best row");
	runBestPRFGraph->Write();

	TCanvas *c7 = new TCanvas("PRF Pad Cor best row","PRF best row");
	TGraphErrors *runBestPRFPadCorGraph = new TGraphErrors( runZPositions.size(), &runZPositions[0], &runBestPRFPadCor[0], &runZPositionsError[0], &runBestPRFErrorPadCor[0] );
	runBestPRFPadCorGraph->SetTitle(Form("module_%i row_%i",bestRow.first,bestRow.second));
	runBestPRFPadCorGraph->GetXaxis()->SetTitle("drift distance [mm]");
	runBestPRFPadCorGraph->GetYaxis()->SetTitle("#sigma_{PRF} [mm]");
	runBestPRFPadCorGraph->SetLineColor(colors[0]);
	runBestPRFPadCorGraph->SetFillColor(kWhite);
	runBestPRFPadCorGraph->Draw("AP");
	gPad->Update();
	runBestPRFPadCorGraph->GetYaxis()->SetRangeUser(0,gPad->GetFrame()->GetY2()*1.5);
	c7->SaveAs(Form("%s%s",pathToOutput,"PRFPadCorBestRow.png"));
	c7->SaveAs(Form("%s%s",pathToOutput,"PRFPadCorBestRow.pdf"));
	runBestPRFPadCorGraph->SetName("PRF Pad cor. best row");
	runBestPRFPadCorGraph->Write();

	TCanvas *c5 = new TCanvas("PRF","PRF");
	runBestPRFGraph->Draw("AP");
	TGraphErrors *meanSinglePRFGraph = new TGraphErrors( runZPositions.size(), &runZPositions[0], &runSinglePRFMean[0], &runZPositionsError[0], &runSinglePRFMeanError[0] );
	meanSinglePRFGraph->SetLineColor(colors[1]);
	meanSinglePRFGraph->SetFillColor(kWhite);
	meanSinglePRFGraph->SetTitle("mean all rows");
	meanSinglePRFGraph->Draw("sameP");
	gPad->BuildLegend(0.5515075,0.2008929,0.8994975,0.3839286,"[mm]");
	c5->SaveAs(Form("%s%s",pathToOutput,"PRF.png"));
	c5->SaveAs(Form("%s%s",pathToOutput,"PRF.pdf"));
	meanSinglePRFGraph->SetName("PRF mean all rows");
	meanSinglePRFGraph->GetXaxis()->SetTitle("drift distance [mm]");
	meanSinglePRFGraph->GetYaxis()->SetTitle("#sigma_{PRF} [mm]");
	meanSinglePRFGraph->Write();

	TCanvas *c8 = new TCanvas("PRFPadCor","PRF");
	runBestPRFPadCorGraph->Draw("AP");
	TGraphErrors *meanSinglePRFPadCorGraph = new TGraphErrors( runZPositions.size(), &runZPositions[0], &runSinglePRFMeanPadCor[0], &runZPositionsError[0], &runSinglePRFMeanErrorPadCor[0] );
	meanSinglePRFPadCorGraph->SetLineColor(colors[1]);
	meanSinglePRFPadCorGraph->SetFillColor(kWhite);
	meanSinglePRFPadCorGraph->SetTitle("mean all rows");
	meanSinglePRFPadCorGraph->Draw("sameP");
	gPad->BuildLegend(0.5515075,0.2008929,0.8994975,0.3839286,"[mm]");
	c8->SaveAs(Form("%s%s",pathToOutput,"PRFPadCor.png"));
	c8->SaveAs(Form("%s%s",pathToOutput,"PRFPadCor.pdf"));
	meanSinglePRFPadCorGraph->SetName("PRF Pad cor. mean all rows");
	meanSinglePRFPadCorGraph->GetXaxis()->SetTitle("drift distance [mm]");
	meanSinglePRFPadCorGraph->GetYaxis()->SetTitle("#sigma_{PRF} [mm]");
	meanSinglePRFPadCorGraph->Write();

	TCanvas *c3 = new TCanvas("PRF all row","PRF all row");
	TGraphErrors *runAllPRFGraph = new TGraphErrors( runZPositions.size(), &runZPositions[0], &runAllPRF[0], &runZPositionsError[0], &runAllPRFError[0] );
	runAllPRFGraph->SetTitle("all rows");
	runAllPRFGraph->GetXaxis()->SetTitle("drift distance [mm]");
	runAllPRFGraph->GetYaxis()->SetTitle("#sigma_{PRF} [mm]");
	runAllPRFGraph->Draw("AP");
	gPad->Update();
	runAllPRFGraph->GetYaxis()->SetRangeUser(0,gPad->GetFrame()->GetY2()*1.5);
	c3->SaveAs(Form("%s%s",pathToOutput,"PRFAllRows.png"));
	c3->SaveAs(Form("%s%s",pathToOutput,"PRFAllRows.pdf"));
	runAllPRFGraph->SetName("PRF all rows");
	runAllPRFGraph->Write();


	TCanvas *c4 = new TCanvas("PRF single row","PRF single row");
	TGraphErrors *runSinglePRFGraph = new TGraphErrors( runZPositionsSingleRow.size(), &runZPositionsSingleRow[0], &runSinglePRF[0], &runZPositionsSingleRowError[0], &runSinglePRFError[0] );
	runSinglePRFGraph->SetTitle("single rows");
	runSinglePRFGraph->GetXaxis()->SetTitle("drift distance [mm]");
	runSinglePRFGraph->GetYaxis()->SetTitle("#sigma_{PRF} [mm]");
	runSinglePRFGraph->Draw("AP");
	gPad->Update();
	runSinglePRFGraph->GetYaxis()->SetRangeUser(0,gPad->GetFrame()->GetY2()*1.5);
	c4->SaveAs(Form("%s%s",pathToOutput,"PRFSingleRows.png"));
	c4->SaveAs(Form("%s%s",pathToOutput,"PRFSingleRows.pdf"));

	gStyle->SetPadRightMargin(0.18);

	TCanvas *c1 = new TCanvas("prf","prf");
	runPRFGraph->Draw("AP");
	gPad->BuildLegend(0.85,0.2,0.99,0.99,"mm");
	c1->SaveAs(Form("%s%s",pathToOutput,"PRFRow.png"));
	c1->SaveAs(Form("%s%s",pathToOutput,"PRFRow.pdf"));

	TCanvas *c6 = new TCanvas("prfPadCor","prfPadCor");
	runPRFPadCorGraph->Draw("AP");
	gPad->BuildLegend(0.85,0.2,0.99,0.99,"mm");
	c6->SaveAs(Form("%s%s",pathToOutput,"PRFRowPadCor.png"));
	c6->SaveAs(Form("%s%s",pathToOutput,"PRFRowPadCor.pdf"));


	c1->Write(); //Saving these to save plot details for future use.  --Peter
	c2->Write();
	c3->Write();
	c4->Write();
	c5->Write();
	c6->Write();
	c7->Write();
	c8->Write();

	outfile->Write();
	outfile->Close();

}
