#include <vector>
#include <iostream>
#include "TLatex.h"
#include "TCanvas.h"
#include "TH1D.h"
#include "TGraphErrors.h"
#include "TFile.h"
#include "TDirectory.h"
#include "sstream"
#include "TROOT.h"
#include <cstdarg>
#include "TROOT.h"
#include "TColor.h"
#include "TStyle.h"
#include "TNtuple.h"


/*How to call this:
.L $MARLINTPC/analysis/rootscripts/TrackParameterDistribution.C++
TrackParameterDistribution("path for output","path to the aida file", number of runs in the following list, list of the runs)
e.g TrackParameterDistribution("../myplots","../AidaFiles/",3,18850,18851,18860)
*/

//helper class
TCanvas * DrawHistogram(const char * pathToAida, const char * histoPathName, std::vector<int> runs){

	TCanvas * canvas =new TCanvas("canvas","canvas");
	//create colors
	const int BINNING=runs.size();
	Int_t colors[BINNING];

	const Int_t NRGBs = 5;

	Double_t stops[NRGBs] = { 0.00, 0.34, 0.61, 0.84, 1.00 };
	Double_t red[NRGBs]   = { 0.00, 0.00, 0.87, 1.00, 0.51 };
	Double_t green[NRGBs] = { 0.00, 0.81, 1.00, 0.20, 0.00 };
	Double_t blue[NRGBs]  = { 0.51, 1.00, 0.12, 0.00, 0.00 };
	Int_t FI = TColor::CreateGradientColorTable(NRGBs, stops, red, green, blue, BINNING);

	for (int i=0;i<BINNING;i++)
	  colors[i] = FI+i;

	//loop over the given runs
	for (unsigned int i=0;i<runs.size();i++)
	{
		//create stringstream containing the aida file name
		std::stringstream runNumberString;
		std::cout<<"run number: "<<runs[i]<<std::endl;
		runNumberString <<pathToAida<<"aida_run" <<runs[i]<< ".root";

		//open the aida file
		TFile *file  = TFile::Open(runNumberString.str().c_str(),"Read");

		//check that the file exists
		if (file==0)
		{
			std::cout<<"This run will be skipped. The aida file does not exist. Please check this!"<<std::endl;
			continue;
		}
		TNtuple * zPosTuple=(TNtuple*)file->Get("MyAnalysisCutSelectorProcessor/RunZPositions");
			//check that the tuples exists
		if (zPosTuple==0 )
		{
			std::cout<<"This run will be skipped. One of the necessary root tuples does not exist. Please check that you ran the corresponding processors and that the tuple is in the aida file!"<<std::endl;
			continue;

		}
		zPosTuple->GetEntry(0);
		int zpos= int(567.6-zPosTuple->GetArgs()[0]) ;
		std::ostringstream legEntry;
		legEntry << "z = "<<zpos;
		//get the tuples from the file
		if (gDirectory->Get(histoPathName)==0)
		{
			std::cout<<"This run will be skipped. The histogram \""<< histoPathName <<"\" does not exist. Please check this!"<<std::endl;
						continue;
		}
		TH1D *histo = dynamic_cast<TH1D*>(gDirectory->Get(histoPathName)->Clone());
		histo->SetDirectory(0);
		histo->SetTitle(legEntry.str().c_str());
		if(i==0)
		{
			histo->Draw("");
		}
		else
		{
			histo->SetLineColor(colors[i]);
			histo->Draw("same");
		}
		file->Close();
		delete file;
	}
	//gPad->BuildLegend(0.85,0.2,0.99,0.99,"[mm]");
	return canvas;
}

void TrackParametersDistribution(char * pathToOutput, char * pathToAida, int nRuns, ...){

//loading the style
gROOT->ProcessLine(".L ${MARLINTPC}/analysis/rootscripts/LCTPCStyle.C");
gROOT->ProcessLine("LCTPCStyle()");
//gStyle->SetPadRightMargin(0.18);

//initialize the input list
va_list argList;
va_start(argList,nRuns);
std::vector<int> runs;
for (int i=0;i<nRuns;i++)
{
	//get run number from input
	runs.push_back(va_arg(argList,int));
}
va_end(argList);

TFile *outfile=new TFile(Form("%s%s",pathToOutput,"TrackParametersDistribution.root"),"RECREATE");

TCanvas * canvas=DrawHistogram(pathToAida,"MyTrackParametersDistributionProcessor/phiDistributionHisto",runs);
canvas->SaveAs(Form("%s%s",pathToOutput,"PhiDistribution.png"));
canvas->SaveAs(Form("%s%s",pathToOutput,"PhiDistribution.pdf"));
outfile->cd();
canvas->Write("PhiDistribution");
delete canvas;

canvas=DrawHistogram(pathToAida,"MyTrackParametersDistributionProcessor/lambdaDistributionHisto",runs);
canvas->SaveAs(Form("%s%s",pathToOutput,"LambdaDistribution.png"));
canvas->SaveAs(Form("%s%s",pathToOutput,"LambdaDistribution.pdf"));
outfile->cd();
canvas->Write("LambdaDistribution");
delete canvas;

canvas=DrawHistogram(pathToAida,"MyTrackParametersDistributionProcessor/tanLambdaDistributionHisto",runs);
canvas->SaveAs(Form("%s%s",pathToOutput,"tanLambdaDistribution.png"));
canvas->SaveAs(Form("%s%s",pathToOutput,"tanLambdaDistribution.pdf"));
outfile->cd();
canvas->Write("tanLambdaDistribution");
delete canvas;

canvas=DrawHistogram(pathToAida,"MyTrackParametersDistributionProcessor/d0DistributionHisto",runs);
canvas->SaveAs(Form("%s%s",pathToOutput,"d0Distribution.png"));
canvas->SaveAs(Form("%s%s",pathToOutput,"d0Distribution.pdf"));
outfile->cd();
canvas->Write("d0Distribution");
delete canvas;

canvas=DrawHistogram(pathToAida,"MyTrackParametersDistributionProcessor/z0DistributionHisto",runs);
canvas->SaveAs(Form("%s%s",pathToOutput,"z0Distribution.png"));
canvas->SaveAs(Form("%s%s",pathToOutput,"z0Distribution.pdf"));
delete canvas;

canvas=DrawHistogram(pathToAida,"MyTrackParametersDistributionProcessor/OmegaDistributionHisto",runs);
canvas->SaveAs(Form("%s%s",pathToOutput,"OmegaDistribution.png"));
canvas->SaveAs(Form("%s%s",pathToOutput,"OmegaDistribution.pdf"));
outfile->cd();
canvas->Write("OmegaDistribution");
delete canvas;

canvas=DrawHistogram(pathToAida,"MyTrackParametersDistributionProcessor/MomentumDistributionHisto",runs);
canvas->SaveAs(Form("%s%s",pathToOutput,"MomentumDistribution.png"));
canvas->SaveAs(Form("%s%s",pathToOutput,"MomentumDistribution.pdf"));
canvas->SaveAs(Form("%s%s",pathToOutput,"MomentumDistribution.tex"));
outfile->cd();
canvas->Write("MomentumDistribution");
delete canvas;

canvas=DrawHistogram(pathToAida,"MyTrackParametersDistributionProcessor/Chi2DistributionHisto",runs);
canvas->SaveAs(Form("%s%s",pathToOutput,"Chi2Distribution.png"));
canvas->SaveAs(Form("%s%s",pathToOutput,"Chi2Distribution.pdf"));
outfile->cd();
canvas->Write("Chi2Distribution");
delete canvas;

canvas=DrawHistogram(pathToAida,"MyTrackParametersDistributionProcessor/NdfDistributionHisto",runs);
canvas->SaveAs(Form("%s%s",pathToOutput,"NdfDistribution.png"));
canvas->SaveAs(Form("%s%s",pathToOutput,"NdfDistribution.pdf"));
outfile->cd();
canvas->Write("NdfDistribution");
delete canvas;

canvas=DrawHistogram(pathToAida,"MyTrackParametersDistributionProcessor/dEdxDistributionHisto",runs);
canvas->SaveAs(Form("%s%s",pathToOutput,"dEdxDistribution.png"));
canvas->SaveAs(Form("%s%s",pathToOutput,"dEdxDistribution.pdf"));
outfile->cd();
canvas->Write("dEdxDistribution");
delete canvas;

canvas=DrawHistogram(pathToAida,"MyTrackParametersDistributionProcessor/NHitsDistributionHisto",runs);
canvas->SaveAs(Form("%s%s",pathToOutput,"NHitsDistribution.png"));
canvas->SaveAs(Form("%s%s",pathToOutput,"NHitsDistribution.pdf"));
outfile->cd();
canvas->Write("NHitsDistribution");
delete canvas;

}
