#include "GEMPulsingChargeEventAnalysisProcessor.h"
#include <iostream>
#include <fstream>
#include <string>
#include <vector>

// include what you need from lcio
#include <EVENT/LCEvent.h>
#include <IMPL/LCCollectionVec.h>
#include <EVENT/TrackerPulse.h>
#include <Exceptions.h>

//MarliNTPC
#include "ChannelMappingListener.h"
#include "ChannelMappingProcessor.h"
#include "TPCCondDataExeptions.h"

//GEAR
#include "gear/TPCModule.h"



using namespace lcio;
using namespace marlin;
using namespace std;


namespace marlintpc
{

	GEMPulsingChargeEventAnalysisProcessor aGEMPulsingChargeEventAnalysisProcessor;


	GEMPulsingChargeEventAnalysisProcessor::GEMPulsingChargeEventAnalysisProcessor() : Processor("GEMPulsingChargeEventAnalysisProcessor")
	{
	  // the processor description
	  _description = "GEMPulsingChargeEventAnalysisProcessor analyses the charge of pulses and generates a tuple which is used to calibrate the charge of each pad in GEMChargeVoltage.C.";

	  /* register steering parameters: name, description, class-variable, default value

		for example:
	  */

			registerInputCollection(    LCIO::TRACKERPULSE,
				"InputCollectionName" ,
				"Name of the input TrackerData collection, default: TPCData"  ,
				_inputColName ,
				string("TPCPulses")) ;

		    registerProcessorParameter("ChannelMappingCollectionName" ,
			   "Name of the collection containing the ChannelMappings",
			   _channelMappingColName,
			   tpcconddata::ADCChannelMapping::getDefaultColName());


		    registerOptionalParameter("Voltage",
		                              "Voltage of the run in V",
		                              _voltage,
		                              double(0));


		    registerOptionalParameter("ChipToChipMin",
		                              "Minimum for Chip to Chip Histogram",
									  _chipToChipMin,
		                              int(0));


		    registerOptionalParameter("ChipToChipMax",
                    				  "Maximum for Chip to Chip Histogram",
									  _chipToChipMax,
		                              int(0));

		    registerOptionalParameter("ChipToChipRMS",
		                              "Maximum for Chip to Chip Histogram RMS",
									  _chipToChipRMS,
		                              int(0));

		    registerOptionalParameter("CableToCableMin",
		                              "Minimum for Cable to Cable Histogram",
									  _cableToCableMin,
		                              int(0));

		    registerOptionalParameter("CableToCableMax",
		                              "Maximum for Cable to Cable Histogram",
									  _cableToCableMax,
		                              int(0));

		    registerOptionalParameter("CableToCableRMS",
		                              "Maximum for Cable to Cable Histogram RMS",
									  _cableToCableRMS,
		                              int(0));

		    registerOptionalParameter("ChipToChipDeviationRange",
                    				  "Range for Chip to Chip Deviation Histogram",
									  _chipToChipDeviationRange,
		                              int(0));


		    registerOptionalParameter("ChipToChipDeviationRMS",
                    				  "Range for Chip to Chip Deviation Histogram RMS",
									  _chipToChipDeviationRMS,
		                              int(0));

		    registerOptionalParameter("CableToCableDeviationRange",
                    				  "Range for Cable to Cable Deviation Histogram",
									  _cableToCableDeviationRange,
		                              int(0));

		    registerOptionalParameter("CableToCableDeviationRMS",
                    				  "Range for Cable to Cable Deviation Histogram RMS",
									  _cableToCableDeviationRMS,
		                              int(0));

		    registerOptionalParameter("eventChargeAverageMin",
                    				  "Minimum for event Charge Average",
									  _eventChargeAverageMin,
		                              int(0));

		    registerOptionalParameter("eventChargeAverageMax",
                    				  "Maximum for event Charge Average",
									  _eventChargeAverageMax,
		                              int(0));

		    registerOptionalParameter("eventChargeAverageRMS",
                    				  "Maximum for event Charge Average RMS",
									  _eventChargeAverageRMS,
		                              int(0));

	  /*
			registerOutputCollection(   LCIO::TRACKERDATA,
			  "OutputCollectionName" ,
			  string("Name of the output TrackerData collection")+
			  string("only the data related to a pulse will be stored")+
			  string(\, default: TPCDataInPulses),
			  _outputCollectionName ,
			  string("TPCDataInPulses")) ;

			// It's nive to have the default value in the comment
			registerProcessorParameter( "MinPulsesPerHit" ,
			   "Minimum number of pulses for a valid hit, default: 3",
			   _minPulsesPerHit,
			   int(3));

			// Optional parameters should only be used when set
			// in the steering file.
			// The default value is there only for technical reasons,
			// it is meaningless.
			//
			// If properly implemented the cut is not applied when the parameter is not set:
			//   if (parameterSet("ChageCut")) { //apply the cut }
			//
			registerOptionalParameter( "ChargeCut" ,
				"If the charge is below this value the data will be ignored",
				_chargeCut,
			   float(17.8));
	  */
	}


	void GEMPulsingChargeEventAnalysisProcessor::init()
	{
	  streamlog_out(DEBUG) << " init() called  " << endl;

	  // you might want to list the used parameters by invoking
	  printParameters();
		AIDAProcessor::tree(this);


		_chargeCalibrationTuple = new TNtuple("chargeCalibrationTuple","chargeCalibrationTuple","module:pad:chargeMean:chargeMeanError:voltage");

		_chipToChipChargeAverage = new TH1F ( "chipToChipChargeAverage", "Charge average of chips", 100, _chipToChipMin, _chipToChipMax );
		_chipToChipChargeAverage->GetXaxis()->SetTitle("Average");
		_chipToChipChargeAverage->GetYaxis()->SetTitle("Chips");

		_RMSChip_Charge = new TH1F ( "ChargeRMSChip", "Charge-RMS of chips", 100, 0, _chipToChipRMS );
		_RMSChip_Charge ->GetXaxis()->SetTitle("Charge-RMS");
		_RMSChip_Charge ->GetYaxis()->SetTitle("Chips");

		_cableToCableChargeAverage = new TH1F ( "CableToCable_Charge_Average", "Charge average of cables", 100, _cableToCableMin, _cableToCableMax);
		_cableToCableChargeAverage->GetXaxis()->SetTitle("Average");
		_cableToCableChargeAverage->GetYaxis()->SetTitle("Cables");

		_RMSCable_Charge = new TH1F ( "ChargeRMSCable", "Charge-RMS of Cable", 100, 0, _cableToCableRMS );
		_RMSCable_Charge ->GetXaxis()->SetTitle("Charge-RMS");
		_RMSCable_Charge ->GetYaxis()->SetTitle("Cable");

		_chipToChipChargeAverageDeviation = new TH1F ( "chipToChipChargeAverage_Deviation", "Charge average_Deviation of chips", 100, -_chipToChipDeviationRange, _chipToChipDeviationRange );
		_chipToChipChargeAverageDeviation->GetXaxis()->SetTitle("Average (Deviation)");
		_chipToChipChargeAverageDeviation->GetYaxis()->SetTitle("Chips");

		_RMSChipChargeDeviation = new TH1F ( "ChargeRMSChip_Deviation", "Charge-RMS_Deviation of chips", 100, 0, _chipToChipDeviationRMS );
		_RMSChipChargeDeviation ->GetXaxis()->SetTitle("Charge-RMS (Deviation)");
		_RMSChipChargeDeviation ->GetYaxis()->SetTitle("Chips");

		_cableToCableChargeAverageDeviation = new TH1F ( "CableToCable_Charge_Average_Deviation", "Charge average_Deviation of cables", 100, -_cableToCableDeviationRange, _cableToCableDeviationRange);
		_cableToCableChargeAverageDeviation->GetXaxis()->SetTitle("Average (Deviation)");
		_cableToCableChargeAverageDeviation->GetYaxis()->SetTitle("Cables");

		_RMSCableChargeDeviation = new TH1F ( "ChargeRMSCable_Deviation", "Charge-RMS_Deviation of Cable", 100, 0, _cableToCableDeviationRMS );
		_RMSCableChargeDeviation ->GetXaxis()->SetTitle("Charge-RMS (Deviation)");
		_RMSCableChargeDeviation ->GetYaxis()->SetTitle("Cables");

		_eventAverage = new TH1F ("event_ChargeAverages", "Charge-average per event", 50, _eventChargeAverageMin, _eventChargeAverageMax);
		_eventAverage -> GetXaxis()->SetTitle("Charge [adc]");
		_eventAverage -> GetYaxis()->SetTitle("#entries");

		_eventRMS = new TH1F ("event_ChargeRMS", "Charge-RMS per event", 50, 0, _eventChargeAverageRMS);
		_eventRMS -> GetXaxis()->SetTitle("RMS [adc]");
		_eventRMS -> GetYaxis()->SetTitle("#entries");

		_eventAverageTemp = new TH1F ("event_ChargeAveragesTemp", "Charge-averageTemp per event", 50, _eventChargeAverageMin, _eventChargeAverageMax);
		_eventRMSTemp = new TH1F ("event_ChargeRMSTemp", "Charge-RMSTemp per event", 50, 0, _eventChargeAverageRMS);


		_padToPadChargeDeviationAverageXY = new TH2F ( "PulseChargeDeviationMeanVSXY", "Charge of pulses (Deviation Mean)", 100, 0, 0, 100, 0, 0 );
		_padToPadChargeDeviationAverageXY -> GetXaxis()->SetTitle("x [mm]");
		_padToPadChargeDeviationAverageXY -> GetYaxis()->SetTitle("y [mm]");
		_padToPadChargeDeviationAverageXY -> GetZaxis()->SetTitle("z [adc]");

		_padToPadChargeDeviationRMSXY = new TH2F ( "PulseChargeDeviationRMSVSXY", "Charge of pulses (Deviation RMS)", 100, 0, 0, 100, 0, 0 );
		_padToPadChargeDeviationRMSXY -> GetXaxis()->SetTitle("x [mm]");
		_padToPadChargeDeviationRMSXY -> GetYaxis()->SetTitle("y [mm]");
		_padToPadChargeDeviationRMSXY -> GetZaxis()->SetTitle("z [adc]");

		_chipToChipChargeAverageXY = new TH2F ( "PulseChargeVSXY_Chip", "Charge of pulses (chip)", 100, 0, 0, 100, 0, 0 );
		_chipToChipChargeAverageXY -> GetXaxis()->SetTitle("x [mm]");
		_chipToChipChargeAverageXY -> GetYaxis()->SetTitle("y [mm]");
		_chipToChipChargeAverageXY -> GetZaxis()->SetTitle("z [adc]");

		_cableToCableChargeAverageXY = new TH2F ( "PulseChargeVSXY_Cable", "Charge of pulses (cable)", 100, 0, 0, 100, 0, 0 );
		_cableToCableChargeAverageXY -> GetXaxis()->SetTitle("x [mm]");
		_cableToCableChargeAverageXY -> GetYaxis()->SetTitle("y [mm]");
		_cableToCableChargeAverageXY -> GetZaxis()->SetTitle("z [adc]");

		_channelMappingListener = new ChannelMappingListener(_channelMappingColName);

		_gearParameters= &marlin::Global::GEAR->getTPCParameters();
	}


	void GEMPulsingChargeEventAnalysisProcessor::processRunHeader(LCRunHeader*  /*run*/)
	{
	  // do something for each run
	}


	void GEMPulsingChargeEventAnalysisProcessor::processEvent(LCEvent* evt)
	{
		AIDAProcessor::tree(this);

		  // do something for each event, usually access the input collection:

		  // use an LCCollectionVec instead of LCCollection so we can use iterators later
		  LCCollectionVec* inputColVec = 0;

		  // do a try / catch in case there is an empty event
		  try
		  {
			inputColVec = dynamic_cast<LCCollectionVec *>(evt->getCollection(_inputColName) );
		  }
		  catch (DataNotAvailableException &e)
		  {
			// no input data, just return and process the next event
			streamlog_out(MESSAGE) << "No pulses available in event " << evt->getEventNumber() << endl;
			return;
		  }

		  // loop the data collection and do something
		  for( LCCollectionVec::iterator inputObjectIter = inputColVec->begin() ;
			   inputObjectIter < inputColVec->end() ;
			   inputObjectIter++ )
		  {


				// dynamic cast the input object (LCObject) to what it really is e.g.

				TrackerPulse * myTrackerPulse = dynamic_cast<TrackerPulse *>( *inputObjectIter );

				if(myTrackerPulse!=0)
				{
//					if(myTrackerPulse->getQuality() != 0)
//						continue;
						// for objects derived from LCGenericObject or LCFixedObject (like conditions data)
						// you have to use special constructor
						// TPCConditions myTPCConditions( dynamic_cast<LCGenericObject*>( *inputObjectIter ) );
						//now do something with the data ...


					// fill map with key: module,pad and charge
					// _inputPulseMap saves only for the event, _inputPulseMapTuple saves the charges of each event in one vector
						int modul = myTrackerPulse->getCellID1();
						int pad   = myTrackerPulse->getCellID0();
						std::pair<int, int> key(modul, pad);

						std::map< std::pair<int,int>, std::vector<float> >::iterator vectoriter;
						vectoriter = _inputPulseMap.find(key);

						if(vectoriter == _inputPulseMap.end())
							{
							std::vector<float> tempVec;
							_inputPulseMap.insert( std::make_pair(key,tempVec));
							}

						std::map< std::pair<int,int>, std::vector<float> >::iterator vectoriterTuple;
						vectoriterTuple = _inputPulseMapTuple.find(key);

						if(vectoriterTuple == _inputPulseMapTuple.end())
							{
							std::vector<float> tempVecTuple;
							_inputPulseMapTuple.insert( std::make_pair(key,tempVecTuple));
							}


						vectoriter = _inputPulseMap.find(key);
						try
							{
								float Charge = myTrackerPulse->getCharge();
								((*vectoriter).second).push_back(Charge);
							}
						catch(std::exception &e)
							{
								streamlog_out(WARNING)<<"calculateDistanceRow for distances failed. Value ignored!"<<std::endl;
							}

						vectoriterTuple = _inputPulseMapTuple.find(key);
						try
							{
								float ChargeTuple = myTrackerPulse->getCharge();
								((*vectoriterTuple).second).push_back(ChargeTuple);
							}
						catch(std::exception &e)
							{
								streamlog_out(WARNING)<<"calculateDistanceRow for distances failed. Value ignored!"<<std::endl;
							}
				}
		  }


			std::map< std::pair<int,int>, std::vector<float> >::iterator it;
			for (it=_inputPulseMap.begin(); it!= _inputPulseMap.end(); ++it)
			{
					try
					{
						//FIXME: not used: std::pair<int, int> tempkey_gear = _channelMappingListener->getGearMapping(std::make_pair((*it).first.second, (*it).first.first));
						std::vector <float> tempvec = (*it).second;
						double tempaverage = TMath::Mean(tempvec.begin(), tempvec.end());
						_inputPulseMapAverage.insert(std::make_pair((*it).first, tempaverage));
					}
					catch (tpcconddata::CondDataNotAvailableException& e)
					{

					}
			}



//			calculate charge average off all pads (per event)
			std::vector<double> eventChargesTemp;
			std::map< std::pair<int,int>, double >::iterator ii;
			for (ii=_inputPulseMapAverage.begin(); ii!= _inputPulseMapAverage.end(); ++ii)
			{
					eventChargesTemp.push_back((*ii).second);
			}


			double eventAverageTemp = TMath::Mean(eventChargesTemp.begin(), eventChargesTemp.end());
			_eventAverageTemp -> Fill(eventAverageTemp);

			double eventRMSTemp = TMath::RMS(eventChargesTemp.begin(), eventChargesTemp.end());
			_eventRMSTemp -> Fill(eventRMSTemp);

			std::vector<double> eventChargesTemp2;
			std::map< std::pair<int,int>, double >::iterator iii;
			for (iii=_inputPulseMapAverage.begin(); iii!= _inputPulseMapAverage.end(); ++iii)
			{
				if(abs((*iii).second-eventAverageTemp) < 5*eventRMSTemp)
				{
					eventChargesTemp2.push_back((*iii).second);
				}
			}

			double event_average = TMath::Mean(eventChargesTemp2.begin(), eventChargesTemp2.end());
			_eventAverage -> Fill(event_average);

			_eventChargeAverages.push_back(event_average);

			double eventRMS = TMath::RMS(eventChargesTemp2.begin(), eventChargesTemp2.end());
			_eventRMS -> Fill(eventRMS);


			eventChargesTemp.clear();
			eventChargesTemp2.clear();



//			calculate deviation of each pad to the average of all pads
			std::map< std::pair<int,int>, double >::iterator i;
			for (i=_inputPulseMapAverage.begin(); i!= _inputPulseMapAverage.end(); ++i)
			{
				std::map< std::pair<int,int>, std::vector<double> >::iterator vectoriter1;
				vectoriter1 = _chargeDeviationMap.find((*i).first);
				if(vectoriter1 == _chargeDeviationMap.end())
					{
						std::vector<double> tempVec1;
						_chargeDeviationMap.insert( std::make_pair((*i).first,tempVec1));
					}


				vectoriter1 = _chargeDeviationMap.find((*i).first);
				try
				{
					double deviationtemp = ((*i).second) - event_average;
					if(abs(deviationtemp) < 5* eventRMS)
					{
						((*vectoriter1).second).push_back(deviationtemp);
					}

				}

				catch (std::exception &e)
				{
					streamlog_out(WARNING)<<"calculateDistanceRow for distances failed. Value ignored!"<<std::endl;
				}
			}


//		chip to chip and cable to cable
		vector<double> ChipAverage;
		vector<double> CableAverage;
		int CounterNew = 0;
		int CounterOld = 0;

		std::map< std::pair<int,int>, double >::iterator iter;
		for (iter=_inputPulseMapAverage.begin(); iter!= _inputPulseMapAverage.end(); ++iter)
		{
			CounterNew = (*iter).first.second;

//			chip to chip (16 pads are one chip)
			if ((int)(CounterNew/16) == (int)(CounterOld/16))
			{
				ChipAverage.push_back((*iter).second);
			}
			else
			{
				std::pair<int,int> key = make_pair(((*iter).first).first, (int)(CounterOld/16) );
				std::map< std::pair<int,int>, std::pair<int, std::vector<double> > >::iterator itermap;

				itermap = _chipMap.find(key);

				if (itermap == _chipMap.end())
				{
					std::vector<double> tempVecchip;
					std::pair<int, std::vector<double> > tempValue = std::make_pair((*iter).first.second, tempVecchip);
					_chipMap.insert( std::make_pair(key,tempValue));
				}

				itermap = _chipMap.find(key);
				try
				{
					double ChipAveragetemp = TMath::Mean(ChipAverage.begin(), ChipAverage.end());
					((*itermap).second.second).push_back(ChipAveragetemp);
				}

				catch (std::exception &e)
				{

				}

				ChipAverage.clear();
				ChipAverage.push_back((*iter).second);

			}

//			cable to cable (32 pads are one cable)
			if ((int)(CounterNew/32) == (int)(CounterOld/32))
			{
				CableAverage.push_back((*iter).second);
			}
			else
			{
				std::pair<int,int> key = make_pair(((*iter).first).first, (int)(CounterOld/32) );
				std::map< std::pair<int,int>, std::pair<int, std::vector<double> > >::iterator itermap2;

				itermap2 = _cableMap.find(key);

				if (itermap2 == _cableMap.end())
				{
					std::vector<double> tempVecCable;
					std::pair<int, std::vector<double> > tempValueCable = std::make_pair((*iter).first.second, tempVecCable);
					_cableMap.insert( std::make_pair(key,tempValueCable));
				}

				itermap2 = _cableMap.find(key);
				try
				{
					double CableAveragetemp = TMath::Mean(CableAverage.begin(), CableAverage.end());
					((*itermap2).second.second).push_back(CableAveragetemp);
				}

				catch (std::exception &e)
				{

				}

				CableAverage.clear();
				CableAverage.push_back((*iter).second);
			}
			CounterOld = CounterNew;
		}

		_inputPulseMap.clear();
		_inputPulseMapAverage.clear();
	}


	void GEMPulsingChargeEventAnalysisProcessor::check(LCEvent*  /*evt*/)
	{
	  // check what is needed here
	}


	void GEMPulsingChargeEventAnalysisProcessor::end()
	{
	  // do something at the end of the processing, e.g. cleaning up


//		Chip to chip --> 1D Histogram Mean of Mean & RMS of Mean
		std::map< std::pair<int,int>, std::pair<int, std::vector<double> > >::iterator it;
		for (it=_chipMap.begin(); it!= _chipMap.end(); ++it)
		{
			_chipToChipChargeAverage-> Fill(TMath::Mean(((*it).second.second).begin(), ((*it).second.second).end()));
			_RMSChip_Charge-> Fill(TMath::RMS(((*it).second.second).begin(), ((*it).second.second).end()));
		}

//		Chip to chip --> 2D Histogram
		std::map< std::pair<int,int>, std::pair<int, std::vector<double> > >::iterator it1;
		for (it1=_chipMap.begin(); it1!= _chipMap.end(); ++it1)
		{
				try
				{
					std::pair<int, int> tempkey_gear = _channelMappingListener->getGearMapping(std::make_pair((*it1).second.first, (*it1).first.first));
					const gear::TPCModule& tempModule = _gearParameters->getModule(tempkey_gear.second);
					gear::Vector2D padcenter=tempModule.getPadCenter(tempkey_gear.first);
					_chipToChipChargeAverageXY -> Fill(padcenter[0], padcenter[1] , TMath::Mean(((*it1).second.second).begin(), ((*it1).second.second).end()));

				}
				catch (tpcconddata::CondDataNotAvailableException& e)
				{

				}
		}


//		Cable to cable --> 1D Histogram
		std::map< std::pair<int,int>, std::pair<int, std::vector<double> > >::iterator it2;
		for (it2=_cableMap.begin(); it2!= _cableMap.end(); ++it2)
		{
			_cableToCableChargeAverage-> Fill(TMath::Mean(((*it2).second.second).begin(), ((*it2).second.second).end()));
			_RMSCable_Charge-> Fill(TMath::RMS(((*it2).second.second).begin(), ((*it2).second.second).end()));
		}



//		Cable to cable --> 2D Histogram

		std::map< std::pair<int,int>, std::pair<int, std::vector<double> > >::iterator it3;
		for (it3=_cableMap.begin(); it3!= _cableMap.end(); ++it3)
		{

				try
				{
					std::pair<int, int> tempkey_gear = _channelMappingListener->getGearMapping(std::make_pair((*it3).second.first, (*it3).first.first));
					const gear::TPCModule& tempModule = _gearParameters->getModule(tempkey_gear.second);
					gear::Vector2D padcenter=tempModule.getPadCenter(tempkey_gear.first);
					_cableToCableChargeAverageXY -> Fill(padcenter[0], padcenter[1], TMath::Mean(((*it3).second.second).begin(), ((*it3).second.second).end()));
				}
				catch (tpcconddata::CondDataNotAvailableException& e)
				{

				}
		}


//		calculate average of deviations
		vector<double> ChipAverage_Deviation;
		vector<double> CableAverage_Deviation;
		int CounterNew_Deviation = 0;
		int CounterOld_Deviation = 0;

		std::map< std::pair<int,int>, std::vector<double> >::iterator iter;
		for (iter=_chargeDeviationMap.begin(); iter!= _chargeDeviationMap.end(); ++iter)
		{
			std::pair<double, double> Average_RMS = make_pair(TMath::Mean(((*iter).second).begin(), ((*iter).second).end()), TMath::RMS(((*iter).second).begin(), ((*iter).second).end()));
			_chargeDeviationMapAverageRMS.insert(make_pair((*iter).first, Average_RMS));


//			pad to pad deviation (Mean + RMS) --> 2D Histogram
			try
			{
				std::pair<int, int> tempkey_gear = _channelMappingListener->getGearMapping(std::make_pair((*iter).first.second, (*iter).first.first));
				std::vector <double> tempvec = (*iter).second;
				double tempaverage = TMath::Mean(tempvec.begin(), tempvec.end());
				double tempRMS = TMath::RMS(tempvec.begin(), tempvec.end());

				const gear::TPCModule& tempModule = _gearParameters->getModule(tempkey_gear.second);
				gear::Vector2D padcenter=tempModule.getPadCenter(tempkey_gear.first);

				streamlog_out(DEBUG) << "x " << padcenter[0] << "; y " << padcenter[1] << endl;

				_padToPadChargeDeviationAverageXY -> Fill(padcenter[0], padcenter[1], tempaverage);
				_padToPadChargeDeviationRMSXY -> Fill(padcenter[0], padcenter[1], tempRMS);
			}
			catch (tpcconddata::CondDataNotAvailableException& e)
			{

			}

			CounterNew_Deviation = (*iter).first.second;


//			chip to chip deviation
			if ((int)CounterNew_Deviation/16 == (int)CounterOld_Deviation/16)
			{
				double tempDeviationChip = TMath::Mean(((*iter).second).begin(), ((*iter).second).end());
				ChipAverage_Deviation.push_back(tempDeviationChip);
				tempDeviationChip = 0;
			}
			else
			{
				_chipToChipChargeAverageDeviation-> Fill(TMath::Mean(ChipAverage_Deviation.begin(), ChipAverage_Deviation.end()));
				_RMSChipChargeDeviation-> Fill(TMath::RMS(ChipAverage_Deviation.begin(), ChipAverage_Deviation.end()));

				streamlog_out(DEBUG) << "Chip Mean" << TMath::Mean(ChipAverage_Deviation.begin(), ChipAverage_Deviation.end()) << endl;

				ChipAverage_Deviation.clear();
				ChipAverage_Deviation.push_back(TMath::Mean(((*iter).second).begin(), ((*iter).second).end()));
			}


//			cable to cable deviation
			if ((int)CounterOld_Deviation/32 == (int)CounterNew_Deviation/32)
			{

				double tempDeviationCable = TMath::Mean(((*iter).second).begin(), ((*iter).second).end());
				CableAverage_Deviation.push_back(tempDeviationCable);
				tempDeviationCable = 0;
			}
			else
			{
				_cableToCableChargeAverageDeviation-> Fill(TMath::Mean(CableAverage_Deviation.begin(), CableAverage_Deviation.end()));
				_RMSCableChargeDeviation-> Fill(TMath::RMS(CableAverage_Deviation.begin(), CableAverage_Deviation.end()));
				CableAverage_Deviation.clear();
				CableAverage_Deviation.push_back(TMath::Mean(((*iter).second).begin(), ((*iter).second).end()));
			}
			CounterOld_Deviation = CounterNew_Deviation;
		}

		_chipToChipChargeAverageDeviation-> Fill(TMath::Mean(ChipAverage_Deviation.begin(), ChipAverage_Deviation.end()));
		_RMSChipChargeDeviation-> Fill(TMath::RMS(ChipAverage_Deviation.begin(), ChipAverage_Deviation.end()));

		_cableToCableChargeAverageDeviation-> Fill(TMath::Mean(CableAverage_Deviation.begin(), CableAverage_Deviation.end()));
		_RMSCableChargeDeviation-> Fill(TMath::StdDev(CableAverage_Deviation.begin(), CableAverage_Deviation.end()));


//		Fill Tuple with module, pad, charge, error and voltage
		std::map< std::pair<int,int>, std::vector<float> > ::iterator tupeliter;
		for(tupeliter=_inputPulseMapTuple.begin(); tupeliter != _inputPulseMapTuple.end(); ++ tupeliter)
		{
			std::vector<float> tempVec = (*tupeliter).second;
			int module = (*tupeliter).first.first;
			int pad = (*tupeliter).first.second;
			double ChargeMean = TMath::Mean(tempVec.begin(), tempVec.end());
			double ChargeMeanErr = TMath::RMS(tempVec.begin(), tempVec.end());

			_chargeCalibrationTuple -> Fill(module, pad, ChargeMean, ChargeMeanErr, _voltage);
		}


	}
} // close namespace brackets

