#include "HitAnaProcessor.h"

#include <iostream>
#include <string>

// include what you need from lcio
#include <EVENT/LCEvent.h>
#include <IMPL/LCCollectionVec.h>
#include <EVENT/TrackerHit.h>
#include <EVENT/TrackerHit.h>
#include <Exceptions.h>
#include <EVENT/TrackerPulse.h>
using namespace lcio;
using namespace marlin;
using namespace std;

// GEAR
#include <gear/TPCParameters.h>
#include <gear/TPCModule.h>
#include <gearimpl/GlobalPadIndex.h>
//MARLIN
#include <marlin/Global.h>


namespace marlintpc
{

HitAnaProcessor aHitAnaProcessor;


HitAnaProcessor::HitAnaProcessor() : Processor("HitAnaProcessor")
{
  // the processor description
  _description = "HitAnaProcessor is intended to ...";
  
  /* register steering parameters: name, description, class-variable, default value

    for example:
  */

        registerInputCollection(    LCIO::TRACKERHIT,
            "InputCollectionName" ,
            "Name of the input TrackerData collection, default: TPCData"  ,
            _inputColName ,
            string("TPCHits")) ;





}


void HitAnaProcessor::init()
{ 
  streamlog_out(DEBUG) << " init() called  " << endl;
  AIDAProcessor::tree(this);
  // you might want to list the used parameters by invoking
  printParameters();

_tuple=new TNtuple("Hit","Hit","Q:T:W:ModID:ChID");

}


void HitAnaProcessor::processRunHeader(LCRunHeader*  /*run*/)
{
  // do something for each run
} 


void HitAnaProcessor::processEvent(LCEvent* evt)
{ 
  // do something for each event, usually access the input collection:

  // use an LCCollectionVec instead of LCCollection so we can use iterators later
  LCCollectionVec* inputColVec = 0;

  // do a try / catch in case there is an empty event
  try
  {
    inputColVec = dynamic_cast<LCCollectionVec *>(evt->getCollection( _inputColName ) );
  }
  catch (DataNotAvailableException &e)
  {
    // no input data, just return and process the next event
    return;
  }
  const gear::TPCParameters& tpcParameters = Global::GEAR->getTPCParameters();
 
  // loop the data collection and do something 
  for( LCCollectionVec::iterator inputObjectIter = inputColVec->begin() ;
       inputObjectIter < inputColVec->end() ;
       inputObjectIter++ )
  {
    // dynamic cast the input object (LCObject) to what it really is e.g.
    TrackerHit * myTrackerHit = dynamic_cast<TrackerHit *>( *inputObjectIter );

    // for objects derrived from LCGenericObject or LCFixedObject (like conditions data)
    // you have to use special constructor
    // TPCConditions myTPCConditions( dynamic_cast<LCGenericObject*>( *inputObjectIter ) );
    
    //now do something with the data ...
    _tuple->Fill( myTrackerHit->getEDep(),myTrackerHit->getTime(), myTrackerHit->getRawHits().size(),
    			myTrackerHit->getCellID0(),myTrackerHit->getCellID1());

    const double *position = myTrackerHit->getPosition();
    const gear::TPCModule& module = tpcParameters.getNearestModule( position[0],position[1]);

    std::vector<std::pair<double,double> > insideHitPos;
    for ( LCObjectVec::const_iterator pulseIter = myTrackerHit->getRawHits().begin() ;
	  pulseIter < myTrackerHit->getRawHits().end() ; pulseIter++ )
      {
	TrackerPulse const * pulse = dynamic_cast<TrackerPulse const *>(*pulseIter);
	gear::Vector2D globalPadCenter = module.getPadCenter( pulse->getCellID0() );
	std::pair<double,double> mypair=std::make_pair(globalPadCenter[1],pulse->getCharge());
	insideHitPos.push_back(mypair);
      }
    //std::cout<<insideHitPos.size()<<" "<<insideHitPos[0].first<<" "<<insideHitPos[0].second<<std::endl;
//     std::sort(insideHitPos.begin(),insideHitPos.end(),padComp);
//     for (int i=0;i<insideHitPos.size();i++)
//       std::cout<<insideHitPos[i].second<<" ";
//     std::cout<<std::endl;
  }
}


void HitAnaProcessor::check(LCEvent*  /*evt*/)
{ 
  // check what is needed here 
}


void HitAnaProcessor::end()
{ 
  // do something at the end of the processing, e.g. cleaning up
}
} // close namespace brackets
