#include "PulseAnaProcessor.h"

#include <iostream>
#include <string>
#include <algorithm>

// include what you need from lcio
#include <EVENT/LCEvent.h>
#include <IMPL/LCCollectionVec.h>
#include <EVENT/TrackerData.h>
#include <EVENT/TrackerPulse.h>
#include <Exceptions.h>

using namespace lcio;
using namespace marlin;
using namespace std;



namespace marlintpc
{

PulseAnaProcessor aPulseAnaProcessor;


PulseAnaProcessor::PulseAnaProcessor() : Processor("PulseAnaProcessor")
{
  // the processor description
  _description = "PulseAnaProcessor is intended to ...";
  
  /* register steering parameters: name, description, class-variable, default value

    for example:
  */

        registerInputCollection(    LCIO::TRACKERPULSE,
            "InputCollectionName" ,
            "Name of the input TrackerData collection, default: TPCData"  ,
            _inputColName ,
            string("TPCPulses")) ;

  /*
        registerOutputCollection(   LCIO::TRACKERDATA,
          "OutputCollectionName" ,
          string("Name of the output TrackerData collection")+
          string("only the data related to a pulse will be stored")+
          string(\, default: TPCDataInPulses),
          _outputCollectionName ,
          string("TPCDataInPulses")) ;

        // It's nive to have the default value in the comment
        registerProcessorParameter( "MinPulsesPerHit" ,
	       "Minimum number of pulses for a valid hit, default: 3",
	       _minPulsesPerHit,
	       int(3));

        // Optional parameters should only be used when set
        // in the steering file.
        // The default value is there only for technical reasons,
        // it is meaningless.
        //
        // If properly implemented the cut is not applied when the parameter is not set:
        //   if (parameterSet("ChageCut")) { //apply the cut }
        //
        registerOptionalParameter( "ChargeCut" ,
            "If the charge is below this value the data will be ignored",
            _chargeCut,
           float(17.8));
  */



}


void PulseAnaProcessor::init()
{ 
  streamlog_out(DEBUG) << " init() called  " << endl;
  AIDAProcessor::tree(this);
  // you might want to list the used parameters by invoking
  printParameters();

_tuple=new TNtuple("pulse","pulse","Q:T:L:Qmax:ModID:ChID");

}


void PulseAnaProcessor::processRunHeader(LCRunHeader*  /*run*/)
{
  // do something for each run
} 


void PulseAnaProcessor::processEvent(LCEvent* evt)
{ 
  // do something for each event, usually access the input collection:

  // use an LCCollectionVec instead of LCCollection so we can use iterators later
  LCCollectionVec* inputColVec = 0;

  // do a try / catch in case there is an empty event
  try
  {
    inputColVec = dynamic_cast<LCCollectionVec *>(evt->getCollection( _inputColName ) );
  }
  catch (DataNotAvailableException &e)
  {
    // no input data, just return and process the next event
    return;
  }

  // loop the data collection and do something 
  for( LCCollectionVec::iterator inputObjectIter = inputColVec->begin() ;
       inputObjectIter < inputColVec->end() ;
       inputObjectIter++ )
  {
    // dynamic cast the input object (LCObject) to what it really is e.g.
    TrackerPulse * myTrackerPulse = dynamic_cast<TrackerPulse *>( *inputObjectIter );
    TrackerData * myTrackerData = myTrackerPulse->getTrackerData();
    std::vector<float> charges=	myTrackerData->getChargeValues();
    float max=*std::max_element(charges.begin(),charges.end());
    // for objects derrived from LCGenericObject or LCFixedObject (like conditions data)
    // you have to use special constructor
    // TPCConditions myTPCConditions( dynamic_cast<LCGenericObject*>( *inputObjectIter ) );
    
    //now do something with the data ...
    _tuple->Fill( myTrackerPulse->getCharge(),myTrackerPulse->getTime(), charges.size(),
    			max,myTrackerPulse->getCellID1(),myTrackerPulse->getCellID0());
  }
}


void PulseAnaProcessor::check(LCEvent*  /*evt*/)
{ 
  // check what is needed here 
}


void PulseAnaProcessor::end()
{ 
  // do something at the end of the processing, e.g. cleaning up
}
} // close namespace brackets
