#include "PulseCounterProcessor.h"

//C++
#include <iostream>
#include <cmath>
#include <sstream>

//LCIO
#include <IMPL/LCCollectionVec.h>
#include <EVENT/Track.h>
#include <EVENT/TrackerPulse.h>

// Aida
//#ifdef MARLIN_USE_AIDA // the whole processor does not make sense without AIDA
#include <AIDA/AIDA.h>
#include <marlin/AIDAProcessor.h>
#include <AIDA/IHistogramFactory.h>
#include <AIDA/IHistogram1D.h>
// endif

//MARLIN
#include <marlin/Global.h>

using namespace std;
using namespace lcio ;
using namespace marlin ;

namespace marlintpc
{

	PulseCounterProcessor aPulseCounterProcessor; ///< Necessary instance of the processor.

	PulseCounterProcessor::PulseCounterProcessor() : Processor ("PulseCounterProcessor")
	{
		// register steering parameters: name, description, class-variable, default value ...
		registerInputCollection (LCIO::TRACKERPULSE,
		                         "InputPulses" ,
		                         "Name of the input pulse collection (default: TPCPulses)"  ,
		                         _pulsesColName ,
		                         std::string ("TPCPulses")) ;

		registerProcessorParameter ("Channels" ,
		                            "Number of Channels (default:  128)"  ,
		                            _nChannels ,
		                            int (128)) ;

		// processor description
		_description = "Generates histogram of number of pulses per channel" ;
	}


	
	PulseCounterProcessor::~PulseCounterProcessor()
	{
	}



	void PulseCounterProcessor::init()
	{
		// usually a good idea to
		printParameters() ;

		// compose title and name of the histograms ...
		string histoTitle = string ("Pulses per channel");
		string histoName  = string ("PulsesPerChannel");

		streamlog_out (DEBUG0) << histoName << " : " << histoTitle << endl;

		// get a histogram from the AIDAProcessor
		_pulsesPerChannelHisto =   AIDAProcessor::histogramFactory (this)->
		                           createHistogram1D (histoName,
		                                              histoTitle,
		                                              _nChannels, 0, _nChannels - 1);
	}


	
	void PulseCounterProcessor::processRunHeader(lcio::LCRunHeader* run)
	{
		// Revision logging ...
		run->parameters().setValue (_processorName + "_revision", "$Rev: 3907 $");

		// Parameter logging ...
		for (ProcParamMap::iterator i = _map.begin(); i != _map.end(); i++)
		{
			if (! i->second->isOptional() || i->second->valueSet())
			{
				run->parameters().setValue (_processorName + "_" + i->second->name(), i->second->value());
			}
		}
	}

  void PulseCounterProcessor::processEvent(lcio::LCEvent * evt)
	{
		//read input data ...
		streamlog_out (MESSAGE3) << "Event: " << evt->getEventNumber() << std::endl;

		LCCollectionVec* pulsesCollection = 0;
		
		try
		{
			pulsesCollection = dynamic_cast<LCCollectionVec*> (evt->getCollection (_pulsesColName)) ;
		}
		catch (DataNotAvailableException &e)
		{
			// there are no pulses
			// continue with next event
			return;
		}

		// if the collection exists iterate the pulses to fill the channels into histogram ...
		if (pulsesCollection != 0)
		{
			for (LCCollectionVec::iterator pulseIter = pulsesCollection->begin() ;
			        pulseIter < pulsesCollection->end(); pulseIter ++)
			{
				TrackerPulse *pulse = dynamic_cast<TrackerPulse *> (*pulseIter) ;

				const int channel = pulse->getCellID0();

				// fill channel number into histogram
				_pulsesPerChannelHisto->fill (channel);
			}
		}
	}
} // namespace
