#include "TrackHitEfficiencyProcessor.h"

// C++
#include <iostream>

// LCIO
#include <EVENT/LCCollection.h>
#include <EVENT/TrackerHit.h>
#include <EVENT/Track.h>
#include <IMPL/LCCollectionVec.h>
#include <UTIL/LCTOOLS.h>
#include <IMPL/LCFlagImpl.h>

//marlin
#include <marlin/ConditionsProcessor.h>
#include <marlin/Global.h>

//GEAR
#include "gear/TPCModule.h"

//ROOT
#include "TGraph.h"
#include "TH1D.h"

//using namespace tpcconddata;
using namespace lcio ;
using namespace marlin ;
using namespace std;

namespace marlintpc
{

TrackHitEfficiencyProcessor aTrackHitEfficiencyProcessor ;


TrackHitEfficiencyProcessor::TrackHitEfficiencyProcessor() : Processor("TrackHitEfficiencyProcessor")
							   ,_ntracks(0),_tuple(0),_gearParameters(0)
{
	_description = "TrackHitEfficiencyProcessor calculates the mean90 of the hit hcarge spectrum.";


	registerInputCollection(LCIO::TRACK,
			"InputCollectionName" ,
			"Name of the input collection (default: ClupatraTracks)"  ,
			_inputColName ,
			std::string("ClupatraTracks")) ;
}


void TrackHitEfficiencyProcessor::init()
{

	// usually a good idea to
	printParameters() ;
	AIDAProcessor::tree(this);
	_gearParameters= &marlin::Global::GEAR->getTPCParameters();
	_tuple=new TNtuple("TrackHitEfficiencyProcessor","TrackHitEfficiencyProcessor","mod:row:nhits:ntracks");
}

void TrackHitEfficiencyProcessor::processRunHeader(lcio::LCRunHeader* run)
{


	run->parameters().setValue(_processorName + "_revision", "$Rev: 2743 $");

	for (ProcParamMap::iterator i = _map.begin(); i != _map.end(); i++)
	{
		if (! i->second->isOptional() || i->second->valueSet())
		{
			run->parameters().setValue(_processorName + "_" + i->second->name(),
					i->second->value());
		}
	}


}

void TrackHitEfficiencyProcessor::processEvent(lcio::LCEvent * evt)
{
	AIDAProcessor::tree(this);
	// get the input collection from the event
	LCCollection* inputCol = 0;
	try
	{
		inputCol = evt->getCollection(_inputColName) ;
	}

	catch (DataNotAvailableException&)
	{
		// if there is not input collection give a warning to the user
		streamlog_out(MESSAGE2)
            				<< "processEvent: input collection "
            				<< _inputColName << " cannot be found in event " << evt->getEventNumber() << std::endl;
		return;
	}

	// loop the input data
	for (int t=0;t< inputCol->getNumberOfElements();t++)
	{
		lcio::Track* currentTrack = dynamic_cast<lcio::Track*>(inputCol->getElementAt(t));
		_ntracks++;
		const TrackerHitVec hitvec= currentTrack->getTrackerHits();
		for(unsigned int h=0;h<hitvec.size();h++)
		{
			lcio::TrackerHit * hit= dynamic_cast<lcio::TrackerHit*>(hitvec[h]);
			const gear::TPCModule& theModule = _gearParameters->getNearestModule(hit->getPosition()[0],hit->getPosition()[1]);
			int row=theModule.getRowNumber(theModule.getNearestPad(hit->getPosition()[0],hit->getPosition()[1]));
			int module=theModule.getModuleID();
			std::pair<int, int> key(module,row);
			std::map< std::pair<int,int>, int>::iterator countMapiter;
			countMapiter = _hitOnRow.find(key);
			if(countMapiter == _hitOnRow.end())
			  _hitOnRow.insert( std::make_pair(key,1));
			
			countMapiter = _hitOnRow.find(key);
			((*countMapiter).second)++;

		}
	}
}


void TrackHitEfficiencyProcessor::check(lcio::LCEvent*  /*evt*/)
{ 
	// check what is needed here
}


void TrackHitEfficiencyProcessor::end()
{ 
  
  std::map< std::pair<int,int>, int>::iterator mapiter = _hitOnRow.begin();
  std::vector<double> radius,hiteff;
  
  for(; mapiter!=_hitOnRow.end(); mapiter++)
    {
      std::pair<int,int> key = (*mapiter).first;
      _tuple->Fill(key.first,key.second,(*mapiter).second,_ntracks);
      const gear::TPCModule& theModule = _gearParameters->getModule(key.first);
      gear::Vector2D padcenter=theModule.getPadCenter(theModule.getPadIndex(key.second,0));
      radius.push_back(sqrt(padcenter[0]*padcenter[0]+padcenter[1]*padcenter[1]));
      hiteff.push_back((double)((*mapiter).second)/_ntracks*100.);
    }
  
  AIDAProcessor::tree(this);
  TGraph *graphRow=new TGraph(radius.size(),&radius[0],&hiteff[0]);
  graphRow->SetNameTitle("effRow","Hit efficiency on track");
  graphRow->GetXaxis()->SetTitle("r_{row} [mm]");
  graphRow->GetYaxis()->SetTitle("Hit efficiency on track");
  graphRow->Write();

}



}
