#include "TripletCalculationTool.h"
#include <cmath>


namespace marlintpc
{

TripletCalculationTool::TripletCalculationTool(const gear::TPCParameters * tpcParameters, EVENT::TrackerHit * currentHit, EVENT::TrackerHit * previousHit, EVENT::TrackerHit * nextHit, int method):
		_tpcParameters(tpcParameters),  _currentHit(currentHit), _previousHit(previousHit), _nextHit(nextHit)
{
	switch(method)
	{
	case 0:
		calculateRowDistance();
		break;
	case 1:
		calculateXYDistance();
		break;
	}

}

TripletCalculationTool::~TripletCalculationTool(){}

void TripletCalculationTool::calculateRowDistance()
{


		const double * HitPosition= _currentHit->getPosition();
		const double * HitPositionIminus= _previousHit->getPosition();
		const double * HitPositionIplus= _nextHit->getPosition();

		const gear::TPCModule &modulec = _tpcParameters->getNearestModule(HitPosition[0],HitPosition[1]);

		if((modulec.getLocalPadLayout()).getPadLayoutType() == 2)
		{
			gear::Vector2D tempc=modulec.globalToLocal(HitPosition[0],HitPosition[1]);
			gear::Vector2D tempa=modulec.globalToLocal(HitPositionIminus[0],HitPositionIminus[1]);
			gear::Vector2D tempb=modulec.globalToLocal(HitPositionIplus[0],HitPositionIplus[1]);

			TVector3 veca(0.,0.,0.);
			veca.SetPtThetaPhi(tempa[0],M_PI/2.,tempa[1]);
			TVector3 vecb(0.,0.,0.);
			vecb.SetPtThetaPhi(tempb[0],M_PI/2.,tempb[1]);
			TVector3 vecc(0.,0.,0.);
			vecc.SetPtThetaPhi(tempc[0],M_PI/2.,tempc[1]);

			TVector3 vecag(HitPositionIminus[0],HitPositionIminus[1],0.);
			TVector3 vecbg(HitPositionIplus[0],HitPositionIplus[1],0.);
			TVector3 veccg(HitPosition[0],HitPosition[1],0.);


			if ((vecag.Dot(vecag)<=veccg.Dot(veccg))&&(veccg.Dot(veccg)<=vecbg.Dot(vecbg)))
			{
				vecb-=veca;
				Double_t tmpst=veca.Dot(vecb)/vecb.Dot(vecb);
				Double_t tvalue=-tmpst+sqrt(tmpst*tmpst+((vecc.Dot(vecc)-veca.Dot(veca))/vecb.Dot(vecb)));
				Double_t Dphi=vecc.DeltaPhi(veca+tvalue*vecb);
				_tripletXY = Dphi*sqrt(vecc.Dot(vecc));
				_tripletZ = HitPosition[2]-HitPositionIminus[2]-(tvalue*(HitPositionIplus[2]-HitPositionIminus[2]));
			}else if ((vecbg.Dot(vecbg)<=veccg.Dot(veccg))&&(veccg.Dot(veccg)<=vecag.Dot(vecag)))
			{
				TVector3 st=veca;
				veca=vecb;
				vecb=st;
				vecb-=veca;
				Double_t tmpst=veca.Dot(vecb)/vecb.Dot(vecb);
				Double_t tvalue=-tmpst+sqrt(tmpst*tmpst+((vecc.Dot(vecc)-veca.Dot(veca))/vecb.Dot(vecb)));
				Double_t Dphi=vecc.DeltaPhi(veca+tvalue*vecb);
				_tripletXY = Dphi*sqrt(vecc.Dot(vecc));
				_tripletZ  = (HitPosition[2]-HitPositionIplus[2]-(tvalue*(HitPositionIminus[2]-HitPositionIplus[2])));
			}else
			{
				//TODO:
			}
		}else
		{
			//TODO: other layouts needs to be implemented here
		}

}

void TripletCalculationTool::calculateXYDistance()
{
	/*This method was used in older processors.
	 * Only calculates the triplet along xy and not z
	 * Is only implemented for backwards compatibility
	 */

	//P1
		double X1 = _previousHit->getPosition()[0];
		double Y1 = _previousHit->getPosition()[1];
		//double Z1 = _previousHit->getPosition()[2];


		//P2
		double X2 = _currentHit->getPosition()[0];
		double Y2 = _currentHit->getPosition()[1];
		//double Z2 = _currentHit->getPosition()[2];


		//P3
		double X3 = _nextHit->getPosition()[0];
		double Y3 = _nextHit->getPosition()[1];
		//double Z3 = _nextHit->getPosition()[2];


		//a
		double a1 = X3 - X1;
		double a2 = Y3 - Y1;
		//double a3 = Z3 - Z1;


		//a*a
		//double a_square = a1*a1 + a2*a2 + a3*a3;
		double a_square = a1*a1 + a2*a2 ;


		//P2 - P1
		double Pb1 = X2 - X1;
		double Pb2 = Y2 - Y1;
		//double Pb3 = Z2 - Z1;


		//double x_F = (Pb1*a1 + Pb2*a2 + Pb3*a3)/a_square;
		double x_F = (Pb1*a1 + Pb2*a2)/a_square;


		double d1 = X2-X1 - x_F*a1;
		double d2 = Y2-Y1 - x_F*a2;
		//double d3 = Z2-Z1 - x_F*a3;


		//F_i = a_i*x_F + b_i point on line next to P2
		//

		//|d| = z(dxa)
		double d = ( a1*d2 - a2*d1 )/ sqrt(a1*a1+a2*a2);


		double d3P = d * sqrt(2./3.);


		//              //Rescaling

		//		//distance of the middle point to middle between outer points
		//		double ab = 0.5 - sqrt( (a1*x_F)*(a1*x_F) + (a2*x_F)*(a2*x_F) )/
		//		    sqrt( (X3-X1)*(X3-X1) + (Y3-Y1)*(Y3-Y1) );
		//
		//		//Scaling factor S
		//		double S = 0.8182-0.4505*pow(ab,2);
		//
		//		d3P = S*d3P;
		_tripletXY=d3P;
		_tripletZ=0;
}

}
