#ifdef USE_MARLINUTIL

#include "MCParticleViewer.h"

#include "MarlinCED.h"
#include "ColorMap.h"

#include <EVENT/MCParticle.h>

//--- Marlin
//#include "marlin/VerbosityLevels.h"
#include "marlin/ConditionsProcessor.h"
#include "marlin/Global.h"

//---- GEAR ----
#include "gear/TPCModule.h"
#include "gear/PadRowLayout2D.h"
#include "gear/BField.h"

#include "ChannelPosMap.h"

using namespace lcio ;
using namespace marlin ;


//===============================================================================================================================


namespace marlintpc
{

  MCParticleViewer aMCParticleViewer ;

  MCParticleViewer::MCParticleViewer() : Processor("MCParticleViewer") ,
      _nRun(0),
      _nEvt(0)
  {

    // modify processor description
    _description = "MCParticleViewer: visualization of MCParticles and the gear geometry." ;


    // register steering parameters: name, description, class-variable, default value




    registerInputCollection(LCIO::MCPARTICLE,
                            "MCParticle CollectionName" ,
                            "Name of the MCParticle collection"  ,
                            _mcParticleColName ,
                            std::string("MCParticle")) ;



    registerProcessorParameter("ColorScaleMaxADC" ,
                               "ADC value used for the maximum of the color scale" ,
                               _colorScaleMaxADC ,
                               int(32)) ;


    registerProcessorParameter("ColorScheme" ,
                               "color scheme to be used for drawing - see startup log MESSAGEs for options" ,
                               _colorScheme ,
                               int(1)) ;

    registerProcessorParameter("WaitForKeyBoard" ,
                               "true: display one event and wait for 'return' - false: continuously display events" ,
                               _waitForKeyboard ,
                               bool(true)) ;
  }


  void MCParticleViewer::init()
  {

    streamlog_out(DEBUG) << "   init called  "  << std::endl ;

    // usually a good idea to
    printParameters() ;


    MarlinCED::init(this) ;


    // define some colors schemes - one of which is used for displaying tracks
    static unsigned bojum_colors[ncol * nscheme] =
    {
      0x510000, 0x660202, 0x720202, 0x840202, 0x960303, 0xa80303, 0xbc0303,
      0xce0404, 0xe00404, 0xef0404, 0xff0000, 0xfc0505, 0xf91111, 0xf92222,
      0xfc4949, 0xf97777, 0xf98e8e, 0xf9aeae, 0xf7b2b2, 0xf7cdcd,

      0x512401, 0x662d03, 0x7a3602, 0x934204, 0xa54a04, 0xb75103, 0xc65803,
      0xd35e04, 0xe56604, 0xff6e00, 0xf4710c, 0xf4791a, 0xf9842a, 0xf98f3e,
      0xf99d57, 0xf9a768, 0xf9b47f, 0xf9bf93, 0xf9c9a4, 0xf9d2b3,

      0x48014c, 0x610266, 0x700277, 0x93039b, 0xb103ba, 0xc904d3, 0xda04e5,
      0xe604f2, 0xfa00ff, 0xf00ffc, 0xec1df7, 0xef2ff9, 0xeb42f4, 0xec58f4,
      0xed6bf4, 0xf486f9, 0xf59af9, 0xf8b5fc, 0xf4c3f7, 0xf8d6f9,

      0x2d0251, 0x3d026d, 0x4a0284, 0x550399, 0x5f03aa, 0x6903bc, 0x7102cc,
      0x8004e5, 0x9800ff, 0x8e0ef7, 0x9922f9, 0xa134f9, 0xa845f9, 0xb057f9,
      0xbc70f9, 0xbf77f9, 0xc98ef9, 0xd3a4f9, 0xddbbf9, 0xecd6ff,

      0x00004f, 0x020268, 0x03037f, 0x030399, 0x0303b2, 0x0404cc, 0x0404e0,
      0x0404ef, 0x0000ff, 0x0c0cf9, 0x1b1bf9, 0x2a2af9, 0x3939f9, 0x4d4df9,
      0x6363f9, 0x7272f9, 0x8484f9, 0x9898f9, 0xb3b3f9, 0xcacaf9,

      0x01434c, 0x025c68, 0x027382, 0x04899b, 0x0397aa, 0x03abc1, 0x04b9d1,
      0x04cbe5, 0x00d8ff, 0x0cdef9, 0x1bddf7, 0x2ae1f9, 0x3ddff4, 0x59e4f7,
      0x6be9f9, 0x7cebf9, 0x94f0fc, 0xa3edf7, 0xb6f2f9, 0xc7f4f9,

      0x014443, 0x01605f, 0x027f7d, 0x039996, 0x03b2af, 0x01c6c3, 0x04ddda,
      0x04efeb, 0x00ffff, 0x09f9f5, 0x0ef9f5, 0x20f9f6, 0x32fcf9, 0x3ef9f6,
      0x52f9f7, 0x6bf9f7, 0x7ff9f7, 0x95f9f8, 0xb1f9f8, 0xcaf9f9,

      0x016001, 0x027002, 0x027f02, 0x029102, 0x05aa05, 0x05bf05, 0x06d306,
      0x04e504, 0x00ff00, 0x09f909, 0x18f918, 0x2cf92c, 0x43f943, 0x52f952,
      0x63f963, 0x77f977, 0x8bf98b, 0x9ff99f, 0xb3f9b3, 0xcaf9ca,

      0x344701, 0x4b6603, 0x608202, 0x739b04, 0x83b203, 0x96cc04, 0xa7e204,
      0xb1ef04, 0xb6ff00, 0xbaf713, 0xbff725, 0xc5f73b, 0xcbf751, 0xd3f968,
      0xd7f97a, 0xd8f48b, 0xe2f9a2, 0xe1f4ad, 0xe7f7bb, 0xe9f4c8,

      0x565501, 0x727002, 0x898702, 0xa5a303, 0xb7b403, 0xd1cd04, 0xe2df04,
      0xefeb04, 0xffff00, 0xf9f509, 0xf9f618, 0xf9f62a, 0xf7f43b, 0xf9f64a,
      0xf9f759, 0xf9f76b, 0xf9f77c, 0xf9f88b, 0xf9f89f, 0xfcfbbd
    };


    switch (_colorScheme) {
      case Red         :
      case Orange      :
      case Plum        :
      case Violet      :
      case Blue        :
      case LightBlue   :
      case Aquamarine  :
      case Green       :
      case Olive       :
      case Yellow      :

        for(int i = 0 ; i<ncol ; ++i)
          _colors.push_back( bojum_colors[  ( _colorScheme * ncol  ) + i ] ) ;

        break;


      case Dark:

        for(int i = 0 ; i<ncol ; ++i )
          for(int j = 0 ; j<nscheme ; ++j )
            _colors.push_back( bojum_colors[  ( j * ncol  ) + i ] ) ;
        break ;

      case Light:

        for(int i = ncol-1 ; i>=0 ; --i )
          for(int j = 0 ; j<nscheme ; ++j )
            _colors.push_back( bojum_colors[  ( j * ncol  ) + i ] ) ;
        break ;

      case Classic:
      default:
        _colors.push_back( 0xff00ff );
        _colors.push_back( 0x00ff00 );
        _colors.push_back( 0xffff00 );
        _colors.push_back( 0x0000ff );
        _colors.push_back( 0xff00ff );
        _colors.push_back( 0x00ffff );
        _colors.push_back( 0xffffff );

        _colors.push_back( 0xff88ff );
        _colors.push_back( 0x88ff88 );
        _colors.push_back( 0xffff88 );
        _colors.push_back( 0x8888ff );
        _colors.push_back( 0xff88ff );
        _colors.push_back( 0x88ffff );
        _colors.push_back( 0xffffff );
        break ;

    }

    streamlog_out(MESSAGE) << " ---- selected color scheme:  "  << _colorScheme << "\n"
                           << "    possible options: "  <<   "\n"
                           << "      Red          "     << Red          <<   "\n"
                           << "      Orange       "     << Orange       <<   "\n"
                           << "      Plum         "     << Plum         <<   "\n"
                           << "      Violet       "     << Violet       <<   "\n"
                           << "      Blue         "     << Blue         <<   "\n"
                           << "      LightBlue    "     << LightBlue    <<   "\n"
                           << "      Aquamarine   "     << Aquamarine   <<   "\n"
                           << "      Green        "     << Green        <<   "\n"
                           << "      Olive        "     << Olive        <<   "\n"
                           << "      Yellow       "     << Yellow       <<   "\n"
                           << "      Dark         "     << Dark         <<   "\n"
                           << "      Light        "     << Light        <<   "\n"
                           << "      Classic      "     << Classic      <<   "\n"
                           << " -------- " << std::endl ;
    _nRun = 0 ;
    _nEvt = 0 ;

  }

  void MCParticleViewer::processRunHeader(lcio::LCRunHeader* /* run */)
  {

    _nRun++ ;
  }

  void MCParticleViewer::processEvent(lcio::LCEvent * evt)
  {

	MarlinCED::newEvent(this , -1) ;   // -1 suppresses drawing of detector (done here)


    // ---- get geometry from GEAR ----
    _tpcParams = &Global::GEAR->getTPCParameters() ;

    float zAnode = Global::GEAR->getTPCParameters().getMaxDriftLength();

    float tpcOuterR = _tpcParams->getDoubleVal("TPC_outer_radius") ;

    double offx     = _tpcParams->getDoubleVal("TPC_cylinder_centre_c0") ;
    double offy     = _tpcParams->getDoubleVal("TPC_cylinder_centre_c1") ;
    double bField = Global::GEAR->getBField().at(gear::Vector3D(0, 0, 0)).z() ;

    //FIXME: not used: get readout frequency from first module (can this be different for different modules ? )
    //double readOutFrequency = _tpcParams->getModule(0).getReadoutFrequency() ;


    gear::Vector3D offset(offx , offy , 0.) ;


    // --- draw a cylinder for the TPC  ----

    const static int nCorners =       40 ;
    //const static int tpcColor  = 0xf5f300 ;
    const static int tpcColor = 0x88888888 ;
    const static int tpcPhi0  =        0 ;

    // static CED_GeoCylinder geoCylindersANY[] = {
    //   { tpcOuterR , nCorners , tpcPhi0,  zAnode ,   -zAnode , tpcColor  },
    // };
    // ced_geocylinders( sizeof(geoCylindersANY)/sizeof(CED_GeoCylinder), geoCylindersANY );

    static const int fDL = NUMBER_DATA_LAYER; //  first non data layer
    static const unsigned  tpcLayer = fDL + 0 ;

    std::vector<CEDGeoTube> gTV ;
    gTV.push_back(CEDGeoTube(0. , tpcOuterR , nCorners, nCorners , tpcPhi0, tpcPhi0,  zAnode ,   0 , tpcColor , tpcLayer, 1 , 0)) ;
    ced_geotubes(gTV.size() , (CED_GeoTube*) &gTV[0]);
    MarlinCED::add_layer_description("TPC", tpcLayer);
//FIXME: not used:
//    double tpccenter[3];
//    tpccenter[0]=offx;
//    tpccenter[1]=offy;
//    tpccenter[2]=zAnode;
//    double tpcrotate[3]={0};

    //ced_geocylinder_r(2.*tpcOuterR,zAnode,tpccenter, tpcrotate, 40,0x88888888,tpcLayer );
    //========================================================================

    int layer = 2 ;
    MarlinCED::add_layer_description("Modules", layer);

    for (int i = 0 ; i < _tpcParams->getNModules(); i++ )
    {
    	const gear::TPCModule& module =_tpcParams->getModule(i);
    	for (int j =0; j<module.getNRows();j++)
    	{
    		const std::vector< int > pads = module.getPadsInRow(j);
    		for (unsigned int k =0; k<pads.size();k++)
    		{
    			const gear::PadRowLayout2D & moduleLayout= module.getLocalPadLayout ();


//FIXME: not used:
//    			double size[3];
//    			double center[3];
//    			center[0]=moduleLayout.getPadCenter(pads[k])[0];
//    			center[1]=moduleLayout.getPadCenter(pads[k])[1];
//    			center[2]=0;
    			double width,height;
    			if(moduleLayout.getCoordinateType()==gear::PadRowLayout2D::POLAR)
    			{
    				width=moduleLayout.getPadWidth(pads[k]);
    				height=moduleLayout.getPadHeight(pads[k]);
    			}
    			else
    			{
    				height=moduleLayout.getPadWidth(pads[k]);
    				width=moduleLayout.getPadHeight(pads[k]);
    			}

    			double upleft[2]={moduleLayout.getPadCenter(pads[k])[0]+height/2,
    					moduleLayout.getPadCenter(pads[k])[1]-width/2};
    			double downright[2]={moduleLayout.getPadCenter(pads[k])[0]-height/2,
    					moduleLayout.getPadCenter(pads[k])[1]+width/2};
    			double  upright[2]={moduleLayout.getPadCenter(pads[k])[0]+height/2,
    			    					moduleLayout.getPadCenter(pads[k])[1]+width/2};
    			double  downleft[2]={moduleLayout.getPadCenter(pads[k])[0]-height/2,
    			    					moduleLayout.getPadCenter(pads[k])[1]-width/2};

    			gear::Vector2D upleftV = module.localToGlobal(upleft[0],upleft[1]);
    			gear::Vector2D uprightV = module.localToGlobal(upright[0],upright[1]);
    			gear::Vector2D downleftV = module.localToGlobal(downleft[0],downleft[1]);
    			gear::Vector2D downrightV = module.localToGlobal(downright[0],downright[1]);
    			ced_line(upleftV[0]-offx,upleftV[1]-offy,0,uprightV[0]-offx,uprightV[1]-offy,0,0,0.1,0xff00ff);
    			ced_line(downrightV[0]-offx,downrightV[1]-offy,0,uprightV[0]-offx,uprightV[1]-offy,0,0,0.1,0xff00ff);
    			ced_line(upleftV[0]-offx,upleftV[1]-offy,0,downleftV[0]-offx,downleftV[1]-offy,0,0,0.1,0xff00ff);
    			ced_line(downrightV[0]-offx,downrightV[1]-offy,0,downleftV[0]-offx,downleftV[1]-offy,0,0,0.1,0xff00ff);
    			//ced_geobox_ID(size,center,layer,0xff00ff,pads[k]);

    		}

    	}
    }

    if (isFirstEvent()) {   }

    layer = 3 ;
//FIXME: not used:    int size = 10 ;
    int marker = 0 ;

    try
      {

        LCCollection* mcCol = evt->getCollection(_mcParticleColName) ;

        MarlinCED::add_layer_description(_mcParticleColName, layer);


        int ml = marker | (layer << CED_LAYER_SHIFT) ;

        int nData = mcCol->getNumberOfElements()  ;

        for (int i = 0; i < nData ; i++)
          {

            MCParticle* mcp = dynamic_cast<MCParticle*>(mcCol->getElementAt(i)) ;

            MarlinCED::drawHelix(bField , mcp->getCharge(),
            		mcp->getVertex()[0]-offx, mcp->getVertex()[1]-offy, mcp->getVertex()[2],
					mcp->getMomentum()[0], mcp->getMomentum()[1], mcp->getMomentum()[2],
											 ml , 3 ,  _colors[ i % _colors.size()] ,
                                             0,  tpcOuterR + 1000.,  //_tpcParams->getPlaneExtent()[1] ,
                                             zAnode + 100., 0) ;

          }
      }
    catch (lcio::DataNotAvailableException& e)
      {

        streamlog_out(DEBUG) << " no hit collection in event .... " << std::endl ;
      }



       //=======================================================================================

    streamlog_out(MESSAGE) << "   processing event: " << evt->getEventNumber()
    << "   in run:  "          << evt->getRunNumber()
    <<  std::endl ;



    //++++++++++++++++++++++++++++++++++++
    MarlinCED::draw(this , _waitForKeyboard) ;
    //++++++++++++++++++++++++++++++++++++

    _nEvt ++ ;
  }

  void MCParticleViewer::check(lcio::LCEvent *  /*evt*/)
  {
    // nothing to check here - could be used to fill checkplots in reconstruction processor
  }


  void MCParticleViewer::end()
  {

    //   std::cout << "MCParticleViewer::end()  " << name()
    //      << " processed " << _nEvt << " events in " << _nRun << " runs "
    //      << std::endl ;

  }
}
#endif // USE_MARLINUTIL
