/*
 * ModuleDistortionCalculatorProcessor.h
 *
 *  Created on: June 6, 2013
 *      Author: amunnich
 */

#ifndef MODULEDISTORTIONCALCULATORPROSECCOR_H
#define MODULEDISTORTIONCALCULATORPROSECCOR_H

#include <marlin/Processor.h>
#include <lcio.h>

#include "ModuleDistortionCalculator.h"


namespace marlintpc
{
  /**
   * Marlin processor for distortion analysis
   * Takes two collections each of type track.
   * The idea is that one is the track with all hits on, so basically the track finding output
   * to provide all relevant hits on all module rows.
   * The other track is used to give the track fit. This could have used only a subset of hits for the fit
   * or different tracking algorithms.
   * This way one can compute the distortions for all hits found on a track with different track fits
   * on those hits.
   * If the HitCollection is not set, the hits of the collection assigned to the track given for the fit are used,
   * which is the standard way.
   * The output is written in a text file for each coordinate choosen (row,xy and z).
   * Eventually this should go into a conditionsdata object.
   */
  class ModuleDistortionCalculatorProcessor : public marlin::Processor
  {
  public:

    virtual Processor*  newProcessor() { return new ModuleDistortionCalculatorProcessor; }
    
    ModuleDistortionCalculatorProcessor();
    
    virtual void init();
    
    virtual void processRunHeader(lcio::LCRunHeader* run);
    
    virtual void processEvent(lcio::LCEvent* evt);
    
    virtual void check(lcio::LCEvent* evt);
    
    virtual void end();
    
    void WriteDistortionFiles(std::string filename,std::map< std::pair<int,int>, std::pair<double,double> > results);
    void WriteRootFile();
    
  protected:
    /* the place for protected and private member data and functions */
    std::string _inputColNameTrackHits; ///< Name of the input collection for the hits
    std::string _inputColNameTrackFit; ///< Name of the input collection for the track parameters

  private:
    int _usedEvents;
    int _fillCount;
    std::string _distortionXYOutputFileName;
    std::string _distortionRowOutputFileName;
    std::string _distortionZOutputFileName;
    std::string _rootOutputFileName;
    
    ModuleDistortionCalculator * _myModuleDistortionCalculator;
    
    /**
     * Histograms containing distances for each module and row
     */
    std::map< std::pair<int,int>, TH1*> _HistogramsXY;
    std::map< std::pair<int,int>, TH1*> _HistogramsRow;
    std::map< std::pair<int,int>, TH1*> _HistogramsZ;
 
   /**
     * Maps containing the results of the fitted histogram
     */
    std::map< std::pair<int,int>, std::pair<double,double> >  _ResultsXY;
    std::map< std::pair<int,int>, std::pair<double,double> >  _ResultsRow;
    std::map< std::pair<int,int>, std::pair<double,double> >  _ResultsZ;
 
    const gear::TPCParameters* _GearParameters;
  };
}

#endif //MODULEDISTORTIONCALCULATORPROSECCOR_H
