#ifndef PRFHitBuilderPROCESSOR_H
#define PRFHitBuilderPROCESSOR_H

#include <marlin/Processor.h>
#include <marlin/Global.h>
#include <lcio.h>
#include <string>


// LCIO:
#include "EVENT/TrackerHit.h"
#include "EVENT/Track.h"

#include "TPCConditionsListener.h"

#include "PadResponseFunctionGEM.h"

#include "TNtuple.h"



namespace marlintpc {

  /** A processor to improve the hit ionformation based on information from a pad response function (PRF).
   *  Improvements made:
   *  - position of the hit
   *  - charge correction if hit containes dead channel or pulse in overrange
   *  Additionallly hit errors are prepared for track finding according to
   *  - error contribution due to diffusion
   *  - error contribution due to hit quality
   *
   * @author A.Muennich, DESY
   */

  class PRFHitBuilderProcessor:public marlin::Processor 
  {
  public:
    virtual Processor* newProcessor() 
    {
      return new PRFHitBuilderProcessor;
    }
    
    PRFHitBuilderProcessor();
    
    virtual void init();
    
    virtual void processRunHeader(EVENT::LCRunHeader* run);
    
    virtual void processEvent(EVENT::LCEvent* evt);
    
    virtual void check(EVENT::LCEvent* evt);
    
    virtual void end();

    
  protected:
    const gear::TPCParameters* _gearTPC ;
    std::string  _inputColName,_outputColName,  _inputTPCConditionsCollectionName;
    TPCConditionsListener *_tpcConditionsListener;
   
    //PRF tool
    PadResponseFunctionGEM *_PadResponseFunctionGEM ;
    ///Switch to disable the fitting, just do the calculation of a correction (like in MultiFit)
    int _PRConly;
    //PRF parameters
    double _defocussing, _diffusion;
    //hit error diffusion parameters
    double _offsetx,_slopex,_offsetz,_slopez;
    //count how often PRF was actually used
    int _PRF_used,_PRF_notused,_PRF_failed;
    TNtuple *_testTuple;
  };

}

#endif
