#ifndef MARLINTPC_SIGNAL_SPLITTER_H
#define MARLINTPC_SIGNAL_SPLITTER_H

#include <vector>
#include <iterator>

namespace marlintpc
{
/**
 *  The signal splitter sparates signals of overlapping pulses.
 *  In a first step it seaches for local minima. Afterwards it unfolds
 *  the signals assuming a gaussian shape per pulse.
 *
 *  Please use the testSignalSplitter programme from the validation
 *  to see how it works.
 */

class SignalSplitter
{
    public:
        /// The split function takes a std::vector of the signal to be split.
        /// The return value is a vector of pairs, where the first value
        /// in the pair is the position of the split pulse in the original pulse
        /// (first bin of the split pulse). The second value in the pair is the
        /// split pulse.
  std::vector< std::pair< size_t, std::vector<float> > > splitSignal(std::vector<float> const &, double noise=0);

    protected:
        //    std::vector< std::pair< size_t, std::vector<float> > > splitSignalTwoPeaks( std::vector<float> const & );
        void splitSignalTwoPeaks(std::vector<float> const &);
        void splitSignalAtMinima(std::vector<float> const &, double noise=0);

        /// search for maxima and return a vector of iterators in the original signal
        std::vector< size_t >    findMaxima(std::vector<float> const &, double noise=0);

        std::vector<size_t> _maxima;
        std::vector< std::pair< size_t, std::vector<float> > > _splitSignals;
	double _noiseThreshold;
};
}



#endif //MARLINTPC_SIGNAL_SPLITTER_H
