#include "SimplePadResponseCorrectionProcessor.h"
#include <iostream>
#include <string>

//GEAR
#include <gear/TPCParameters.h>
#include <gear/TPCModule.h>

// include what you need from lcio
#include <EVENT/LCEvent.h>
#include <IMPL/LCCollectionVec.h>
#include <IMPL/TrackerHitImpl.h>
#include <IMPL/LCFlagImpl.h>
#include <Exceptions.h>

//MARLIN
#include <marlin/Global.h>

using namespace lcio;
using namespace marlin;
using namespace std;


namespace marlintpc
{

  SimplePadResponseCorrectionProcessor aSimplePadResponseCorrectionProcessor;


  SimplePadResponseCorrectionProcessor::SimplePadResponseCorrectionProcessor() : Processor("SimplePadResponseCorrectionProcessor")
  {
    _description = "SimplePadResponseCorrectionProcessor corrects the actual position of a hit on the pad according to a formula; since the position is inaccurate if too few pads are hit.";

    registerInputCollection(LCIO::TRACKERHIT,
                            "InputCollectionName" ,
                            "Name of the input TrackerHit collection, default: TPCHits"  ,
                            _inputCollectionName ,
                            string("TPCHits")) ;

    registerOutputCollection(LCIO::TRACKERHIT,
                             "OutputCollectionName" ,
                             "Name of the output TrackerHit collection, default: TPCCorrectedHits"  ,
                             _outputTrackCollectionName ,
                             string("TPCCorrectedHits")) ;


    registerProcessorParameter("WriteOutputToStorage",
                               "If true the output collection is written to file",
                               _outputIsPersistent,
                               true);
  }



  void SimplePadResponseCorrectionProcessor::init()
  {
    streamlog_out(DEBUG) << " init() called  " << endl;

    // you might want to list the used parameters by invoking
    printParameters();
  }



  void SimplePadResponseCorrectionProcessor::processRunHeader(lcio::LCRunHeader*  /*run*/)
  {
  }



  void SimplePadResponseCorrectionProcessor::processEvent(lcio::LCEvent* evt)
  {
    // use an LCCollectionVec instead of LCCollection so we can use iterators later
    LCCollectionVec* inputCollectionVector = 0;

    // create the output collection with the right name and attributes
    LCFlagImpl trkFlag(0) ;
    trkFlag.setBit(LCIO::TRBIT_HITS) ;

    LCCollectionVec* outputHitCollection = new LCCollectionVec(LCIO::TRACKERHIT);
    outputHitCollection->setTransient(!_outputIsPersistent);
    outputHitCollection->setFlag(trkFlag.getFlag());

    // do a try / catch in case there is an empty event
    try
      {
        inputCollectionVector = dynamic_cast<LCCollectionVec *>(evt->getCollection(_inputCollectionName));
      }
    catch (DataNotAvailableException &e)
      {
        // no input data, just return and process the next event
        return;
      }

    // loop over the input hit collection
    for (LCCollectionVec::iterator aHit = inputCollectionVector->begin(), endHit = inputCollectionVector->end();  aHit < endHit; ++aHit)
      {
        // dynamic cast the input object to a tracker hit
        TrackerHit* theHit = dynamic_cast<TrackerHit *>(*aHit);

        // access the geometry: the module; the main parameter for the geometry description are the gear TPCParameters
        //~ const gear::TPCParameters& tpcParameters = Global::GEAR->getTPCParameters();
        //~ const gear::TPCModule& module             = tpcParameters.getModule(theHit->getCellID1());

        // find the pad the hit is on; coordinate system is cartesian, the units are (should be) mm
        //~ int thePadIndex = module.getNearestPad(theHit->getPosition()[0], theHit->getPosition()[1]);

        // as long as it doesn't do anything, this will generate a warning during compilation -- thus commented out
        //~ float padWidth = module.getPadWidth(thePadIndex);
        //~ float padCenter[2] = { module.getPadCenter(thePadIndex)[0] , module.getPadCenter(thePadIndex)[1] };

        // the covariance matrix of the hit. LOWER triangle matrix (!)
        FloatVec theInitialCovarianceMatrix = theHit->getCovMatrix();

        // the output part  ! please set
        TrackerHitImpl* theNewHit = new TrackerHitImpl();
        //theNewHit->setPosition( .. );
        //theNewHit->setCovMatrix( .. );
        //theNewHit->setEDep( .. );
        //theNewHit->setEDepError( .. );
        //theNewHit->setCellID0();
        //theNewHit->setCellID1();
        //theNewHit->setQuality();
        outputHitCollection->addElement(theNewHit);
      }

    // final addition
    evt->addCollection(outputHitCollection, _outputTrackCollectionName);
  }



  void SimplePadResponseCorrectionProcessor::check(lcio::LCEvent*  /*evt*/)
  {
  }



  void SimplePadResponseCorrectionProcessor::end()
  {
  }

} // close namespace brackets
