/*
 * PhotoelectricClusterTools.cc
 *
 *  Created on: Apr 24, 2014
 *      Author: zenker
 */

#include "PhotoelectricClusterTools.h"

// std
#include <iostream>

// Marlin
#include <marlin/Global.h>
#include <marlin/Processor.h>

// gear
#include "gear/TPCModule.h"

// Minuit
#include <Minuit2/MnPrint.h>
#include <Minuit2/MnUserParameters.h>
#include <Minuit2/MnUserCovariance.h>
#include <Minuit2/MnMinimize.h>
#include <Minuit2/FunctionMinimum.h>


using namespace std;
using namespace CLHEP;
using namespace lcio;
using namespace gear;
using namespace ROOT;
using namespace Minuit2;

namespace marlintpc{

CLHEP::Hep2Vector calculateSeed(const std::set<lcio::TrackerPulse*> &pulses, const gear::TPCParameters& tpc_parameters, TrackerHitImpl *seed_photodot){
  Hep2Vector seed_position;
  double total_charge = 0.;
  for (set<TrackerPulse*>::const_iterator i_pulse = pulses.begin();
      i_pulse != pulses.end(); i_pulse++) {

    Vector2D pad_centre =
        tpc_parameters.getModule((*i_pulse)->getCellID1()).getPadCenter(
            (*i_pulse)->getCellID0());
    // if global system is not caretsian switch to cartesian!
    if (tpc_parameters.getCoordinateType() == PadRowLayout2D::POLAR) {

      // convert to Cartesian....
      const double r = pad_centre[0];
      pad_centre[0] = r * cos(pad_centre[1]);
      pad_centre[1] = r * sin(pad_centre[1]);
    }

    seed_position += (*i_pulse)->getCharge()
        * Hep2Vector(pad_centre[0], pad_centre[1]);
    total_charge += (*i_pulse)->getCharge();

    // add the TrackerPulse to the photodot....
    if(seed_photodot)
      seed_photodot->rawHits().push_back((*i_pulse));
  }

  seed_position *= 1. / total_charge;
  double position[3];
  position[0] = seed_position[0];
  position[1] = seed_position[1];
  position[2] = 0.;

  // calculate the error in row direction
//  Hep2Vector myUncertainty(0,0);
  // uncertainty from the pad width in x myUncertainty[0] and y myUncertainty[1] plus uncertainty from the charge in x myUncertainty[2] and y myUncertainty[3]
  double myUncertainty[4] = {0,0,0,0};

  for (set<TrackerPulse*>::const_iterator i_pulse = pulses.begin();
       i_pulse != pulses.end(); i_pulse++) {
    int module_id = (*i_pulse)->getCellID1();
    int pad_id = (*i_pulse)->getCellID0();
    Vector2D pad_centre =  tpc_parameters.getModule(module_id).getPadCenter(pad_id);
    // this is in local coordinates!
    double pad_width[2] = {tpc_parameters.getModule(module_id).getPadHeight(pad_id),tpc_parameters.getModule(module_id).getPadWidth(pad_id)};
    if (tpc_parameters.getModule(module_id).getLocalPadLayout().getCoordinateType() == PadRowLayout2D::POLAR){
      double r = tpc_parameters.getModule(module_id).globalToLocal(pad_centre[0],pad_centre[1])[0];
      pad_width[1] = 2*r*sin(pad_width[1]/2);
    }
    // if global system is not caretsian switch to cartesian!
    if (tpc_parameters.getCoordinateType() == PadRowLayout2D::POLAR) {
      // convert to Cartesian....
      const double r = pad_centre[0];
      pad_centre[0] = r * cos(pad_centre[1]);//x
      pad_centre[1] = r * sin(pad_centre[1]);//y

    }
//    error += pow((*i_pulse)->getCharge()*tpc_parameters.getModule((*i_pulse)->getCellID1()).getPadWidth((*i_pulse)->getCellID0())/total_charge,2)/12
//      + pow((total_charge-(*i_pulse)->getCharge())*((*i_pulse)->getCovMatrix())[0]*pad_centre[0]/(total_charge*total_charge),2);
    for(int i=0;i<2;i++){
      myUncertainty[i] += pow(((*i_pulse)->getCharge()*pad_width[i])/total_charge,2)/12;
      myUncertainty[i+2] += pow((total_charge*pad_centre[i]-seed_position[i]*total_charge)*((*i_pulse)->getCovMatrix())[0]/(total_charge*total_charge),2);
    }
//    myUncertainty += (total_charge*Hep2Vector(pad_centre[0], pad_centre[1])-(seed_position*total_charge))/(total_charge*total_charge)
//        *(total_charge*Hep2Vector(pad_centre[0], pad_centre[1])-(seed_position*total_charge))/(total_charge*total_charge)
//        *((*i_pulse)->getCovMatrix())[0]
//        +((*i_pulse)->getCharge()*tpc_parameters.getModule((*i_pulse)->getCellID1()).getPadWidth((*i_pulse)->getCellID0())/total_charge)
//        *((*i_pulse)->getCharge()*tpc_parameters.getModule((*i_pulse)->getCellID1()).getPadWidth((*i_pulse)->getCellID0())/total_charge)/12;
  }
  cout <<"Set error" << myUncertainty[0] << " " << myUncertainty[1] << endl;
  float cov[4] = { float(myUncertainty[0]), float(myUncertainty[1]), float(myUncertainty[2]), float(myUncertainty[3]) };
  if(seed_photodot != 0){
    seed_photodot->setPosition(position);
    seed_photodot->setEDep(total_charge);
    seed_photodot->setCovMatrix(cov);
  }
  streamlog_out(MESSAGE) << "Seed position is " << position[0] << " : " << position[1] << endl;
  return seed_position;
}

 bool doLikelihoodFit(const TrackerHit &seed, TrackerHitImpl* fit_result, double sigma, bool fitSigma){
  if((int) seed.getRawHits().size() == 0){
    return false;
  }
  const double *seed_position = seed.getPosition();

  if(seed.getRawHits().size() == 1){
    // set the position
    double position[3];
    position[0] = seed_position[0];
    position[1] = seed_position[1];
    position[2] = seed_position[2];
    fit_result->setPosition(position);

    // set the covariance
    float covariance[6];
    covariance[0] = 0.;
    covariance[1] = 0.;
    covariance[2] = 0.;
    covariance[3] = 0.;
    covariance[4] = 0.;
    covariance[5] = 0.;
    fit_result->setCovMatrix(covariance);
    return true;
  }

  streamlog_out(DEBUG) << "Fitting photodot with seed position (" << fit_result->getPosition()[0] << ", " << fit_result->getPosition()[1] << ")." << endl;
  PhotodotLikelihood photodot_likelihood(seed);

  MnUserParameters user_parameters;

  user_parameters.Add("x", seed_position[0], 1.E-01);
  user_parameters.Add("y", seed_position[1], 1.E-01);
  if(fitSigma) {
    // fit also sigma with a given limit.
    user_parameters.Add("sigma", sigma,1.E-01,  0 , 5);
  } else {
    user_parameters.Add("sigma", sigma);
  }

  MnMinimize minimizer(photodot_likelihood, user_parameters);

  FunctionMinimum minimum = minimizer();
//  cout << minimum << endl;
  // store the reconstructed position
  const MnUserParameters &fit_parameters = minimum.UserParameters();
  double fit_position[3];
  float fit_error[3];
  fit_error[0] = fit_parameters.Error("x");
  fit_error[1] = fit_parameters.Error("y");
  fit_error[2] = fit_parameters.Error("sigma");

  fit_position[0] = fit_parameters.Value("x");
  fit_position[1] = fit_parameters.Value("y");
  fit_position[2] = fit_parameters.Value("sigma");
  fit_result->setPosition(fit_position);
  fit_result->setCovMatrix(fit_error);
  fit_result->setEDep(seed.getEDep());
  for(std::vector<LCObject*>::const_iterator it = seed.getRawHits().begin(); it != seed.getRawHits().end(); it++){
    TrackerPulse* pulse = dynamic_cast<TrackerPulse*>(*it);
    if(pulse){
      fit_result->rawHits().push_back(pulse);
    }
  }

//  if (minimum.HasCovariance()){
//    // store the covariance matrix
//    // cov(x,x) , cov(y,x) , cov(y,y) , cov(z,x) , cov(z,y) , cov(z,z)
//    const MnUserCovariance &fit_covariance = minimum.UserCovariance();
//    float covariance[6];
//    covariance[0] = fit_covariance(0, 0);
//    covariance[1] = fit_covariance(0, 1);
//    covariance[2] = fit_covariance(1, 1);
//    covariance[3] = 0.;
//    covariance[4] = 0.;
//    covariance[5] = fit_parameters.Value("sigma");
//    fit_result->setCovMatrix(covariance);
//  }


  return minimum.IsValid();
}

PhotodotLikelihood::PhotodotLikelihood(const TrackerHit &photodot_data): _photodot_data(photodot_data)
{

}


double PhotodotLikelihood::operator()(const vector<double>& x) const
{

  // the parameters are global x, global y, and sigma
  const double sigma = x[2];

  // get the TPC parameters
  const gear::TPCParameters &tpc_parameters = marlin::Global::GEAR->getTPCParameters();

  double negative_log_likelihood = 0.;

  float c0 = 0.;
  float c1 = 0.;

  // convert the parameters to the global coordinate system of the TPC
  if (tpc_parameters.getCoordinateType() == PadRowLayout2D::CARTESIAN){
    c0 = x[0];
    c1 = x[1];
  } else if (tpc_parameters.getCoordinateType() == PadRowLayout2D::POLAR){
    c0 = sqrt(x[0] * x[0] + x[1] * x[1]);
    c1 = atan2(x[1], x[0]);
  }

  // iterate through the pads calculating how much charge they were expected to collect
  const LCObjectVec &pulses = _photodot_data.getRawHits();

  double total_charge = 0.;

  for (LCObjectVec::const_iterator i_data_pulse = pulses.begin(); i_data_pulse != pulses.end(); i_data_pulse++){

    const TrackerPulse *pulse = dynamic_cast< TrackerPulse* >(*i_data_pulse);

    const TPCModule &module = tpc_parameters.getModule(pulse->getCellID1());

    Vector2D local_photodot_coords = module.globalToLocal(c0, c1);

    Vector2D global_pad_centre = module.getPadCenter(pulse->getCellID0());

    Vector2D local_pad_centre = module.globalToLocal(global_pad_centre[0], global_pad_centre[1]);

    double vx;      // the centre of the dot, relative to the pad centre
    double vy;      // the centre of the dot, relative to the pad centre

    double half_pad_width = 0.5 * module.getPadWidth(pulse->getCellID0());
    double half_pad_height = 0.5 * module.getPadHeight(pulse->getCellID0());

    // convert the result to cartesian coordinates
    switch (module.getLocalPadLayout().getCoordinateType()){
      case PadRowLayout2D::POLAR:
        {

          float r = local_photodot_coords[0];
          local_photodot_coords[0] = r * cos(local_photodot_coords[1]);
          local_photodot_coords[1] = r * sin(local_photodot_coords[1]);

          r = local_pad_centre[0];
          half_pad_width *= r;
          const double sin_p = sin(local_pad_centre[1]);
          const double cos_p = cos(local_pad_centre[1]);

          local_pad_centre[0] = r * cos_p;
          local_pad_centre[1] = r * sin_p;

          // rotate the displacement vector so the pad height is aligned with the local y-axis
          const double vxp = local_photodot_coords[0] - local_pad_centre[0];
          const double vyp = local_photodot_coords[1] - local_pad_centre[1];

          vx = (vxp * sin_p) - (vyp * cos_p);
          vy = (vyp * sin_p) + (vxp * cos_p);
        }
        break;

      case PadRowLayout2D::CARTESIAN:
        {

        vx = local_photodot_coords[0] - local_pad_centre[0];
        vy = local_photodot_coords[1] - local_pad_centre[1];

        }
        break;

      default:
        throw gear::Exception("bad layout type");
    }


    // now perform the integral
    const double d = M_SQRT1_2 / sigma;
    const double A = erf((vx + half_pad_width) * d) - erf((vx - half_pad_width) * d);
    const double B = erf((vy + half_pad_height) * d) - erf((vy - half_pad_height) * d);
    const double expected_charge = 0.25 * A * B;

    negative_log_likelihood -= std::log(expected_charge) * pulse->getCharge();

    total_charge += pulse->getCharge();
  }

  return negative_log_likelihood;
}


double PhotodotLikelihood::Up() const
{
  return 0.5;
}

}


