#ifndef GridPixPixelClusterProcessor_h
#define GridPixPixelClusterProcessor_h 1

#include "GridPixXrayObject.h"

//c++
#include <string>
#include <sstream>
#include <vector>
#include <map>
#include <list>
#include <cmath>

//lcio
#include <lcio.h>
#include <EVENT/LCCollection.h>
#include <EVENT/LCRelation.h>
#include <EVENT/TrackerHit.h>
#include <EVENT/TrackerData.h>
#include <IMPL/LCCollectionVec.h>
#include <IMPL/LCRelationImpl.h>
#include <IMPL/TrackerHitImpl.h>
#include <IMPL/TrackerDataImpl.h>

//marlin
#include <marlin/Global.h>
#include <marlin/Processor.h>

//gear
#include <gear/GEAR.h>
#include <gear/TPCModule.h>
#include <gear/TPCParameters.h>
#include <gear/PadRowLayout2D.h>

//aida
#include <AIDA/AIDA.h>
#include <marlin/AIDAProcessor.h>

//root
#include <TH1D.h>
#include <TH2D.h>

namespace marlintpc{

class CompareByCharge
{

	std::map<int,int>* _chargeMap;

public:

	CompareByCharge(std::map<int,int>* chargeMap) : _chargeMap(chargeMap) {}

	bool operator()(int firstCellID0, int secondCellID0) const
	{
		return (*_chargeMap)[firstCellID0] < (*_chargeMap)[secondCellID0];
	}

};


class GridPixPixelClusterProcessor : public marlin::Processor
{

public:

	virtual Processor* newProcessor(){ return new GridPixPixelClusterProcessor; }

	GridPixPixelClusterProcessor();

	~GridPixPixelClusterProcessor();

	virtual void init();

	virtual void processRunHeader( lcio::LCRunHeader* run );

	virtual void processEvent( lcio::LCEvent* evt );

	virtual void check( lcio::LCEvent* evt );

	virtual void end();

protected:

	std::string _inputCol;

	std::string _outputHits;
	std::string _outputData;

	int _outputIsTransient;

	double _maxSharedFraction;

	int _minSharingCharge;
	int _minChargeOnPixel;

	int _minNoPixel;

	std::vector<TH1D*> _histoVec;
	std::vector<TH2D*> _scatterVec;

	int _nColumn;
	int _nRow;

};//class GridPixPixelClusterProcessor

}//namespace marlintpc

#endif
