#ifndef GridPixXrayObject_h
#define GridPixXrayObject_h 1

#include "iostream"
#include "string"

#include "lcio.h"
#include "UTIL/LCFixedObject.h"

#define XRAYNINTVals 2
#define XRAYNFLOATVals 0
#define XRAYNDOUBLEVals 6

namespace marlintpc{
  
  class GridPixXrayObject;
  
  class GridPixXrayObject : public UTIL::LCFixedObject<XRAYNINTVals,XRAYNFLOATVals,XRAYNDOUBLEVals> {
    
    public:
      
      //constructor
      GridPixXrayObject();
      
      //copy constructor: needed to interpret LCCollection read from file/database
      GridPixXrayObject(EVENT::LCObject* obj) : UTIL::LCFixedObject<XRAYNINTVals,XRAYNFLOATVals,XRAYNDOUBLEVals>(obj) {}
      
      GridPixXrayObject(const GridPixXrayObject &original) : UTIL::LCFixedObject<XRAYNINTVals,XRAYNFLOATVals,XRAYNDOUBLEVals>(original) {
	
	if( _createdObject )  _obj = new IMPL::LCGenericObjectImpl(*(original._obj));
	
      }
     
      //destructor
      virtual ~GridPixXrayObject();
      
      //class interface
      int getPixels() const;
      
      int getCharge() const;
      
      double getEnergyFromPixels() const;

      double getEnergyFromCharge() const;
      
      double getLikelihood() const;
      
      double getCenterX() const;
      
      double getCenterY() const;
      
      double getRotAngle() const;
      
      
      void setPixels( int value );
      
      void setCharge( int value );
      
      void setEnergyFromPixels( double value );

      void setEnergyFromCharge( double value );
      
      void setLikelihood( double value );
      
      void setCenterX( double value );
      
      void setCenterY( double value );
      
      void setRotAngle( double value );
      
      
      static std::string getRevision();
      
      static std::string getDefaultColName();
      
      void print( std::ostream& os = std::cout ) const;
      
      const std::string getTypeName() const {
	return "GridPixXrayObject";
      }
      
      const std::string getDataDescription() const {
	return std::string("f:Pixels,Charge,Energy,Likelihood,CenterX,CenterY,RotAngle");
      }
      
  };//class GridPixXrayObject
  
  std::ostream &operator<<(std::ostream &os, const GridPixXrayObject &xray);
  
}//namespace marlintpc

#endif
