#ifndef GridPixXrayObjectCalculatorProcessor_h
#define GridPixXrayObjectCalculatorProcessor_h 1

#include "GridPixXrayObject.h"
#include "TimePixMapHandlerProcessor.h"

//c++
#include <string>
#include <vector>
#include <cmath>

//lcio
#include <lcio.h>
#include <EVENT/TrackerHit.h>
#include <IMPL/TrackerHitImpl.h>
#include <EVENT/LCCollection.h>
#include <IMPL/LCCollectionVec.h>
#include <EVENT/TrackerData.h>
#include <IMPL/TrackerDataImpl.h>
#include <IMPL/LCRelationImpl.h>

//marlin
#include <marlin/Processor.h>
#include <marlin/ConditionsProcessor.h>
#include <marlin/Global.h>
#include <TimePixPixelMode.h>
#include <lccd/ConditionsMap.hh>
#include <TimePixMapListener.h>

//gear
#include <gear/GEAR.h>
#include <gear/TPCModule.h>
#include <gear/TPCParameters.h>
#include <gear/PadRowLayout2D.h>

//aida
#include <AIDA/AIDA.h>
#include <marlin/AIDAProcessor.h>

//root
#include <TROOT.h>
#include <TSystem.h>
#include <Minuit2/FCNBase.h>
#include <TFitterMinuit.h>
#include <TMath.h>
#include <TH1D.h>
#include <TH2D.h>


namespace marlintpc{
  
  class excentricity;
  
  class GridPixXrayObjectCalculatorProcessor : public marlin::Processor{
  
    public:
      
      virtual Processor* newProcessor() { return new GridPixXrayObjectCalculatorProcessor; }
      
      GridPixXrayObjectCalculatorProcessor();
      
      ~GridPixXrayObjectCalculatorProcessor();
      
      virtual void init();
      
      virtual void processRunHeader( lcio::LCRunHeader* run );
      
      virtual void processEvent( lcio::LCEvent* evt );
      
      virtual void end();
      
    protected:
      
      std::string _inputCol;
      
      std::string _outputCol;
      
      std::string _outputHits;
      
      std::string _outputXrays;
      
      std::string _modeMapColName;
      std::string _modeMapCorrName;
  
      int _minNoPixels;
      
      double _shutter;
      double _readoutFrequency;
      double _driftVelocity;
      
      bool _multiShutter;
      
      int _outputIsTransient;
      
      tpcconddata::TimePixMap * _timePixMap;
      
      TH1D* _histoRotAngle;
      TH1D* _histoExcentricity;
      
      TH2D* _scatterExcentricity;
  
    private:
      
      TFitterMinuit* _minuit;
      excentricity* _excentricity;
      
      int _numberXrays;
      
  };//class GridPixXrayObjectCalculatorProcessor

  
}//namespace marlintpc

#endif
