#ifndef KALTRACKINGSTRAIGHT_H
#define KALTRACKINGSTRAIGHT_H

#include "KalTrackingTemplate.h"
#include "StraightTrackParameters.h"
#include <TObject.h>

#include <cmath>
#include <TMatrixD.h>

namespace marlintpc
{

/** This is the straight track class, which inherits from AttKalTrack.
 *  Kalman filter algorithm is implemented inside this class.
 *
 *  @author Bo Li, Tsinghua University
 *  @author Keisuke Fujii, KEK
 */
class KalTrackingStraight : public KalTrackingTemplate<StraightTrackParameters>
{
    public:
        KalTrackingStraight(const gear::GearMgr &gearMgr, 
			    unsigned int minTrackHits, 
			    unsigned int maxSkipRows, 
			    double maxDeltaChi2,
				int hitDistanceFactor,
				int hitStartingMode,
			    double vDrift,
				std::string moduleNotUsed);

        /** Get track type: straight line
         */
        inline int      getType()
        {
            return 0;
        }

        inline double   getSlope()
        {
            return _containerTrack->_sv[0][0];
        }

        inline double   getIntercept()
        {
            return _containerTrack->_sv[1][0];
        }

        /** Get the Chi2 increment at one step
         */
        inline double   getDeltaChi2()
        {
            return _containerTrack->_deltaChi2;
        }

    protected:
        /** Add a new site to track and update track parameters
         */
        virtual bool doAddAndFilter(StraightTrackParameters * track, ModuleRowHit hit) ;

		//TODO::
		//this function is not implemented correctly now
		virtual double getTrackTangentAndLayerAngle(StraightTrackParameters *  /*track*/){return 1.;}

	//
  	//virtual StraightTrackParameters * createTrackCandidate( ModuleRowHit hit );
	
	/** Search track starting from hit
         */
        virtual bool searchTrack(ModuleRowHit hit);

       virtual void setLCIOTrackParameters(IMPL::TrackImpl* trk);

        void printStateVector();

	/** Get the deltaChi2 from the track candidate
	 */
	virtual double calculateDeltaChi2(StraightTrackParameters * trackCandidate);

	virtual Intersection extrapolateIntoNextLayer(int targetDistance,
						      bool useFarHit);
	virtual const kaldet::GearTPCMeasLayer* extrapolateIntoNextLayer(int targetDistance);

 private:
	KalTrackingStraight(const KalTrackingStraight& orig);
	KalTrackingStraight & operator=(const KalTrackingStraight&);

	// a friend class for unit testing the protected member functions
	friend class KalTrackingStraightTester;
};

}//namespace
#endif
