#ifndef TPCHit_FixCellIDs_forTracking_h
#define TPCHit_FixCellIDs_forTracking_h 1

#include "marlin/Processor.h"

#include "lcio.h"
#include <string>


using namespace lcio ;
using namespace marlin ;


namespace marlintpc
{

/** Fix in TPC TrackerHits:
 *   - CellID0s from old data files according to ILDCellID0::encoder_string
 *
 * @param CollectionName Name of the TrackerHit collection
 *
 * @author O. Volynets (with input from F. Gaede's FixCellIDs), DESY
 *         modified by Ch. Rosemann, DESY -- move to standard processor
 * @version $Id: $
 */
  
  class TPCHit_FixCellIDs_forTracking : public Processor
    {

        public:

            virtual Processor*  newProcessor()
            {
                return new TPCHit_FixCellIDs_forTracking ;
            }


            TPCHit_FixCellIDs_forTracking() ;

            virtual const std::string & name() const
            {
                return Processor::name() ;
            }

            /** Called at the begin of the job before anything is read.
             * Use to initialize the processor, e.g. book histograms.
             */
            virtual void init() ;

            /** Called for every run.
             */
            virtual void processRunHeader(lcio::LCRunHeader* run) ;

            /** Called for every event - the working horse.
             */
            virtual void processEvent(lcio::LCEvent * evt) ;


            virtual void check(lcio::LCEvent * evt) ;


            /** Called after data processing for clean up.
             */
            virtual void end() ;


        protected:

            /** Input collection name.
             */
            std::string _tpcInputColName ;

            /** output collection name.
             */
            std::string _tpcOutputColName ;

            bool _outputHitsPersistent;
            bool _encodeModuleID;

            int _nRun ;
            int _nEvt ;
    } ;
}
#endif



