#ifndef TPCHit_FixErrors_forTracking_h
#define TPCHit_FixErrors_forTracking_h 1

#include "marlin/Processor.h"

#include "lcio.h"
#include <string>

// MARLINTPC
#include "TPCConditionsListener.h"


using namespace lcio ;
using namespace marlin ;



namespace marlintpc
{
/** Fix in TPC TrackerHits:
 *   - hit uncertainties according to the charge drift in the TPC. The statistical
 *     uncertainties calculated from the pulses are ignored
 *
 * @param CollectionName Name of the TrackerHit collection
 *
 * @author O. Volynets (with input from F. Gaede's FixCellIDs), DESY
 *         changes by Ch. Rosemann, DESY -- move to normal processor!
 *         fixes by A. Muennich, DESY
 * @version $Id: $
 *
 * 140428 C. Kleinwort, DESY: Implemented radial error component, CARTESIAN pad layout (for timepix).
 */

  class TPCHit_FixErrors_forTracking : public Processor
    {

        public:

            virtual Processor*  newProcessor()
            {
                return new TPCHit_FixErrors_forTracking ;
            }


            TPCHit_FixErrors_forTracking() ;

            virtual const std::string & name() const
            {
                return Processor::name() ;
            }

            /** Called at the begin of the job before anything is read.
             * Use to initialize the processor, e.g. book histograms.
             */
            virtual void init() ;

            /** Called for every run.
             */
            virtual void processRunHeader(lcio::LCRunHeader* run) ;

            /** Called for every event - the working horse.
             */
            virtual void processEvent(lcio::LCEvent * evt) ;


            virtual void check(lcio::LCEvent * evt) ;


            /** Called after data processing for clean up.
             */
            virtual void end() ;


        protected:


            /** Input collection name.
             */
            std::string _tpcInputColName ;

            /** output collection name.
             */
            std::string _tpcOutputColName ;

            /** persistency flag
             */
            bool _outputHitsPersistent;

            int _nRun ;
            int _nEvt ;
            /** the name of the collection the TPC conditions are stored with
             */
            std::string _inputTPCConditionsCollectionName;

            /** the standard way to access TPC conditions: the TPCConditions object via listening
            */
            TPCConditionsListener* _tpcConditionsListener;

            /** Values to describe the resolution curve in xy and z in dependence on z.
             * The dependence on the resolution in z follows sigma=sqrt(_offsetx+_slopex*z)
             * where _offsetx represents the crossing point at z=0 with the y axis and the slope
             * depends on diffusion and Neff: _slopex=D_t*D_t/Neff.
             * Same law applies in z, also the slope here is usually a lot smaller D_l<D_t
             */
            double _offsetx;
            double _slopex;
            double _offsetz;
            double _slopez;
	    double _offsetr;
	    double _sloper;
} ;
}
#endif



