#ifndef TRACKFITTERSIMPLEHELIXPROCESSOR_H
#define TRACKFITTERSIMPLEHELIXPROCESSOR_H

#include <marlin/Processor.h>
#include <lcio.h>
#include <string>
#include "TrackFitterFactory.h"
#include <streamlog/streamlog.h>


namespace marlintpc
{
/** A processor which instantiates a TrackFitterSimpleHelix and fits the tracks with it.
 *  @author F. Mueller
 *
 *  <h4>Input - Prerequisites</h4>
 *  \li A collection of track candidates (lcio::Tracks) containing the TrackerHits.<br>
 *  Note: The LCIO::TRBIT_HITS flag has to be set when this collection is created so
 *      the hits are stored in the track.
 *
 *  <h4>Output</h4>
 *  \li Collection of fitted tracks.
 *
 *  @param InputTrackCandidates The the name of the input collection of track candidates (default: TPCSeedTracks)
 *  @param OutputTracks The name of the collection the straight tracks be stored under (default: TPCHelixTracks)
 *  @param SetOutputTransient If not 0 the output collection is set transient (default: 0)
 *
 *  The parameters for the starting values of the fit are optional parameter. If the parameter is given, it is use.
 *  If the parameter is not given the value from the seed track is used.
 *
 *  @param D0Start \c optional Use this start value for D0 instead of the value from the seed track
 *  @param TanLambdaStart \c optional Use this start value for TanLambda instead of the value from the seed track
 *  @param Z0Start \c optional Use this start value for Z0 instead of the value from the seed track
 *  @param OmegaStart \c optional Use this start value for Omega instead of the value from the seed track
 *  @param PhiStart \c optional Use this start value for Phi instead of the value from the seed track
 */


class TrackFitterSimpleHelixProcessor : public marlin::Processor
{
public:

	virtual Processor*  newProcessor() { return new TrackFitterSimpleHelixProcessor; }

	TrackFitterSimpleHelixProcessor();

	virtual void init();

	virtual void processRunHeader(EVENT::LCRunHeader* run);

	virtual void processEvent(EVENT::LCEvent* evt);

	virtual void check(EVENT::LCEvent* evt);

	virtual void end();


protected:
	/* the place for protected and private member data and functions */
	std::string _inputColName; ///< Name of the input collection
	std::string _outputCollectionName; ///< Name of the output collection
	int _outputIsTransient; //< if not 0 the hits collection is set transient

	EVENT::LCParameters * _trackCollectionParameters;

	/// the optional start parameters
	float _d0Start, _phiStart, _omegaStart, _tanLambdaStart, _z0Start;

};
} // namespace marlintpc
#endif // TRACKFITTERSIMPLEHELIXPROCESSOR_H
