#include "TrackFitterLinearRegression.h"
#include "TrackFitterFactory.h"
#include <iostream>
#include <string>
#include <cmath>
#include "LCObjectCopier.h"
#include <streamlog/streamlog.h>

// include what you need from lcio
#include <Exceptions.h>

using namespace lcio;
using namespace marlin;
using namespace std;


namespace marlintpc
{

TrackFitterBase * TrackFitterLinearRegression::getInstance(LCParameters const *)
{
	static TrackFitterLinearRegression theLinearRegressionFitter;
	return &theLinearRegressionFitter;
}
TrackFitterLinearRegression::TrackFitterLinearRegression()
{
	streamlog_out(DEBUG2) << "TrackFitterLinearRegression:: constructor" << std::endl;
}

std::string TrackFitterLinearRegression::getRevision() const
{
	return std::string("$Rev:$");
}


unsigned char TrackFitterLinearRegression::getFitterType() const
{
	return TrackFitterBase::FitterTypes::LINEARREGRESSION;
}

IMPL::TrackImpl * TrackFitterLinearRegression::fitTrack(EVENT::Track const * seedTrack) const
{
	//perform a linear regression in the xy projection
	EVENT::DoubleVec xyRegression=TrackFitterLinearRegression::doRegressionXY(seedTrack);

	//create return track
	IMPL::TrackImpl * fittedTrack=LCObjectCopier::copy(seedTrack);
	fittedTrack->setD0(xyRegression[0]);
	fittedTrack->setPhi(xyRegression[1]);
	fittedTrack->setOmega(xyRegression[2]);

	//perform a linear regression in the sz projection
	EVENT::DoubleVec szRegression=TrackFitterLinearRegression::doRegressionSZ(fittedTrack);

	fittedTrack->setZ0(szRegression[0]);
	fittedTrack->setTanLambda(szRegression[1]);
	float covMatrix[15]={0};
	covMatrix[0]=xyRegression[3]*xyRegression[3];
	covMatrix[2]=xyRegression[4]*xyRegression[4];
	covMatrix[5]=xyRegression[5]*xyRegression[5];
	covMatrix[9]=szRegression[2]*szRegression[2];
	covMatrix[14]=szRegression[3]*szRegression[3];
	fittedTrack->setCovMatrix(covMatrix);
	float refPoint[3]={0,0,0};
	fittedTrack->setReferencePoint(refPoint);

	fittedTrack->setChi2(xyRegression[6] + szRegression[4]);

	return fittedTrack;
}

EVENT::DoubleVec TrackFitterLinearRegression::doRegressionXY(EVENT::Track const * seedTrack) const
{
	double slope ;
	if (_usePhiStart)
		slope =tan(_phiStart);
	else
		slope =tan(seedTrack->getPhi());
	//  slope = 0.;//I am not sure if the linear regression is correct for other cases...
	//cout << "Slope of straight line xy: " << slope << endl;

	TrackerHitVec  inputHitVec=seedTrack->getTrackerHits();

	double mean_x = 0.;
	double mean_y = 0.;
	double mean_x2 = 0.;
	double mean_xy = 0.;
	double mean_y2 = 0.;

	for(unsigned int iHit = 0 ; iHit < inputHitVec.size(); iHit++)
	{
		double pos[2];
		pos[0] = inputHitVec.at(iHit)->getPosition()[0];
		pos[1] = inputHitVec.at(iHit)->getPosition()[1];

		mean_x += pos[0];
		mean_y += pos[1];
		mean_x2 += pos[0] * pos[0];
		mean_xy += pos[0] * pos[1];
		mean_y2 += pos[1] * pos[1];
	}

	double phi_fit;
	double d0_fit;
	double omega_fit = 0.;

	double phiErr_fit;
	double d0Err_fit;
	double omegaErr_fit = 0.;

	double m = ((double)inputHitVec.size() * mean_xy - mean_x * mean_y) /
			((double)inputHitVec.size() * mean_x2 - mean_x * mean_x);
	//  double mprime = ((double)inputHitVec.size() * mean_x2 - mean_x * mean_x)/((double)inputHitVec.size() * mean_xy - mean_x * mean_y);
	double mprime = ((double)inputHitVec.size() * mean_xy - mean_x * mean_y) /
			((double)inputHitVec.size() * mean_y2 - mean_y * mean_y);
	double b = (mean_x2 * mean_y - mean_x * mean_xy) / ((double)inputHitVec.size() * mean_x2 - mean_x * mean_x);
	//  double bprime = (-mean_y2 * mean_x + mean_y * mean_xy) / ((double)inputHitVec.size() * mean_y2 - mean_y * mean_y);
	double bprime = (mean_y2 * mean_x - mean_y * mean_xy) / ((double)inputHitVec.size() * mean_y2 - mean_y * mean_y);

	double chi2 = 0.; 

	//  slope = 0;

	//if slope is smaller than 1, minimize the y-distance of the hits to the track
	if(slope >= -1. && slope <= 1.)
	{

		phi_fit = atan2(m, 1);
		d0_fit = cos(phi_fit) * b;
		/*
	            phi_fit = atan2(((double)inputHitVec.size() * mean_xy - mean_x * mean_y),
	                            ((double)inputHitVec.size() * mean_x2 - mean_x * mean_x));

	            d0_fit = cos(phi_fit) * (mean_x2 * mean_y - mean_x * mean_xy) / ((double)inputHitVec.size() * mean_x2 - mean_x * mean_x);
		 */
	}
	//if slope is larger than 1, minimize the x-distance of the hits to the track
	else
	{
		//Note: no guarantee that this is correct yet
		phi_fit = atan2(-mprime, 1);
		//      if(phi_fit<0) phi_fit = phi_fit + M_PI;
		//else phi_fit = phi_fit - M_PI;
		phi_fit = phi_fit - M_PI / 2.;

		d0_fit = -1 * sin(phi_fit) * bprime;
		/*
	            phi_fit = atan2((mean_y * mean_y - (double)inputHitVec.size() * mean_y2) * (-1.) ,
	                            ((double)inputHitVec.size() * mean_xy - mean_x * mean_y));

	            d0_fit = sin(phi_fit) * (mean_y * mean_xy - mean_x * mean_y2) / (mean_y * mean_y - (double)inputHitVec.size() * mean_y2) * (-1.);
		 */
	}

	double mean_ybmx2 = 0.;
	double mean_ybmx2prime = 0.;
	for(unsigned int iHit = 0 ; iHit < inputHitVec.size(); iHit++)
	{
		double pos[2];
		pos[0] = inputHitVec.at(iHit)->getPosition()[0];
		pos[1] = inputHitVec.at(iHit)->getPosition()[1];

		mean_ybmx2 += (pos[1] - b - m * pos[0]) * (pos[1] - b - m * pos[0]);
		mean_ybmx2prime += (pos[0] - bprime - mprime * pos[1]) * (pos[0] - bprime - mprime * pos[1]);
	}

	double mErr = sqrt(((double)inputHitVec.size() / ((double)inputHitVec.size() - 2.) * mean_ybmx2) /
			((double)inputHitVec.size() * mean_x2 - mean_x * mean_x));
	//  double mprimeErr = mErr/m/m;
	double mprimeErr = sqrt(((double)inputHitVec.size() / ((double)inputHitVec.size() - 2.) * mean_ybmx2prime) /
			((double)inputHitVec.size() * mean_y2 - mean_y * mean_y));
	double bErr = sqrt((mean_x2 * mean_ybmx2 / ((double)inputHitVec.size() - 2.)) / ((double)inputHitVec.size() * mean_x2 - mean_x * mean_x));
	//  double bprimeErr = sqrt( (mean_y2*mean_ybmx2prime/((double)inputHitVec.size()-2.))/((double)inputHitVec.size()* mean_y2 - mean_y*mean_y));
	double bprimeErr = sqrt((mean_y2 * mean_ybmx2prime / ((double)inputHitVec.size() - 2.)) / ((double)inputHitVec.size() * mean_y2 - mean_y * mean_y));

	if(slope >= -1. && slope <= 1.)
	{
		phiErr_fit = mErr / (1 + m * m);
		d0Err_fit = sqrt((sin(phi_fit) * b * phiErr_fit) * (sin(phi_fit) * b * phiErr_fit) + (cos(phi_fit) * bErr) * (cos(phi_fit) * bErr));

		chi2 = ((double)inputHitVec.size() / ((double)inputHitVec.size() - 2.) * mean_ybmx2);
	}
	else
	{
		//Note: no garantee that this is correct...
		phiErr_fit = mprimeErr / (1 + mprime * mprime);
		d0Err_fit = sqrt((cos(phi_fit) * bprime * phiErr_fit) * (cos(phi_fit) * bprime * phiErr_fit) + (sin(phi_fit) * bprimeErr) * (sin(phi_fit) * bprimeErr));

		chi2 = ((double)inputHitVec.size() / ((double)inputHitVec.size() - 2.) * mean_ybmx2prime);
	}

	// cout << "phi_fit: " << phi_fit << " d0_fit: " << d0_fit << endl;

	//cout << "Karimaeki Fit Phi: " << phi_fit <<" +- "<<phiErr_fit<< endl;
	//cout << "Karimaeki Fit D0: " << d0_fit <<" +- "<<d0Err_fit<< endl;

	EVENT::DoubleVec returnVector;
	returnVector.push_back(d0_fit);
	returnVector.push_back(phi_fit);
	returnVector.push_back(omega_fit);
	returnVector.push_back(d0Err_fit);
	returnVector.push_back(phiErr_fit);
	returnVector.push_back(omegaErr_fit);
	returnVector.push_back(chi2);
	return returnVector;


}

EVENT::DoubleVec TrackFitterLinearRegression::doRegressionSZ(EVENT::Track const * seedTrack) const
{
	double slope;
	if (_useTanLambdaStart)
		slope=_tanLambdaStart;
	else
		slope=seedTrack->getTanLambda();
	//  slope = 0.;//I am not sure if the linear regression is correct for other cases...
	// cout << "Slope of straight line sz: " << slope << endl;

	TrackerHitVec inputHitVec=seedTrack->getTrackerHits();
	double mean_x = 0.;
	double mean_y = 0.;
	double mean_x2 = 0.;
	double mean_xy = 0.;
	double mean_y2 = 0.;
	EVENT::DoubleVec pca=TrackFitterLinearRegression::calculatePCA(seedTrack);
	for(unsigned int iHit = 0 ; iHit < inputHitVec.size(); iHit++)
	{
		double pos[2];
		pos[0] = TrackFitterLinearRegression::calculateS(inputHitVec.at(iHit),pca,seedTrack);
		pos[1] = inputHitVec.at(iHit)->getPosition()[2];

		mean_x += pos[0];
		mean_y += pos[1];
		mean_x2 += pos[0] * pos[0];
		mean_xy += pos[0] * pos[1];
		mean_y2 += pos[1] * pos[1];

	}

	double tanl_fit;
	double z0_fit;

	double tanlErr_fit;
	double z0Err_fit;

	double m = ((double)inputHitVec.size() * mean_xy - mean_x * mean_y) /
			((double)inputHitVec.size() * mean_x2 - mean_x * mean_x);
	//double mprime = ((double)inputHitVec.size() * mean_x2 - mean_x * mean_x)/((double)inputHitVec.size() * mean_xy - mean_x * mean_y);
	double mprime = ((double)inputHitVec.size() * mean_xy - mean_x * mean_y) /
			((double)inputHitVec.size() * mean_y2 - mean_y * mean_y);
	double b = (mean_x2 * mean_y - mean_x * mean_xy) / ((double)inputHitVec.size() * mean_x2 - mean_x * mean_x);
	//  double bprime = (-mean_y2 * mean_x + mean_y * mean_xy) / ((double)inputHitVec.size() * mean_y2 - mean_y * mean_y);
	double bprime = (mean_y2 * mean_x - mean_y * mean_xy) / ((double)inputHitVec.size() * mean_y2 - mean_y * mean_y);

	double chi2 = 0.; 

	//if slope is smaller than 1, minimize the y-distance of the hits to the track
	if(slope >= -1. && slope <= 1.)
	{
		tanl_fit = m;
		z0_fit = b;
		/*
            tanl_fit = (((double)inputHitVec.size() * mean_xy - mean_x * mean_y) /
                        ((double)inputHitVec.size() * mean_x2 - mean_x * mean_x));

            z0_fit = (mean_x2 * mean_y - mean_x * mean_xy) / ((double)inputHitVec.size() * mean_x2 - mean_x * mean_x);
		 */
	}
	//if slope is larger than 1, minimize the x-distance of the hits to the track
	else
	{
		tanl_fit = -1. / mprime;
		z0_fit = bprime / mprime;
		/*
            tanl_fit = ((mean_y * mean_y - (double)inputHitVec.size() * mean_y2) * (-1.) /
                        ((double)inputHitVec.size() * mean_xy - mean_x * mean_y));

            z0_fit = tanl_fit * (mean_y * mean_xy - mean_x * mean_y2) / (mean_y * mean_y - (double)inputHitVec.size() * mean_y2) * (-1.);
		 */
	}


	double mean_ybmx2 = 0.;
	double mean_ybmx2prime = 0.;
	for(unsigned int iHit = 0 ; iHit < inputHitVec.size(); iHit++)
	{
		double pos[2];
		pos[0] = TrackFitterLinearRegression::calculateS(inputHitVec.at(iHit),pca,seedTrack);
		pos[1] = inputHitVec.at(iHit)->getPosition()[2];

		mean_ybmx2 += (pos[1] - b - m * pos[0]) * (pos[1] - b - m * pos[0]);
		mean_ybmx2prime += (pos[0] - bprime - mprime * pos[1]) * (pos[0] - bprime - mprime * pos[1]);
	}

	double mErr = sqrt(((double)inputHitVec.size() / ((double)inputHitVec.size() - 2.) * mean_ybmx2) /
			((double)inputHitVec.size() * mean_x2 - mean_x * mean_x));
	//  double mprimeErr = mErr/m/m;
	double mprimeErr = sqrt(((double)inputHitVec.size() / ((double)inputHitVec.size() - 2.) * mean_ybmx2prime) /
			((double)inputHitVec.size() * mean_y2 - mean_y * mean_y));
	double bErr = sqrt((mean_x2 * mean_ybmx2 / ((double)inputHitVec.size() - 2.)) / ((double)inputHitVec.size() * mean_x2 - mean_x * mean_x));
	//  double bprimeErr = sqrt( (mean_y2*mean_ybmx2prime/((double)inputHitVec.size()-2.))/((double)inputHitVec.size()* mean_y2 - mean_y*mean_y));
	double bprimeErr = sqrt((mean_y2 * mean_ybmx2prime / ((double)inputHitVec.size() - 2.)) / ((double)inputHitVec.size() * mean_y2 - mean_y * mean_y));

	if(slope >= -1. && slope <= 1.)
	{
		tanlErr_fit = mErr;
		z0Err_fit = bErr;

		chi2 = ((double)inputHitVec.size() / ((double)inputHitVec.size() - 2.) * mean_ybmx2);
	}
	else
	{
		//Note: no guarantee that this is correct...
		tanlErr_fit = mprimeErr / mprime / mprime;
		z0Err_fit = sqrt((bprime * mprimeErr / mprime / mprime) + (bprimeErr / mprime) * (bprimeErr / mprime));

		chi2 = ((double)inputHitVec.size() / ((double)inputHitVec.size() - 2.) * mean_ybmx2prime);
	}


	// cout << "tanl_fit: " << tanl_fit <<" +- "<<tanlErr_fit<< " z0_fit: " << z0_fit <<" +- "<<z0Err_fit<< endl;
	EVENT::DoubleVec returnVec;
	returnVec.push_back(z0_fit);
	returnVec.push_back(tanl_fit);
	returnVec.push_back(z0Err_fit);
	returnVec.push_back(tanlErr_fit);
	returnVec.push_back(chi2); 
	return returnVec;
}

EVENT::DoubleVec TrackFitterLinearRegression::calculatePCA(EVENT::Track const * seedTrack) const
{
	DoubleVec pca;
	double d0,phi;
	if (_useD0Start)
		d0=_d0Start;
	else
		d0=seedTrack->getD0();
	if (_usePhiStart)
		phi=_phiStart;
	else
		phi=seedTrack->getPhi();

	pca.push_back(-d0*sin(phi));
	pca.push_back(d0*cos(phi));
	return pca;
}

double TrackFitterLinearRegression::calculateS(EVENT::TrackerHit * aHit, EVENT::DoubleVec pca, EVENT::Track const * seedTrack ) const
{
	double s_xy = 0.;

	double phi;
	if (_usePhiStart)
		phi=_phiStart;
	else
		phi=seedTrack->getPhi();

	s_xy = -sqrt((aHit->getPosition()[0] - pca[0]) * (aHit->getPosition()[0] - pca[0]) +
			(aHit->getPosition()[1] - pca[1]) * (aHit->getPosition()[1] - pca[1]));

	if((-1.25 * M_PI <= phi && phi <= -0.75 * M_PI) ||
			(-0.25 * M_PI <= phi && phi <= 0.25 * M_PI) ||
			(0.75 * M_PI <= phi && phi <= 1.25 * M_PI))
	{
		if(pca[0] < aHit->getPosition()[0])
		{
			s_xy = (-1.) * s_xy;
		}
	}
	else
	{
		if(pca[1] < aHit->getPosition()[1])
		{
			s_xy = (-1.) * s_xy;
		}
	}

	return s_xy;

}
} // close namespace brackets
