#ifndef ELECTRONCLOUDGEMAMPLIFICATIONPROCESSOR_H
#define ELECTRONCLOUDGEMAMPLIFICATIONPROCESSOR_H 1

// CLHEP
#include <CLHEP/Vector/ThreeVector.h>

// Marlin
#include <marlin/Processor.h>

#include <lcio.h>
namespace EVENT {

	class LCRunHeader;
	class LCEvent;
}

namespace marlintpc {

/** ElectronCloudGEMAmplificationProcessor : This processor takes a cloud of electrons and passes it through a GEM simulating an electron avalanche
 *
 *    @author J. Abernathy (UVic)<br>
 */
class ElectronCloudGEMAmplificationProcessor : public marlin::Processor {

public:
	/** constructor
	 */
	ElectronCloudGEMAmplificationProcessor();

	/** destructor
	 */
	~ElectronCloudGEMAmplificationProcessor();

	/**
	 * used by Marlin to create a new processor
	 */
	virtual Processor* newProcessor() { return new ElectronCloudGEMAmplificationProcessor() ; }

	/**
	 * Called at the begin of the job before anything is read.
	 * Use to initialize the processor, e.g. book histograms.
	 */
	virtual void init();

	/**
	 * Called for every run
	 */
	virtual void processRunHeader(lcio::LCRunHeader* run );

	/**
	 * Called for every event - the working horse.
	 */
	virtual void processEvent(lcio::LCEvent * evt );

	/**
	 * Called after the event has been processed
	 */
	virtual void check(lcio::LCEvent * evt );

	/**
	 * Called after data processing for clean up.
	 */
	virtual void end();

protected:

private:

	/**
	 * the name of the input cloud collection
	 */
	std::string _input_collection_name;

	/**
	* the name of the output cloud collection from this processor
	 */
	std::string _output_collection_name;

	/** the name to use for the tpc conditions collection
	 */
	std::string _tpc_conditions_collection_name;

	/** a name for this GEM
	 */
	std::string _gem_name;

	/**
	 * the parameterized collection efficiency of the GEM [0..1]
	 */
	double _collection_efficiency;

	/**
	 * the parameterized extraction efficiency of the GEM [0..1]
	 */
	double _extraction_efficiency;

	/**
	 * the parameterized amplification of the GEM
	 */
	double _amplification;

	/**
	 * the parameterized longitudinal defocussing of this GEM (mm)
	 */
	double _longitudinal_defocussing;

	/**
	 * the parameterized transverse defocussing of this GEM (mm)
	 */
	double _transverse_defocussing;

	/**
	 * the width of the GEM (mm)
	 */
	double _width;

};

} // namespace marlintpc

#endif // ELECTRONCLOUDGEMAMPLIFICATIONPROCESSOR_H
