#ifndef SIGNALSHAPERGAUSSIANPROCESSOR_H
#define SIGNALSHAPERGAUSSIANPROCESSOR_H 1

// Marlin
#include <marlin/Processor.h>

namespace EVENT {

	class LCRunHeader;
	class LCEvent;
}

namespace marlintpc {

/** SignalShaperGaussianProcessor : This processor takes parameterized charge signals and turns them into a gaussian shaped signal
 *
 *    @author J. Abernathy (UVic)<br>
 */
class SignalShaperGaussianProcessor : public marlin::Processor {

public:
	/** constructor
	 */
	SignalShaperGaussianProcessor();

	/** destructor
	 */
	virtual ~SignalShaperGaussianProcessor();

	/** used by Marlin to create a new processor
	 */
	virtual Processor* newProcessor() { return new SignalShaperGaussianProcessor() ; }

	/**
	 * Called at the begin of the job before anything is read.
	 * Use to initialize the processor, e.g. book histograms.
	 */
	virtual void init();

	/**
	 * Called for every run
	 */
	virtual void processRunHeader(lcio::LCRunHeader* run );

	/**
	 * Called for every event - the working horse.
	 */
	virtual void processEvent(lcio::LCEvent * evt );

	/**
	 * Called after the event has been processed
	 */
	virtual void check(lcio::LCEvent * evt );

	/**
	 * Called after data processing for clean up.
	 */
	virtual void end();

protected:

private:

	/** the name of the input charge signal collection
	 */
	std::string _input_collection_name;

	/** the name of the output tracker pulse collection from this processor
	 */
	std::string _output_collection_name;

	/** the name of the output channel correction collection for this processor
	 */
	std::string _tpc_channel_correction_collection_name;

	/** the gain of this electronic shaper
	 */
	double _gain;

	/** the polarity of the produced signal (false = negative)
	 */
	bool _polarity;

	/** the linear charge to voltage constant of this shaper
	 */
	double _mV_per_electron;

	/** the amount of time the signal has been induced for
	 */
	double _induction_time;

	/** the width of a signal sample (1/Hz) in ns
	 */
	double _time_resolution;
};

} // namespace marlintpc

#endif // SIGNALSHAPERGAUSSIANPROCESSOR_H
