//LCIO
#include "lcio.h"
#include "IO/LCWriter.h"
#include "EVENT/LCIO.h"
#include "IMPL/LCEventImpl.h"
#include "IMPL/LCGenericObjectImpl.h"
#include "IMPL/LCCollectionVec.h"
#include "IMPL/LCTOOLS.h"
#include "UTIL/LCTime.h"
//LCCD
#include "lccd.h"
#include "lccd/DBInterface.hh"
#include "lccd/LCConditionsMgr.hh"
#include "lccd/SimpleFileHandler.hh"
#include "lccd/IConditionsHandler.hh"
//GEAR
#include "gear/TPCParameters.h"
#include "gear/TPCModule.h"
#include "gearxml/GearXML.h"
#include "gear/GearMgr.h"
#include "gear/GEAR.h"
//TPCCONDDATA
#include "ChannelCorrectionNew.h"
//MARLINTPC
#include "ChannelMappingProcessor.h"
#include "ADCChannelMapping.h"

//C++
#include <ctype.h>
#include <stdlib.h>
#include <iostream>
#include <fstream>
#include <string>
#include <sstream>

void printUsage()
{
    std::cout<<"Usage:"<<std::endl;
    std::cout<<"To write data to a lcio simple file:"<<std::endl;
    std::cout<<"  ChannelCorrectionGenerator channelCalibrationList.dat gearfile.xml channelmapping.slcio [outfile.slcio]" <<std::endl;
    std::cout<<"  (outfile name is optional, default is TPCChannelCorrection.slcio)"<< std::endl;
    std::cout<<"To write data to a database:"<<std::endl;
    std::cout<<"  ChannelCorrectionGenerator channelCalibrationList.dat gearfile.xml channelmapping.slcio connectionString folder startLCIOTimeStamp endLCIOTimeStamp"<<std::endl;
    std::cout<<"Formats:"<<std::endl;
    std::cout<<" Text file: readoutID channelID calibrationOffset calibrationSlope calibrationOffsetError calibrationSlopeError timeOffset deadOrNoisy(\"DEAD\" or \"NOISY\")"  << std::endl;
    std::cout<<" Connection String: hostname:database:login:password[:port]"<<std::endl;
    std::cout<<" LCIO time stamp: nano seconds since 1.1.1970 00:00:00"<<std::endl;
}

void fillCollection(std::filebuf * infile,lcio::LCCollectionVec* correctionCol, gear::TPCParameters const & /*tpcParams*/, marlintpc::ChannelMappingListener * channelMappingListener)
{
	std::istream inputstream(infile);
	int linenumber = -1;
	// read all lines in the file
	while (!inputstream.eof() )
	{
		// read the next line from the file
		std::string line;
		std::getline(inputstream, line);
		if (inputstream.eof())   //eof is reached
		{
			if (!line.empty())
			{
				std::cout << "Ignoring non-empty line at end of file: " << line << std::endl;
			}
			break;
		}
		// the line as a stream so ew can stream into the variables
		std::stringstream linestream(line);
		linenumber++;
		// check if the next line is a comment
		if (linestream.peek() == '#')
		{
			std::cout << "DEBUG: skipping comment line " << linenumber << " : "  << line <<  std::endl;
			continue;
		}
		int moduleID,channel;
		double chargeOffset,chargeSlope,chargeOffsetErr,chargeSlopeErr,timeOffset;
		std::string deadOrNoisy="";
		// stream the data into the variables
		linestream >>  moduleID >> channel >> chargeOffset >> chargeSlope >> chargeOffsetErr >> chargeSlopeErr >> timeOffset >> deadOrNoisy;
		//std::cout << "DEBUG:  reading line " << linenumber << " : "<< line << std::endl;
		// test if everything went fine
		if (linestream.fail())
		{
			std::cout << "error reading line " << linenumber << " : "<< line << std::endl;
			continue;
		}

		//std::cout << "DEBUG: retrieving module "<<moduleID << std::std::endl;
		//gear::TPCModule const & module = tpcParams.getModule( moduleID );
		//std::cout << "DEBUG: got module with id "<< module.getModuleID() << " at "<< &module << std::endl;

		std::pair<int, int> gearMapping = channelMappingListener->getGearMapping(std::make_pair(channel,moduleID));
		tpcconddata::ChannelCorrectionNew * channelCorrection =
				new  tpcconddata::ChannelCorrectionNew(gearMapping.first, gearMapping.second,channel, moduleID );
		//tpcconddata::ChannelCorrectionNew * channelCorrection =
						//new  tpcconddata::ChannelCorrectionNew(pad_index, moduleID, -1, -1);
		channelCorrection->setCalibrationFactor(chargeSlope);
		channelCorrection->setTimeOffset(timeOffset);
		channelCorrection->setCalibrationOffset(chargeOffset);

		if (deadOrNoisy=="DEAD")
		{
			channelCorrection->setIsDead(true);
		}
		else if (deadOrNoisy=="NOISY")
		{
			channelCorrection->setIsNoisy(true);
		}

		correctionCol->addElement( channelCorrection );
	}       //  while (!inputstream.eof() )
}


int main(int argc, char **argv)
{
	bool writeToDatabase=false;
    std::string outFileName;
    lcio::LCWriter* lcWrt=NULL;
    lcio::LCEventImpl*  evt=NULL;
    std::cout << argc<< std::endl;
    if ( argc == 4 || argc == 5 )
    {
    	// get outfile name

        if (argc==5)
            outFileName=argv[4];
        else
            outFileName="TPCChannelCorrection.slcio";
        // open outfile
        lcWrt = lcio::LCFactory::getInstance()->createLCWriter() ;
        lcWrt->open( outFileName, lcio::LCIO::WRITE_NEW ) ;
        // create new lcio event

        evt = new lcio::LCEventImpl() ;
    }
    else if ( argc == 8 )
    {
        writeToDatabase=true;
    }
    else
    {
        printUsage();
        return(1);
    }

    //get channelmapping from the slcio file
    lccd::LCConditionsMgr::instance()->registerHandler( tpcconddata::ADCChannelMapping::getDefaultColName(),  new lccd::SimpleFileHandler(argv[3], tpcconddata::ADCChannelMapping::getDefaultColName(),tpcconddata::ADCChannelMapping::getDefaultColName()) ) ;
    marlintpc::ChannelMappingListener * channelMappingListener = new marlintpc::ChannelMappingListener(tpcconddata::ADCChannelMapping::getDefaultColName());
    if(!channelMappingListener)
    {
    	throw lcio::DataNotAvailableException("Unable to find the channel mapping collection");
    }
    //time value is not used -> inserted just a dummy value
    lccd::LCConditionsMgr::instance()->update(lccd::LCCDTimeStamp(   123456789   ));
    //DEBUG
    for (int i=0; i< argc ; ++i)
    {
        std::cout<<"DEBUG: argument ["<<i<<"]: "<<argv[i]<<std::endl;
    } 
    
    // read pad layout from gear
    gear::GearXML gearXML( argv[2] ) ;
    gear::GearMgr* gearMgr = gearXML.createGearMgr() ;
    gear::TPCParameters const &tpcParams=gearMgr->getTPCParameters();

    // create the collection we want to store the data in
    lcio::LCCollectionVec* correctionCol = new lcio::LCCollectionVec( lcio::LCIO::LCGENERICOBJECT );

    // open the input files
    std::filebuf infileDead;
    std::filebuf infileNoisy;
    infileDead.open(argv[1],std::ios::in);
    fillCollection(&infileDead,correctionCol,tpcParams,channelMappingListener);


    if ( writeToDatabase )
    {
        std::string connectionString=argv[4];
        std::string folder=argv[5];
        lcio::long64 startTimeStamp;
        std::stringstream startstr(argv[6]);
        startstr >> startTimeStamp;
        lcio::long64 endTimeStamp;
        std::stringstream endstr(argv[7]);
        endstr >> endTimeStamp;
        lccd::DBInterface* dbConnection = new lccd::DBInterface(connectionString, folder, true );
        dbConnection->storeCollection( startTimeStamp, endTimeStamp, correctionCol, tpcconddata::ChannelCorrectionNew::getDefaultColName());
        delete dbConnection;
    }
    else
    {
        // add the mapping collection to the event
        evt->addCollection( correctionCol , tpcconddata::ChannelCorrectionNew::getDefaultColName()) ;
        // write the event to the file
        lcWrt->writeEvent( evt ) ;
        // dump the event to the screen
        lcio::LCTOOLS::dumpEvent( evt ) ;
        lcWrt->close() ;
        // ------------ IMPORTANT ------------- !
        // we created the event so we need to delete it ...
        delete evt ;
        // -------------------------------------
    }
    // close the input text file
    infileDead.close();
    infileNoisy.close();
    return 0;
}
