/*
 * TIEOFBlock.hh
 *
 *  Created on: Jul 10, 2012
 *      Author: caiazza
 */

#ifndef TIEOFBLOCK_HH_
#define TIEOFBLOCK_HH_

#include "TIAltroBlock.hh"

//LCIO
#include "lcio.h"
#include "UTIL/LCTime.h"

namespace marlintpc
{
/**
 * Additional interface for end of block objects
 */
class TIEOFBlock : public TIAltroBlock
{
public:
    /**
     * Virtual destructor
     */
    virtual ~TIEOFBlock(){}

    /**
     * Timestamp of the pause
     * @return Timestamp of the pause
     */
    virtual lcio::LCTime GetTimestamp() const = 0;

    /**
     * Number of events collected at the pause
     * @return Number of events collected at the pause
     */
    virtual uint32_t GetNEvents() const = 0;

    /**
     * Get the file number counter
     * @return File index
     */
    virtual uint32_t GetFileNumber() const = 0;
};//End interface

/**
 * Implementation of the End of file class for DAQ version 4.2+
 */
class TEOFBlock_V42 : public TIEOFBlock
{
public:
    TEOFBlock_V42(std::vector<uint32_t>& Content);

    virtual ~TEOFBlock_V42();

    /**
     * Get the block lenght in number of 32 bit words
     * @return number of 32 bit words in the block
     */
    virtual uint32_t GetLength() const;

    /**
     * Verifies that the content of the block matches the expected content of the type of block we build.
     * @return True if the content of the block matches the expectation, False otherwise
     */
    virtual bool CheckIntegrity() const;

    /**
     * Returns a constant reference to the content of the block
     * @return vector of 32 bit data words
     */
    virtual const std::vector<uint32_t>& Content() const;

    /**
     * Returns a data word that identifies the block type
     * @return 32 bit data word identifying the block
     */
    virtual uint32_t GetIdentifier() const;

    /**
     * Timestamp of the pause
     * @return Timestamp of the pause
     */
    virtual lcio::LCTime GetTimestamp() const;

    /**
     * Number of events collected at the pause
     * @return Number of events collected at the pause
     */
    virtual uint32_t GetNEvents() const;

    /**
     * Get the file number counter
     * @return File index
     */
    virtual uint32_t GetFileNumber() const;

    /**
     * Allows to set additional parameters to the RunHeader.
     * @param RunHeader
     */
    virtual void SetAdditionalParameters(lcio::LCRunHeader* RunHeader);

private:
    std::vector<uint32_t> _content;
};//end implementation v4.2

}

#endif /* TIEOFBLOCK_HH_ */
