/*
 * TIEORBlock.hh
 *
 *  Created on: Jul 10, 2012
 *      Author: caiazza
 */

#ifndef TIEORBLOCK_HH_
#define TIEORBLOCK_HH_

#include "TIAltroBlock.hh"

//LCIO
#include "lcio.h"
#include "EVENT/LCRunHeader.h"
#include "UTIL/LCTime.h"

namespace marlintpc
{
class TIEORBlock : public TIAltroBlock
{
public:
    /**
     * Virtual destructor
     */
    virtual ~TIEORBlock(){}

    /**
     * Decodes the timestamp of the end of the run
     * @return LCTime object encoding the end time
     */
    virtual lcio::LCTime GetEndTime() const = 0;

    /**
     * Decodes the number of events collected during the run
     * @return Number of events collected in the run
     */
    virtual uint32_t GetNEvents() const = 0;
};//end interface

/**
 * Implementation of the EOR class for DAQ version 4.2+
 */
class TEORBlock_V42 : public TIEORBlock
{
public:
    /**
     * Standard constructor
     * @param Content
     */
    TEORBlock_V42(std::vector<uint32_t>& Content);
    /**
     * Virtual destructor
     */
    virtual ~TEORBlock_V42();

    /**
     * Get the block lenght in number of 32 bit words
     * @return number of 32 bit words in the block
     */
    virtual uint32_t GetLength() const;

    /**
     * Verifies that the content of the block matches the expected content of the type of block we build.
     * @return True if the content of the block matches the expectation, False otherwise
     */
    virtual bool CheckIntegrity() const;

    /**
     * Returns a constant reference to the content of the block
     * @return vector of 32 bit data words
     */
    virtual const std::vector<uint32_t>& Content() const;

    /**
     * Returns a data word that identifies the block type
     * @return 32 bit data word identifying the block
     */
    virtual uint32_t GetIdentifier() const;

    /**
     * Decodes the timestamp of the end of the run
     * @return LCTime object encoding the end time
     */
    virtual lcio::LCTime GetEndTime() const;

    /**
     * Decodes the number of events collected during the run
     * @return Number of events collected in the run
     */
    virtual uint32_t GetNEvents() const;

    /**
     * Allows to set additional parameters to the RunHeader.
     * @param RunHeader
     */
    virtual void SetAdditionalParameters(lcio::LCRunHeader* RunHeader);
private:
    std::vector<uint32_t> _content;
};//end implementation v4.2

/**
 * Implementation of the EOR class for DAQ version 4.7+
 */
class TEORBlock_V47 : public TEORBlock_V42
{
public:
    /**
     * Standard constructor
     * @param Content
     */
    TEORBlock_V47(std::vector<uint32_t>& Content);
    /**
     * Virtual destructor
     */
    virtual ~TEORBlock_V47();

    /**
     * Decodes the timestamp of the end of the run
     * @return LCTime object encoding the end time
     */
    virtual lcio::LCTime GetEndTime() const;


};//end implementation v4.7

}//End namespace

#endif /* TIEORBLOCK_HH_ */
