/*
 * TEORBlock_V42.cpp
 *
 *  Created on: Jul 10, 2012
 *      Author: caiazza
 */

#include "TIEORBlock.hh"
#include <string>
#include <sstream>
#include <iostream>

namespace marlintpc
{
TEORBlock_V42::TEORBlock_V42(std::vector<uint32_t>& Content):
        _content(Content)
{
    if(!CheckIntegrity())
    {
        std::stringstream message;
        message << "Failed integrity check while analyzing the End of run Block" << std::endl;
        message << "Dumping the block Content" << std::hex << std::endl;
        std::vector<uint32_t>::iterator itContent = _content.begin();
        for(; itContent != _content.end(); ++itContent)
        {
            message << *itContent << std::endl;
        }
        message << std::dec;
        throw TAltroConverterDecodeException("TEORBlock_V42", message.str());
    }
}

TEORBlock_V42::~TEORBlock_V42()
{

}

uint32_t
TEORBlock_V42::GetLength() const
{
    return _content.size();
}

bool
TEORBlock_V42::CheckIntegrity() const
{
    bool result = true;
    result = result && (GetLength() == _content.at(0) + 1)
            && (GetLength() >= 6)
            && (_content.at(2) == 0x33333333);

    return result;
}

const std::vector<uint32_t>&
TEORBlock_V42::Content() const
{
    return _content;
}

uint32_t
TEORBlock_V42::GetIdentifier() const
{
    return _content[2];
}

lcio::LCTime
TEORBlock_V42::GetEndTime() const
{
    int day = _content[4] & 0x000000ff;
    int month = (_content[4] & 0x0000ff00) >> 8;
    int year = (_content[4] & 0xffff0000) >> 16;
    int second = _content[5] & 0x000000ff;
    int minute = (_content[5] & 0x0000ff00) >> 8;
    int hour = (_content[5] & 0xffff0000) >> 16;
    return UTIL::LCTime(year, month, day, hour, minute, second);
}

uint32_t
TEORBlock_V42::GetNEvents() const
{
    return _content[3];
}

void
TEORBlock_V42::SetAdditionalParameters(lcio::LCRunHeader* RunHeader)
{
    lcio::LCTime EndTime = GetEndTime();
    RunHeader->parameters().setValue("RUN End Date", EndTime.getDateString());
    RunHeader->parameters().setValue("RUN End Timestamp", EndTime.unixTime());
}

}//end namespace



