/*
 * DataBlock.hh
 *
 *  Created on: Aug 1, 2011
 *      Author: caiazza
 */

#ifndef DATABLOCK_HH_
#define DATABLOCK_HH_

#include "AltroBlocksLib.h"

namespace A2Lconverter
{

class DataBlock : public AltroBlock
{

public:
    /**
     * From stream constructor that works as a default as well
     */
    DataBlock(std::ifstream* const inputFile = 0);
    DataBlock(std::ifstream* const inputFile, int start_address, uint32_t length = 0);

    /**
     * Construct the derived class from an instance of the mother class
     */
    DataBlock(const AltroBlock & theMotherBlock);

    /**
     * Construct the class from the data vector
     * This constructor makes a copy of the input vector inside of the member vector
     */
    DataBlock(const std::vector<uint32_t> & refContent);

    /**
     * Construct the class from the data vector specifying the starting point of the
     * relevant section of the data. The function will identify the correct length of
     * the data vector and copy in the member vector containing the data only the
     * the relevant part
     */
    DataBlock(const std::vector<uint32_t>::const_iterator refStart, uint32_t payload);

    /**
     * Destructor
     */
    virtual
    ~DataBlock()
    {
    }

    /**
     * Derived class implementation of the decode function
     */
    virtual void
    Decode();

    /**
     * Derived class implementation of the writer function
     */
    virtual void
    Write2LCIO(IO::LCWriter* lcwriter) const
    {
    }

    /**
     * Derived class implementation of the printer function
     */
    virtual std::string
    PrintSummary() const;

    /**
     * Derived class implementation of the Check Integrity function
     */
    virtual bool
    CheckIntegrity() const;

    /**
     * Add a new collection containing the actual ADC data.If the collection already exists in case of
     * multiple RCUs it will append the data to the existing collection
     */
    void
    AddDataCollection(EVENT::LCCollection* altrocollection, int RCUID) const;

protected:
    /**
     * Decoder unit for the Dataformat 1
     */
    void
    DecodeDF1(std::vector<uint32_t>::const_iterator& decodePosition, int RCUID,
            EVENT::LCCollection* altrocollection) const;

    struct DataMasks
    {
        uint32_t WORDID;
        uint32_t ERRORBIT;
        uint32_t CHANNELHEADER;
        uint32_t DWNUM;
        uint32_t CHANNELADDRESS;
        uint32_t DW1;
        uint32_t DW2;
        uint32_t DW3;
    };

    static const DataMasks DF1MASKS;
    static DataMasks const * Masks;

    std::vector<uint32_t>::const_iterator refStart;
};

}
#endif /* DATABLOCK_HH_ */
