/*
 * EOFBlock.cpp
 *
 *  Created on: Jul 26, 2011
 *      Author: caiazza
 */

#include "EOFBlock.hh"
#include "AltroBlocksLib.h"

namespace A2Lconverter
{
EOFBlock::EOFBlock(std::ifstream * const inputFile)
        : AltroBlock(inputFile)
{

}
EOFBlock::EOFBlock(std::ifstream* const inputFile, int start_address,
        uint32_t length)
        : AltroBlock(inputFile, start_address, length)
{

}

EOFBlock::EOFBlock(const AltroBlock & theMotherBlock)
        : AltroBlock(theMotherBlock)
{

    if (this->CheckIntegrity())
    {
        id = AltroBlock::BLOCK_EOF;
    }
    else
    {
        throw AltroBlockException(
                "The Altro Block copied in the EOF block doesn't appear to be a valid EOF block");
    }
}

EOFBlock::EOFBlock(const std::vector<uint32_t> & refContent)
        : AltroBlock(refContent)
{

}

void
EOFBlock::Decode()
{
    try
    {
        if (content.size() == 0)
        {
            inputFile->clear();
            inputFile->exceptions(std::ifstream::badbit);
            inputFile->seekg(start, std::ios::beg);
            length = ReadWord32() + 1;
            end = start + length * WORD32;
            inputFile->seekg(start, std::ios::beg);
            FillContent(length);
        }
        else
        {
            length = content.size();
            end = start + length * WORD32;
        }
        if (content[0] != content[1] + 1)
            throw AltroBlockException(
                    "The total length doesn't match the header length +1");
        if (content[2] != AltroBlock::BLOCK_EOF)
            throw AltroBlockException(
                    "The block identification doesn't match with BLOCK_EOF");
        id = AltroBlock::BlockIds(content[2]);
        lastEventN = content[3];
        int day = content[4] & 0x000000ff;
        int month = (content[4] & 0x0000ff00) >> 8;
        int year = (content[4] & 0xffff0000) >> 16;
        int second = content[5] & 0x000000ff;
        int minute = (content[5] & 0x0000ff00) >> 8;
        int hour = (content[5] & 0xffff0000) >> 16;
        timestamp = UTIL::LCTime(year, month, day, hour, minute, second);
        fileNumber = content[6];

    }
    catch (std::ifstream::failure &f)
    {
        std::ostringstream err;
        err
                << "ERROR: Exception caught while decoding the EOF block. Unreadable file"
                << std::endl;
        err << f.what() << std::endl;
        std::string tmp = err.str();
        throw AltroBlockException(tmp);
    }
    catch (std::exception &e)
    {
        std::ostringstream err;
        err << "ERROR: Exception caught while decoding the EOR block. "
                << std::endl;
        err << e.what() << std::endl;
        std::string tmp = err.str();
        throw AltroBlockException(tmp);
    }
}

std::string
EOFBlock::PrintSummary() const
{
    std::ostringstream oss;
    oss << std::dec << "End of File summary" << std::endl << std::endl;
    oss << "Ending File Number " << fileNumber << " after event " << lastEventN
            << std::endl << std::endl;
    return oss.str();
}

bool
EOFBlock::CheckIntegrity() const
{
    if (AltroBlock::CheckIntegrity())
    {
        if (content.size() == 0)
        {
            //Check that the block id is correct and that the block ends at the end of file
            bool result = false;
            inputFile->exceptions(std::ifstream::badbit);
            inputFile->seekg(start, std::ios::beg);
            uint32_t tmpLength = ReadWord32() + 1;
            inputFile->seekg(0, std::ios::end);
            result = (static_cast<int>(inputFile->tellg())
                    == static_cast<int>(start + tmpLength * WORD32));
            inputFile->seekg(start + 2 * WORD32, std::ios::beg);
            result = result && (ReadWord32() == AltroBlock::BLOCK_EOF);
            return result;
        }
        else
        {
            return (content[2] == AltroBlock::BLOCK_EOF);
        }
    }
    return false;
}

}

