//LCIO
#include "lcio.h"
#include "IO/LCWriter.h"
#include "EVENT/LCIO.h"
#include "IMPL/LCEventImpl.h"
#include "IMPL/LCGenericObjectImpl.h"
#include "IMPL/LCCollectionVec.h"
#include "IMPL/LCTOOLS.h"
#include "UTIL/LCTime.h"

//LCCD
#include "lccd.h"
#include "lccd/DBInterface.hh"

//TPCCONDDATA
#include "Pedestal.h"
#include "ADCChannelMapping.h"

//C++
#include <ctype.h>
//#include <stdlib.h>
#include <iostream>
#include <fstream>
#include <string>
#include <sstream>
#include <stdio.h>
#include <cstdlib>

void printUsage()
{
    std::cout<<"Usage:"<<std::endl;
    std::cout<<"To write data to a lcio simple file:"<<std::endl;
    std::cout<<"  altroPedestalFile2LCCD -S outfile \\"<<std::endl;
    std::cout<<"  [-M connectionstring folder timeStamp] \\"<<std::endl;
    std::cout<<"  pedestalfile.txt [pedestalfile.txt [pedestalfile.txt ... ]]"<<std::endl;
    std::cout<<"To write data to a database:"<<std::endl;
    std::cout<<"  altroPedestalFile2LCCD -D connectionString folder startLCIOTimeStamp endLCIOTimeStamp \\"<<std::endl;
    std::cout<<"  [-M connectionstring folder timeStamp] \\"<<std::endl;
    std::cout<<"  pedestalfile.txt [pedestalfile.txt [pedestalfile.txt ... ]]"<<std::endl;
    std::cout<<"-M option reads in channelmapping from a database and translates "<<std::endl;
    std::cout<<"   hardware channel and rcu into padID and moduleID from Gear"<<std::endl;
    std::cout<<"Formats:"<<std::endl;
    std::cout<<" Text file: hwChannel intPedestal floatPedestal floatPedestalWidth integer" << std::endl;
    std::cout<<" Connection String: hostname:database:login:password[:port]"<<std::endl;
    std::cout<<" folder: /database/folder"<<std::endl;
    std::cout<<" LCIO time stamp: nano seconds since 1.1.1970 00:00:00"<<std::endl;
}



int main(int argc, char **argv)
{
    bool writeSimpleFile=false;
    bool writeToDatabase=false;
    bool doMapping = false;
    int  optionCounter=0;
    std::string outFileName;
    std::vector<std::string> inFileNames;
    
    lcio::LCWriter* lcWrt=NULL;
    lcio::LCEventImpl*  evt=NULL;
    
    if ( argc < 2 )
    {
        printUsage();
        return(1);
    }

    std::string option = argv[1];
    //DEBUG
    std::cout<<"option= "<<option<<std::endl;
    std::string connectionString;
    std::string folder;
    lcio::long64 startTimeStamp;
    lcio::long64 endTimeStamp;

    std::string mappingConnectionString;
    std::string mappingFolder;
    lcio::long64 mappingTimeStamp;
    
        
    if ( option == "-S" )
    {
        writeSimpleFile=true;
        outFileName=argv[2];
        optionCounter=3;
        lcWrt = lcio::LCFactory::getInstance()->createLCWriter() ;
        lcWrt->open( outFileName, lcio::LCIO::WRITE_NEW ) ;
        // create new lcio event
        evt = new lcio::LCEventImpl() ;
    } 
    else if ( option == "-D" )
    {
        writeToDatabase=true;
        connectionString=argv[2];
        folder=argv[3];
        std::stringstream startstr(argv[4]);
        startstr >> startTimeStamp;
        std::stringstream endstr(argv[5]);
        endstr >> endTimeStamp;
        optionCounter=6;
    }
    else
    {
        printUsage();
        return(1);
    }
    
    
    //see if data should be re-mapped, if so, set variables
    std::map< std::pair<int, int>, std::pair<int, int> > mappingMap;
    option = argv[optionCounter];
    if ( option == "-M" )
    {
        doMapping = true;
        optionCounter++;
        mappingConnectionString = argv[optionCounter];
        optionCounter++;
        mappingFolder = argv[optionCounter];
        optionCounter++;
        std::stringstream startstr(argv[optionCounter]);
        startstr >> mappingTimeStamp;
        optionCounter++;
        
        // fill a map with the channel mapping values
        lccd::DBInterface* dbConnection = new lccd::DBInterface(mappingConnectionString, mappingFolder, false);
        lcio::LCCollectionVec* condMappingCol = dynamic_cast<lcio::LCCollectionVec*>( dbConnection->findCollection(mappingTimeStamp));
        if ( condMappingCol )
        {
            tpcconddata::ADCChannelMapping*    Entry = 0;
            std::pair<  std::map< std::pair<int, int>,std::pair<int, int> >::iterator,bool> iInsertReturn;
            int nEntries = condMappingCol->getNumberOfElements();
            for (int iEntry = 0; iEntry < nEntries; ++iEntry)
            {
                Entry = new tpcconddata::ADCChannelMapping( dynamic_cast<LCGenericObject*>( condMappingCol->getElementAt( iEntry ) ) );
                //std::cout<<"Inserting in the channel map: hw=("<<Entry->getChannelID()<<","<<Entry->getReadoutGroup()<<") gear=("<<Entry->getPadID()<<","<<Entry->getModuleID()<<") "<<std::endl;
                iInsertReturn=mappingMap.insert( std::make_pair(std::make_pair(Entry->getChannelID(), Entry->getReadoutGroup()),
                                                                std::make_pair(Entry->getPadID(), Entry->getModuleID())        ) 
                                               );
                if ( !iInsertReturn.second )
                {
                    std::cerr<<"ChannelMapping: could not store ChannelMapping for hardwareID: ("<<Entry->getChannelID()<<","<<Entry->getReadoutGroup()<<") in the readoutChannelMapping map because there was already a value stored for this channel"<<std::endl;
                }
            }
        }
        else
        {
            std::cerr<<"Error: no valid mapping collection found for time stamp: "<<mappingTimeStamp<<std::endl;
            return 1;
        }
    }
    
    for ( int i=optionCounter; i<argc ; ++i)
    {
        inFileNames.push_back(argv[i]);
    }
    
    /*DEBUG
    for (int i=0; i< argc ; ++i)
    {
        std::cout<<"DEBUG: argument ["<<i<<"]: "<<argv[i]<<std::endl;
    } 
    */

    // create the collection we want to store the data in
    lcio::LCCollectionVec* pedestalCol = new lcio::LCCollectionVec( lcio::LCIO::LCGENERICOBJECT );
    
    int fileNum=1;
    std::vector< std::pair<int,int> > warningList;
    // loop over the input files
    for (std::vector<std::string>::iterator inFile = inFileNames.begin(); inFile != inFileNames.end(); ++inFile)
    {
        int rcu=0;
        std::filebuf infile;
        infile.open(inFile->c_str(),std::ios::in);
        std::istream inputstream(&infile);
        std::cout<<" Reading file #"<<fileNum<<" : "<<*inFile<<std::endl;
        fileNum++;
        int linenumber = -1;
        // read all lines in the file
        while (!inputstream.eof() )
        {
            // read the next line from the file
            std::string line;
            std::getline(inputstream, line);
            if (inputstream.eof())   //eof is reached
            {
                if (!line.empty())
                {
                    std::cout << " Ignoring non-empty line at end of file: " << line << std::endl;
                }
                break;
            }
            // the line as a stream so ew can stream into the variables
            std::stringstream linestream(line);
            linenumber++;
            // check if the next line is a comment
            if (linestream.peek() == '#')
            {
                std::stringbuf name_buf;
                linestream.get( name_buf, ' ' );
                std::string name = name_buf.str();            
                if ( name == "#RCUID" )
                {
                    linestream.get(name_buf);
                    std::string dump=name_buf.str();
                    size_t found;
                    found=dump.find_last_of(' ');
                    std::string rcustring = dump.substr(found+1);
                    const char* rcunum=rcustring.c_str();
                    //std::cout<<" rcunum = '"<<rcunum<<"'"<<std::endl;
                    rcu=std::atoi(rcunum);
                    std::cout<<" Found in File RCU # = "<<rcu<<std::endl; 
                }
                else
                {
                    //std::cout << "  DEBUG: skipping comment line " << linenumber << " : "  << line <<  std::endl;
                }
                continue;
            }
            int hw_channel, intPedestal, counter;
            float floatPedestal, pedestalWidth;

            // stream the data into the variables
            linestream >>  hw_channel >> intPedestal >> floatPedestal >> pedestalWidth >> counter;
            // std::cout << "DEBUG:  reading line " << linenumber << " : "<< line << std::std::endl;
            // test if everything went fine
            if (linestream.fail())
            {
                std::cout << "error reading line " << linenumber << " : "<< line << std::endl;
                continue;
            }
            tpcconddata::Pedestal* pedestalObject=NULL;
            if (!doMapping) 
            {
                pedestalObject = new  tpcconddata::Pedestal(hw_channel, rcu , floatPedestal , pedestalWidth);
                pedestalCol->addElement( pedestalObject );
            } 
            else
            {
                std::map< std::pair<int, int>, std::pair<int, int> >::iterator result = 
                    mappingMap.find(std::make_pair(hw_channel, rcu));
                if ( result != mappingMap.end() )
                {
                    int padID=((*result).second).first;
                    int moduleID=((*result).second).second;
                    //std::cout<<"PadID/ModuleID="<<padID<<"/"<<moduleID<<std::endl;
                    pedestalObject = new  tpcconddata::Pedestal(padID, moduleID , floatPedestal , pedestalWidth);
                    pedestalCol->addElement( pedestalObject );
                }
                else
                {
                    if ( intPedestal!=1023)
                        warningList.push_back(std::make_pair(hw_channel,rcu));
                }
            }
        }                            //  while (!inputstream.eof() )
        // close the input text file
        infile.close();
    }

    
    if (warningList.size() >0 )
    {
        std::cerr<<"WARNING: the following pedestal values could not be mapped, since there is no entry in the pad mapping:"<<std::endl;
        for (unsigned int i=0; i<warningList.size(); i++)
        {
            std::cerr<<"("<<warningList[i].first<<","<<warningList[i].second<<") ";
        }
        std::cerr<<std::endl;
    }
    
            
    if ( writeToDatabase )
    {
        lccd::DBInterface* dbConnection = new lccd::DBInterface(connectionString, folder, true );
        dbConnection->storeCollection( startTimeStamp, endTimeStamp, pedestalCol, "TPCPedestal");
        delete dbConnection;
    }

    if ( writeSimpleFile )
    {
        // add the pedestal collection to the event
        evt->addCollection( pedestalCol , "TPCPedestal" ) ;
        // dump the event to the screen
        lcio::LCTOOLS::dumpEvent( evt ) ;
        // write event to file
        lcWrt->writeEvent( evt ) ;
        lcWrt->close() ;
        // ------------ IMPORTANT ------------- !
        delete evt ;
        // -------------------------------------
    }
    return 0;
}
