#include "lcio.h"
//
#include "IO/LCWriter.h"
#include "EVENT/LCIO.h"
//
#include "IMPL/LCEventImpl.h" 
#include "IMPL/LCRunHeaderImpl.h"
#include "IMPL/LCTOOLS.h"

//#include <iostream>
//#include <fstream>
#include <string>
//#include <sstream>
#include <cstdlib>
#include <cstring>

using namespace std ;
using namespace lcio ;

void printUsage() {

	cout << "\nEmptyLCIOFileGenerator: Generates an LCIO file with the specified parameters. Useful for simulations in MarlinTPC with no input file.\n\n" << "\tEmptyLCIOFileGenerator <run number> <number of events> [filename==\"empty_run.slcio\'] [detector_model]\n" << endl;
}

int main(int argc, char **argv) {

	if ( ( argc <= 3 ) || ( strcmp ( argv[1], "--help" ) == 0 ) || ( strcmp ( argv[1], "-?" ) == 0 ) ) {

		printUsage ();
		exit (1);
	}

	// get the run number
	const int run_number = atoi( argv[1] );

	// get the event count
	const int event_count = atoi( argv[2] );

	// get outfile name
	string output_filename = ( argc < 4 )? string ( "empty_run.slcio" ): argv[3];

	// get detector names
	string detector_name = ( argc < 5 )? string ( "" ): argv[4];

	// open outfile
	LCWriter* writer = LCFactory::getInstance()->createLCWriter();

	writer->open( output_filename, LCIO::WRITE_NEW ) ;

	// create the header
	LCRunHeaderImpl *run_header = new LCRunHeaderImpl();


	run_header->setDetectorName ( detector_name );
	run_header->setRunNumber( run_number );

	writer->writeRunHeader( run_header );

	for ( int i = 0; i < event_count; i++ ) {

		// create new lcio event
		LCEventImpl *event = new LCEventImpl();

		event->setRunNumber( run_number );
		event->setEventNumber( i );

		writer->writeEvent( event );

		delete event;
	}

	cout << "Summary:\n" << "\tRun: " << run_number << "\n\tEvents: " << event_count << "\n\tFilename: " << output_filename << "\n\tDetector name: " << detector_name << endl;

	delete run_header;

	writer->close() ;

	return 0;

}
