#ifndef CGAELECTRICFIELD_H
#define CGAELECTRICFIELD_H 1

// MarlinTPC
#include "Field.h"

// CLHEP
#include <CLHEP/Matrix/Vector.h>

// stl
#include <string>
#include <vector>

namespace LCIO {

	class LCRunHeader;
}

namespace EVENT {

	class LCCollection;
	class LCRunHeader;
}

namespace marlintpc {

/** GEARElectricField : An electric field which gets it's values from Mokka through CGA (Common Geometry Access).
 *
 *    @author J. Abernathy (UVic)<br>
 */
class CGAElectricField: public Field {

public:
	/** constructor
	 */
	CGAElectricField ( lcio::LCRunHeader *run_header );

	/** destructor
	 */
	virtual ~CGAElectricField();

	/** returns a new field... similar to how the Marlin processor registration works
	 */
	virtual Field *new_field( void );

	/** constructs the field from an LCCollection of data objects (could be anything... field bin values or just parameters)
	 */
	virtual bool construct( lcio::LCCollection * );

	/** gets the field value at a specified point
	 */
	CLHEP::Hep3Vector get_field_value( const CLHEP::Hep3Vector & );

	/** returns the typename of the field
	 */
	std::string get_typename();

protected:

private:

};

} // namespace marlintpc

#endif // CGAMAGNETICFIELD_H
