#ifndef CGAMAGNETICFIELD_H
#define CGAMAGNETICFIELD_H 1

// MarlinTPC
#include "Field.h"

// CLHEP
#include <CLHEP/Matrix/Vector.h>

// stl
#include <string>
#include <vector>

namespace LCIO {

	class LCRunHeader;
}

namespace EVENT {

	class LCCollection;
	class LCRunHeader;
}

namespace marlintpc {

	class CGAElectricField;
}

namespace marlintpc {

/** CGAMagneticField : A magnetic field which gets it's values from Mokka through CGA (Common Geometry Access).
 *
 *    @author J. Abernathy (UVic)<br>
 */
class CGAMagneticField: public Field {

public:
	/** constructor
	 */
    CGAMagneticField ( lcio::LCRunHeader *run_header );

	/** destructor
	 */
	virtual ~CGAMagneticField();

	/** returns a new field... similar to how the Marlin processor registration works
	 */
	virtual Field *new_field( void );

	/** constructs the field from an LCCollection of data objects (could be anything... field bin values or just parameters)
	 */
	virtual bool construct( lcio::LCCollection * );

	/** gets the field value at a specified point
	 */
	CLHEP::Hep3Vector get_field_value( const CLHEP::Hep3Vector & );

	/** returns the typename of the field
	 */
	std::string get_typename();

	// go give access to the _cga_initialized variable
	friend class CGAElectricField;

protected:

private:

	#ifdef USE_CGA
	/** this variable keeps track of whether CGA has been initialized or not
	 */
	static bool _cga_initialized;
	#endif

};

} // namespace marlintpc

#endif // CGAMAGNETICFIELD_H
