#ifndef IDEALSOLENOIDFIELD_H
#define IDEALSOLENOIDFIELD_H 1

// stl
#include <string>
#include <vector>

// CLHEP

// MarlinTPC
#include "Field.h"

namespace EVENT {

	class LCCollection;
}

namespace marlintpc {

/** IdealSolenoidField : An ideal cylindrical solenoid field
 *
 *    @author J. Abernathy (UVic)<br>
 */
class IdealSolenoidField: public Field {

public:
	/** constructor
	 */
	IdealSolenoidField();

	/** destructor
	 */
	virtual ~IdealSolenoidField();

	/** returns a new field... similar to how the Marlin processor registration works
	 */
	virtual Field *new_field( void );

	/** constructs the field from an LCCollection of data objects (could be anything... field bin values or just parameters)
	 */
	virtual bool construct( EVENT::LCCollection * );

	/** gets the field value at a specified point
	 */
	CLHEP::Hep3Vector get_field_value( const CLHEP::Hep3Vector & );

	/** returns the typename of the field
	 */
	std::string get_typename();

protected:

private:

	/** the boundaries of the solenoid map
	 */
	double _r_min;
	double _r_max;
	double _z_max;

	/** the ideal field value
	 */
	CLHEP::Hep3Vector _field_value;
};

} // namespace marlintpc

#endif // IDEALSOLENOIDFIELD_H
