// take nm output for ClusterDensity and fit the parametrisation-function 
// to get the function parameters for the PrimaryIonisation processor


#include <fstream>

#include "TCanvas.h"
#include "TF1.h"
#include "TFile.h"
#include "TGraph.h"
#include "TH1.h"
#include "TStyle.h"

using namespace std;

void parametrise(const char *infilename);

// the main program in case this is compiled instead of run in CINT:
// compile example: 
//   g++ -o parametrise parametrise.C `root-config --cflags --glibs` 
// run example:
//   ./parametrise ClusterDensity240VcmDelta_T2K.root
# ifndef __CINT__
int main(int argc, char** argv)
{
  parametrise(argv[1]);
  return 0;
}
# endif

void parametrise(const char *infilename){

  // read input file
  TFile *infile = TFile::Open(infilename);
  TGraph *grClusterDensity=(TGraph*)infile->Get("grClusterDensity");
  int npoints=grClusterDensity->GetN();

  // just a few drawing settings, and 
  // convert the graph to a histogram (works better for fitting for some reason)
  TCanvas *cHist = new TCanvas("cHist", "cluster density histogram");
  cHist->SetLogx();

  TH1F *hClusterDensity = new TH1F("hClusterDensity", grClusterDensity->GetTitle(), npoints-1, grClusterDensity->GetX());
  for (int i=1; i<10000; i++){
    hClusterDensity->SetBinContent(i, grClusterDensity->Eval( hClusterDensity->GetBinCenter(i) ));
  }
  hClusterDensity->SetLineColor(8);
  hClusterDensity->SetMarkerColor(8);
  hClusterDensity->SetMarkerStyle(21);
  hClusterDensity->SetMarkerSize(1.1);
  hClusterDensity->SetStats(0);
  hClusterDensity->SetOption("P");
  hClusterDensity->Draw("P");
  hClusterDensity->GetXaxis()->SetRangeUser(1,1e4);
  hClusterDensity->GetYaxis()->SetRangeUser(25, 43);


  // function used as parametrisation in MarlinTPC (PrimaryIonisationProcessor)
  TF1 *fHEED = new TF1("fHEED", "[0]+[1]*log( sqrt(x*x-1) )+[2]*pow(log( sqrt(x*x-1) ),2)+[3]*pow(log( sqrt(x*x-1) ),3)+[4]*pow(log( sqrt(x*x-1) ),4)+[5]*pow(log( sqrt(x*x-1) ),5)+[6]*pow(log( sqrt(x*x-1) ),6)+[7]*pow(log( sqrt(x*x-1) ),7)", 1e4);
  fHEED->SetParameters(hClusterDensity->GetBinContent(npoints-1), 0, 0, 0, 0, 0, 0, 0); 
  fHEED->SetLineColor(kRed);
  fHEED->Draw("same");
  hClusterDensity->Fit("fHEED");

  cHist->SaveAs("heedparameterfit.png");
  cHist->SaveAs("heedparameterfit.pdf");
  hClusterDensity->SaveAs("heedparameters.root");

  // dump the parameters into a text file with the required format
  std::ofstream outfile;
  outfile.open("heedparameters.dat");
  for (int ipar=0; ipar<=7; ipar++){
    outfile << fHEED->GetParameter(ipar) << endl;
  }
  outfile.close();

  return;
}
