/*
 * TrackCircleIntersectionCalculator.hh 
 *
 *  Created on: April 22, 2013
 *      Author: iheinze, amunnich
 */

#ifndef TRACKCIRCLEINTERSECTIONCALCULATOR
#define TRACKCIRCLEINTERSECTIONCALCULATOR

#include<utility>

namespace  marlintpc
{
  /**
   * Class to calculate the intersection between the track and a circle (for example a measurement layer)
   * Inputs and outputs are in global coordinates
   */
  class TrackCircleIntersectionCalculator
  {
  public:
    /** Calculate the intersection of a circular pad row with a track circle.
     *\param circlecenter The center of the circle of the pad row in global coordinates.
     *\param radius The radius of the pad row.
     *\param d0 LCIO track paramter d0
     *\param phi  LCIO track paramter phi
     *\param omega  LCIO track paramter omega
     *
     * Return value:
     * \li x and y in global coordinates of the intersection with positive arclength (following the direction of the helix).
     * WARNING: still some offset (400 mum) if track has curvature. NEEDS more debugging!!
     */
    TrackCircleIntersectionCalculator(std::pair<double,double> circlecenter, double radius, double d0, double phi, double omega);
                                    
    /**
     * Virtual destructor
     */
    virtual ~TrackCircleIntersectionCalculator();


    ///returns the point of intersection
    std::pair<double,double> GetIntersection()
    {
      return _Intersection;
    };

  private:
    ///calculate point of intersection
    void CalculateIntersection();

    ///center of the cicle (measurement layer)
    std::pair<double,double> _CircleCenter;
    ///radius of the circle (measurement layer)
    double _Radius;
    ///track d0
    double _D0;
    ///track phi
    double _Phi0;
    ///track curvature
    double _Omega;
    
    ///point of intersection: return value in cartesian coordinates
    std::pair<double,double> _Intersection;

    ///default constructor: disabled
    TrackCircleIntersectionCalculator();

  };
  
}

#endif //TRACKCIRCLEINTERSECTIONCALCULATOR
