#ifndef CLOCKSYNCPROCESSOR_H
#define CLOCKSYNCPROCESSOR_H

#include <marlin/Processor.h>
#include <lcio.h>
#include <string>

//MarlinTPC
#include "ADCChannelMapping.h"
#include "TPCConditionsListener.h"
#include "ChannelMappingListener.h"


namespace marlintpc
{
/**This Processor shifts hits of the RCU which is out of sync. Therefore, it calculates the
 * difference of the hit time and the hit time of the reference row on a reference (which is hard coded to be 7).
 * Then the five rows above the reference row of the
 * 
 * @author F. Mueller, DESY
 *
 */

class ClockSyncProcessor : public marlin::Processor
{
public:

	virtual Processor*  newProcessor() { return new ClockSyncProcessor; }

	ClockSyncProcessor();

	virtual void init();

	virtual void processRunHeader(EVENT::LCRunHeader* run);

	virtual void processEvent(EVENT::LCEvent* evt);

	virtual void check(EVENT::LCEvent* evt);

	virtual void end();


protected:
	/* the place for protected and private member data and functions */
	std::string _inputColName; ///< Name of the input collection
	std::string _outputCollectionName;
	std::string _channelMappingColName;
	bool _outputHitsPersistent;
	int _rcuOutOfSync;
	double _reduceTime;
	double _delayTime;
	double _cut;
	float _driftVelocity;
	/** drift velocity in [mm/mu s]
	 */
	float _driftVelocityOverride;

	/** the name of the collection the TPC conditions are stored with
	 */
	std::string _inputTPCConditionsCollectionName;

	TPCConditionsListener* _tpcConditionsListener;
	ChannelMappingListener* _channelMappingListener;
};
} // namespace marlintpc
#endif // CLOCKSYNCPROCESSOR_H
