#ifndef SelectNthEventsProcessor_h
#define SelectNthEventsProcessor_h 1

#include "marlin/Processor.h"
#include "lcio.h"
#include <string>


using namespace lcio ;
using namespace marlin ;


/** Select a number n to use only every nth event by throwing a SkipEventException for all other ones.
 *  All further processors in the chain will not process the skipped events.
 *  This selection can be inverted.
 * 
 * @param nEventToSelect   Integer number. Every nth event is processed by the chain, all other events are skipped.
 * @param InvertSelection   Inverts the selection. If true, every nth event is skipped, all other events are processed.
 * 
 * @author U Einhaus, DESY
 * @version $Id: SelectNthEventsProcessor.h,v 0.1 2015-06-30 $ 
 */

class SelectNthEventsProcessor : public Processor {
  
 public:
  
  virtual Processor*  newProcessor() { return new SelectNthEventsProcessor ; }
  
  
  SelectNthEventsProcessor() ;
  
  /** Called at the begin of the job before anything is read.
   * Use to initialize the processor, e.g. book histograms.
   */
  virtual void init() ;
  
  /** Called for every run.
   */
  virtual void processRunHeader( LCRunHeader* run ) ;
  
  /** Called for every event - the working horse.
   */
  virtual void processEvent( LCEvent * evt ) ; 
  
  
  virtual void check( LCEvent * evt ) ; 
  
  
  /** Called after data processing for clean up.
   */
  virtual void end() ;
  
  
 protected:

  int _nRun ;
  int _nEvt ;
  int _nEvtProc;
  int _nEvtSkip;
  
  int _nEventToSelect;
  int _selectionOffset;
  bool _invertSelection;

} ;

#endif



