#ifndef TimePixMaskCreaterProcessor_h
#define TimePixMaskCreaterProcessor_h 1

#include "marlin/DataSourceProcessor.h"
#include "marlin/ProcessorMgr.h"

#include "TimePixPixelMode.h"

#include "IO/LCWriter.h"
#include "IMPL/LCEventImpl.h"
#include "IMPL/LCRunHeaderImpl.h"
#include "IMPL/TrackerRawDataImpl.h"
#include "IMPL/LCCollectionVec.h"

#include "UTIL/LCTOOLS.h"

#include <vector>
#include <map>

namespace marlin{
  
  /** Creates an lcio file containing the masking of dead and noisy pixel
   *  
   *
   *  <h4>Input - Prerequisites</h4>
   *  TimePix occupancy histogram.
   *
   *  <h4>Output</h4>
   *  \li A runheader
   *  \li Events with TimePixRawData (lcio::TrackerRawData)
   *  \li First event contains TimePixModeMap collection
   *
   * @param OutputCollectionName Name of the output status map collection, default \c TimePixStatusMap
   * @param DetectorName Detector name for run header, default "TPC"
   * @param Description  Data descripton for run header, default "no description given"
   * @param SubDetectorName Sub-detector name for run header, default "TimePix"
   * @param ChipID Interer ID number of the chip (lcio::TrackerRawData::CellID0), should be 0 as it's only one chip in the current version
   * @param SetOutputTransient If not 0 the output collection is set transient
   * @param NoisyPixels List of pixels to be marked as noisy
   * @param NoisyColumns List of columns to be marked as noisy
   * @param NoisyRows List of rows to be marked as noisy
   * @param DeadPixels List of pixels to be marked as dead
   * @param DeadColumns List of columns to be marked as dead
   * @param DeadRows List of rows to be marked as dead
   *
   * @author Simone Zimmermann(University of Bonn) modified by Christoph Krieger(University of Bonn)
   */
  
  class TimePixMaskCreaterProcessor : public DataSourceProcessor {
    
  public:

   TimePixMaskCreaterProcessor() ;

    virtual TimePixMaskCreaterProcessor*  newProcessor() ;


    /** Creates events with TrackerRawData collections from the TimePix raw data input files and
     *  calls all active processors' processEvent() and processRunHeader Method.
     *
     */
    virtual void readDataSource( int numEvents ) ;
    
    
    virtual void init() ;
    virtual void end() ;

    
  protected:
    
    std::string _outputColName;

    bool _isFirstEvent;

    int _chipID;
    int _runNr;
    int _evtNr;
    int _maxRecordN;
    int _nColumn;
    int _nRow;

    std::string _detName;
    std::string _description;
    std::string _subDetName;
    std::string _statusMapColName;

    std::vector<int> _noisyPixels;
    std::vector<int> _deadPixels;
    std::vector<int> _noisyColumns;
    std::vector<int> _deadColumns;
    std::vector<int> _noisyRows;
    std::vector<int> _deadRows;

/** give the status which is set to the transient flag of the output collection
 * it is an int instead of a boolean, because the processor parameter cannot be a boolean
 */
    int _outputIsTransient;
    
  };

 
} // end namespace marlin 

#endif
