#!/bin/bash

if [ $# == 0 ]
then
  echo "Usage: `basename $0` oldSteeringFile.xlm [newSteeringFile.xml]"
  echo "       If newSteeringFile.xml is not given, the input file will be modified."
  exit 1
fi

if [ $# -gt 2 ]
then
  echo "Too many arguments. Run `basename $0` without arguments for syntax."
  exit 2
fi

OUTPUT_FILENAME="tmp.xml"

if [ $# == 2 ]
then
  OUTPUT_FILENAME=$2
  echo OUTPUT_FILENAME is $OUTPUT_FILENAME
fi

# the [?r] in UseParabolicFit* also fixes the erroneously added r at the end of the parameter name in the previous version of this script
cat $1 | sed "{s/IsPedestalSubstractedOverride/IsPedestalSubtractedOverride/g}"\
| sed "{s/PedestalWidthScale/NoiseValuePedestalWidthFactor/g}"\
| sed "{s/PulseEndThreshold/PulseEndPedestalWidthFactor/g}"\
| sed "{s/PulseStartThreshold/PulseStartPedestalWidthFactor/g}"\
| sed "{s/UseParabolicFitAmplitudeAsCharge[?r]/UseParabolicFitAmplitudeAsCharge/g}"\
| sed "{s/UseParabolicFitMaxAsTime[?r]/UseParabolicFitMaxAsTime/g}"\
> tmp2.xml 

#second step: replace the UseParabolicFit parameters (bool) with CalculationMode parametes (int)
# Also replace the comments. The meaning of the integers is important to know.
# Replace true and false with the corresponding integers (1 and 0). By chance this is the usual interpretation.
cat tmp2.xml |  sed "{s/UseParabolicFitMaxAsTime\".*type=\"bool\".*>.*true.*</TimeCalculationMode\" type=\"int\">1 </ig}"\
| sed "{s/UseParabolicFitMaxAsTime\".*type=\"bool\".*>.*false.*</TimeCalculationMode\" type=\"int\">0 </ig}"\
| sed "{s/UseParabolicFitAmplitudeAsCharge\".*type=\"bool\".*>.*true.*</ChargeCalculationMode\" type=\"int\">1 </ig}"\
| sed "{s/UseParabolicFitAmplitudeAsCharge\".*type=\"bool\".*>.*false.*</ChargeCalculationMode\" type=\"int\">0 </ig}"\
| sed "{s/Set this flag to use the amplitude calculated by fitting a parabola to the log of the pulse data.*-->/SUMQ = 0 (default), FIT_PARABOLA = 1, MAXQ = 2-->/g}"\
| sed "{s/Set this flag to use the time caclulated by fitting a parabola to the log of the pulse data.*-->/INFLEXION_POINT = 0 (default), FIT_PARABOLA = 1,  CENTRE_OF_GRAVITY = 2, FIRST_ABOVE_THRESHOLD = 3, AT_MAXIMUM = 4-->/g}"\
> $OUTPUT_FILENAME

if [ $# == 1 ]
then
  mv $OUTPUT_FILENAME $1
fi

rm tmp2.xml

echo Please check your updated xml file. The parameter names have been replaced globally. Parameters in other processors might also be affected!
echo " "
echo The ReadoutFrequencyOverride has changed from Hz to MHz. Please update this value manually!
