/* FieldSetting.cc
 * $Id$ 
 */

#include "FieldSetting.h"

//LCIO
#include "lcio.h"
#include "EVENT/LCGenericObject.h"
#include "IMPL/LCGenericObjectImpl.h"

namespace tpcconddata{

std::string FieldSetting::getRevision(){
    return std::string("$Rev: 5472 $");
}

std::string FieldSetting::getDefaultColName(){
    return std::string("TPCFieldSetting");
}

FieldSetting::FieldSetting( int NGEMs ){
    createObj=true;
    myObj = new IMPL::LCGenericObjectImpl();

    setNGEMs(NGEMs);

    setGEMStatus( 0 );
    setDriftFieldStatus( 0 );
    setMagneticFieldStatus( 0 );
    for (int iGem = 0; iGem<NGEMs; iGem++){
	setGEMVoltage( iGem, 0 );
    }
    for (int iGem = 0; iGem<(NGEMs-1); iGem++){
	setTransferField( iGem, 0 );
    }
    setInductionField( 0 );
    setCathodeVoltage( 0 );
    setGuardRingVoltage( 0 );
    setAnodeVoltage( 0 );
    setMagneticField( 0 );
}

FieldSetting::FieldSetting(EVENT::LCObject* Obj){
    createObj=false;
    myObj = dynamic_cast<IMPL::LCGenericObjectImpl*>( Obj );
    
    if (myObj ==0){
	//could be an FieldSetting
	  FieldSetting* dummyObj = dynamic_cast<FieldSetting*>( Obj );
	  if (dummyObj != 0){
	      myObj=dummyObj->obj();
	  }
    }
}

FieldSetting::~FieldSetting(){
    if (createObj) delete myObj;
}

int FieldSetting::getNGEMs() const {
    return myObj->getIntVal( 0 );
}

int FieldSetting::getDriftFieldStatus() const {
    return myObj->getIntVal( 1 );
}

int FieldSetting::getMagneticFieldStatus() const {
    return myObj->getIntVal( 2 );
}

int FieldSetting::getGEMStatus() const {
    return myObj->getIntVal( 3 );
}

float FieldSetting::getGEMVoltage( int GEMID ) const {
    if ( GEMID<getNGEMs() )
	return myObj->getFloatVal( 3+GEMID );
    return 0;
}

float FieldSetting::getTransferField( int GEMID ) const {
    if ( GEMID<getNGEMs()-1 )
	return myObj->getFloatVal( 3+getNGEMs()+GEMID );
    return 0;
}

float FieldSetting::getInductionField() const {
    return myObj->getFloatVal( 3+getNGEMs()*2 );
}    

float FieldSetting::getCathodeVoltage() const {
    return myObj->getFloatVal( 0 );
}

float FieldSetting::getGuardRingVoltage() const {
    return myObj->getFloatVal( 3+getNGEMs()*2 + 1 );
}

float FieldSetting::getAnodeVoltage() const {
    return myObj->getFloatVal( 1 );
}

float FieldSetting::getDriftField( float maxDriftLegth ) const {
    return (getCathodeVoltage()-getAnodeVoltage() )/maxDriftLegth;
}

float FieldSetting::getMagneticField() const {
    return myObj->getFloatVal( 2 );
}

void FieldSetting::setNGEMs( int Value){
    myObj->setIntVal( 0, Value );
}

void FieldSetting::setDriftFieldStatus( int Value ){
    myObj->setIntVal( 1, Value );
}

void FieldSetting::setMagneticFieldStatus( int Value ){
    myObj->setIntVal( 2, Value );
}

void FieldSetting::setGEMStatus( int Value ){
    myObj->setIntVal( 3, Value );
}

void FieldSetting::setGEMVoltage( int GEMID, float Value ){
    if ( GEMID<getNGEMs() )
	myObj->setFloatVal( 3+GEMID, Value );
}
    
void FieldSetting::setTransferField( int GEMID, float Value ){
    if ( GEMID<getNGEMs()-1 )
	myObj->setFloatVal( 3+getNGEMs()+GEMID, Value );
}

void FieldSetting::setInductionField( float Value ){
    myObj->setFloatVal( 3+getNGEMs()*2, Value );
}    

void FieldSetting::setCathodeVoltage( float Value ){
    myObj->setFloatVal( 0, Value );
}

void FieldSetting::setGuardRingVoltage( float Value ){
    myObj->setFloatVal( 3+getNGEMs()*2 + 1, Value );
}

void FieldSetting::setAnodeVoltage( float Value ){
    myObj->setFloatVal( 1, Value );
}

void FieldSetting::setMagneticField( float Value ){
    myObj->setFloatVal( 2, Value );
}

IMPL::LCGenericObjectImpl* FieldSetting::obj() { 
    return myObj; 
}
    
int FieldSetting::id() { 
    return myObj->id(); 
}


// void FieldSetting::print ( std::ostream& os = std::cout ) const {

// 	os<<"cathode:   "<<getCathodeVoltage()
// 	  <<" V   \t-----------------------"<<std::endl;
// 	os<<"magnetic field "<<getMagneticField()<<" T"<<std::endl;
// 	os<<"anode:     "<<getAnodeVoltage()  
// 	  <<" V   \t-----------------------"<<std::endl;	
// 	for (int iGem = 0; iGem<getNGEMs()-1; iGem++){
// 	    os<<"GEM:     "<<getGEMVoltage(iGem)
// 	      <<" V   \t-----------------------"<<std::endl;
// 	    os<<"transferfield: \t"<<getTransferField(iGem)<<" V/mm"<<std::endl;
// 	}
// 	os<<"GEM:       "<<getGEMVoltage(getNGEMs()-1)
// 	  <<" V   \t-----------------------"<<std::endl;
// 	os<<"inductionfield "<<getInductionField()<<" V/mm"<<std::endl;  
// 	os<<"pad plane: 0"
// 	  <<" V   \t-----------------------"<<std::endl;

//     } 

// std::ostream &operator<<(std::ostream &os, const FieldSetting &fs) {
//     fs.print(os);
//     return os;
// }

int FieldSetting::getNInt() const    { return 4;}//myObj->getNInt(); } 
int FieldSetting::getNFloat() const  { return myObj->getNFloat(); }  
int FieldSetting::getNDouble() const { return 0; } 

int FieldSetting::getIntVal(int index) const {
    return myObj->getIntVal( index ) ;
}
float FieldSetting::getFloatVal(int index) const { 
    return myObj->getFloatVal( index ) ;
}
double FieldSetting::getDoubleVal(int  /*index*/) const {
    return 0.0;//myObj->getDoubleVal( index ) ;
}

}
