/* GenericADCElectronicsParameters.cc
 * $Id$
 */

#include "GenericADCElectronicsParameters.h"

namespace tpcconddata
{

    std::string GenericADCElectronicsParameters::getRevision()
    {
        return std::string("$Rev: 1649 $");
    }

    std::string GenericADCElectronicsParameters::getDefaultColName()
    {
        return std::string("GenericADCElectronicsParameters");
    }

    GenericADCElectronicsParameters::GenericADCElectronicsParameters(int readoutGroupID, int polarity, int maxADCValue, float frequency,bool isPedestalSubtracted, bool isZeroSupressed)
    {
        setReadoutGroup(readoutGroupID);
        setPolarity(polarity);
        setMaxADCValue(maxADCValue);
        setFrequency(frequency);
        setPedestalSubtracted(isPedestalSubtracted);
        setZeroSupressed(isZeroSupressed);        
    }
    
    GenericADCElectronicsParameters::~GenericADCElectronicsParameters() { /* no op*/  }

    void GenericADCElectronicsParameters::setReadoutGroup(int readoutGroupID)
    {
        obj()->setIntVal( 0, readoutGroupID );
    }

    void GenericADCElectronicsParameters::setPolarity(int polarity)
    {
        obj()->setIntVal( 1, polarity );
    }

    void GenericADCElectronicsParameters::setMaxADCValue(int maxADCValue)
    {
        obj()->setIntVal( 2, maxADCValue );
    }

    void GenericADCElectronicsParameters::setFrequency(float frequency)
    {
        obj()->setFloatVal( 0, frequency );
    }

    void GenericADCElectronicsParameters::setPedestalSubtracted(bool onoff)
    {
        obj()->setIntVal( 3, (int)onoff);
    }
    
    void GenericADCElectronicsParameters::setZeroSupressed(bool onoff)
    {
        obj()->setIntVal( 4, (int)onoff);
    }
    
    
    int GenericADCElectronicsParameters::getReadoutGroup() const
    {
        return getIntVal( 0 );
    }

    int GenericADCElectronicsParameters::getPolarity() const
    {
        return getIntVal( 1 );
    }

    int GenericADCElectronicsParameters::getMaxADCValue() const
    {
        return getIntVal( 2 );
    }

    float GenericADCElectronicsParameters::getFrequency() const
    {
        return getFloatVal( 0 );
    }

    bool GenericADCElectronicsParameters::isPedestalSubtracted() const
    {
        return (bool)getIntVal( 3 );
    }
    
    bool GenericADCElectronicsParameters::isZeroSupressed() const
    {
        return (bool)getIntVal( 4 );
    }

    
        
    
    void GenericADCElectronicsParameters::print(  std::ostream& os ) const
    {
        std::streamsize PrecisionBefore = os.precision();
        std::streamsize WidthBefore = os.width();
        os.precision(3);  os.setf(std::ios_base::fixed);
        os << "Electronics Parameters: ReadoutGroup :"<<getReadoutGroup()<<" Polarity: " << getPolarity() << "[+/-1] Precision:" << getMaxADCValue() <<" [ADC counts] (maximum ADC value)"<< std::fixed;
        os << " Read-out Frequency:" << getFrequency() << " [MHz]"<< std::endl;
        os << " Pedestal subtracted:" << isPedestalSubtracted() << " Zero Suppressed:"<<isZeroSupressed()<< std::endl;
        os.unsetf( std::ios_base::fixed );
        os.precision( PrecisionBefore );
        os.width( WidthBefore );
    }

    std::ostream &operator<<(std::ostream &os, const GenericADCElectronicsParameters &p)
    {
        p.print(os);
        return os;
    }

    #ifdef USE_LCCD
	GenericADCElectronicsParameters::key_type GenericADCElectronicsParameters::conditions_key() const {

		return getReadoutGroup();
	}
	#endif

}
