/* Pedestal.cc
 * $Id$ 
 */

#include "Pedestal.h"

namespace tpcconddata{

    std::string Pedestal::getRevision()
    {
        return std::string("$Rev: 2567 $");
    }
    
    
    std::string Pedestal::getDefaultColName()
    {
        return std::string("TPCPedestal");
    }
    
        
    Pedestal::Pedestal(int channelID, int readoutGroup, float pedestalValue, float pedestalWidth)
    {
        setHardwareID(channelID, readoutGroup);
        setPedestalValue(pedestalValue);
        setPedestalWidth(pedestalWidth);
    }
    
    
    Pedestal::Pedestal(std::pair<int,int> hardwareID,  float pedestalValue, float pedestalWidth)
    {
        setHardwareID(hardwareID.first, hardwareID.second);    
        setPedestalValue(pedestalValue);
        setPedestalWidth(pedestalWidth);
    }
    
        
    Pedestal::Pedestal(int channelID, int readoutGroup)
    {
        setHardwareID(channelID, readoutGroup);
        setPedestalValue(0.0);
        setPedestalWidth(0.0);
    }
    
    
    Pedestal::Pedestal(std::pair< int , int > hardwareID) 
    {
        setHardwareID(hardwareID.first, hardwareID.second);
        setPedestalValue(0.0);
        setPedestalWidth(0.0);
    }
    
    
    Pedestal::~Pedestal() { /* no op*/  }
    
    
    std::pair<int,int> Pedestal::getHardwareID() const
    {
        std::pair<int,int> hardwareid;
        hardwareid.first = getIntVal( 0 );
        hardwareid.second= getIntVal( 1 );
        return hardwareid;
    } 
       
     
    float Pedestal::getPedestalValue() const
    {
        return getFloatVal( 0 );
    } 
    
    
    float Pedestal::getPedestalWidth() const
    {
        return getFloatVal( 1 );
    } 

        
    void Pedestal::setPedestalValue(float pedestalValue)
    {
        obj()->setFloatVal( 0, pedestalValue );
    }
    
    
    void Pedestal::setPedestalWidth(float pedestalWidth)
    {
        obj()->setFloatVal( 1, pedestalWidth );
    }

    #ifdef USE_LCCD
	Pedestal::key_type Pedestal::conditions_key() const {

		return getHardwareID();
	}
	#endif
    
    
    void Pedestal::setHardwareID(int channelID, int readoutGroup)
    { 
        obj()->setIntVal( 0, channelID );
        obj()->setIntVal( 1, readoutGroup );
    }
    
        
    void Pedestal::print(  std::ostream& os ) const
    {
        std::streamsize PrecisionBefore = os.precision();
        std::streamsize WidthBefore = os.width();
        os.precision(3);  os.setf(std::ios_base::fixed);
        os << "Pedestal: Channel: " << getHardwareID().first << "ReadoutGroup:" << getHardwareID().second << std::fixed; 
        os << " PedestalValue: " << getPedestalValue() << " PedestalWidth: " << getPedestalWidth() << std::endl;
        os.unsetf( std::ios_base::fixed );
        os.precision( PrecisionBefore );
        os.width( WidthBefore );
    }
    
    
    std::ostream &operator<<(std::ostream &os, const Pedestal &p) {
	p.print(os);
	return os;
    }

}
