#define LP1
//#define DEBUG

#include <lcio.h>

#include <IO/LCWriter.h>
#include <EVENT/LCIO.h>

#include <IMPL/LCEventImpl.h> 
#include <IMPL/LCRunHeaderImpl.h> 
#include <IMPL/LCCollectionVec.h>
#include <IMPL/LCTOOLS.h>
#include <IMPL/TrackerRawDataImpl.h>
#include <IMPL/TrackerHitImpl.h>
#include <IMPL/LCFlagImpl.h>

#ifdef LP1
#include <gearimpl/VersatileDiskRowLayout.h>
#else
#include <gearimpl/RectangularPadRowLayout.h>
#endif

#include <gearimpl/Util.h>
#include <gearxml/GearXML.h>
#include <gear/GearMgr.h>
#include <gear/GEAR.h>

// global constants from Marlin, used for the global pointer to the GearMgr
#include <marlin/Global.h>

#include "ADCChannelMapping.h"
#include "Pedestal.h"

#include <iostream>
#include <sstream>
#include <string>
#include <cstdio>
#include <typeinfo>
#include <CLHEP/Random/RandGauss.h>
#include <CLHEP/Random/RandFlat.h>

#include "helix.h"
#include <TRandom1.h>
#include "EXTPCMeasLayer.h"
#include <TKalDetCradle.h>
#include <TKalTrackState.h>
#include <TKalTrackSite.h>
#include <TVTrackHit.h>
#include <EXTPCKalDetector.h>
#include <EXTPCMeasLayer.h>
#include <EXTPCHit.h>
#include <EXEventGen.h>
#include <TObjArray.h>

using namespace std ;
using namespace lcio ;
using namespace gear;
using namespace tpcconddata;

void printUsage()
{
  cout << "generateHelix gearfile.xml [outfile.slcio]" <<endl;
    cout <<" outfile name is optional, "
	 << "default is TPCRawData.slcio"<< endl;
}

int main(int argc, char **argv)
{

  if (argc!=2 && argc!=3 )
  {
    printUsage();
    return(1);
  }
  
  // read pad layout from gear
  GearXML gearXML( argv[1] ) ;
  GearMgr* gearMgr = gearXML.createGearMgr() ;
  marlin::Global::GEAR = gearMgr;
 
  // get outfile name
  string outFileName;
  if (argc==3)
    outFileName=argv[2];
  else
    outFileName="TPCRawData.slcio";

  // open outfile			     
  try {
    // create sio writer
    LCWriter* lcWrt = LCFactory::getInstance()->createLCWriter() ;
    
    try {
      lcWrt->open( outFileName, LCIO::WRITE_NEW ) ;
    }
    catch (IOException& e) {    
      cout << e.what() << endl ;
      return 0 ;
    }
      
    // create runheader and write it
    // the information should come from a file
    // or be given interactively in a future version
    LCRunHeaderImpl* runHdr = new LCRunHeaderImpl ; 
    int runNr=0;
    runHdr->setRunNumber( runNr) ;
      
    string detName("lp1")  ;
    runHdr->setDetectorName( detName ) ;
      
    stringstream description ; 
    description << "lp1 rawData along straight line";
    runHdr->setDescription( description.str()  ) ;
      
    string tpcName("lp1") ;
    runHdr->addActiveSubdetector( tpcName ) ;
      
    lcWrt->writeRunHeader( runHdr ) ;
  
    EXTPCKalDetector &tpcdet=*EXTPCKalDetector::GetInstance(); // TPC (tpc)
    TKalDetCradle    &lp1= const_cast<TKalDetCradle &>(dynamic_cast<const TKalDetCradle &>(tpcdet.GetParent()));
	lp1.SwitchOffMS();

    TObjArray kalhits;                // array to store hits
    EXEventGen gen(lp1, kalhits);  // create event generator

    /////////FIXME//////////////////
    double pt=5;
    double cosmin=-0.05;
    double cosmax=+0.05;
    double phimin=-TMath::Pi()/2-0.05;//0.01
    double phimax=-TMath::Pi()/2+0.05;//0.01
     
    // loop the events, generate 11 events
    int maxEvents=1000;
    for (int eventNr=0; eventNr<maxEvents; eventNr++)
    {
      // create new lcio event
      LCEventImpl*  evt = new LCEventImpl() ;

	  LCCollectionVec* tpcHitCollection = new LCCollectionVec(LCIO::TRACKERHIT);
	  tpcHitCollection->setTransient(0);

      evt->setRunNumber(  runNr   ) ;
      evt->setEventNumber( eventNr ) ;
      evt->setDetectorName( detName ) ;
      
      // we want to store a collection of trackerRawData
      LCCollectionVec* trackerRawDataVec = new LCCollectionVec( LCIO::TRACKERRAWDATA )  ;
      LCFlagImpl flag(0);
      flag.setBit( LCIO:: TRAWBIT_ID1);
      trackerRawDataVec->setFlag( flag.getFlag() );

      kalhits.Delete(); // clear hit data

      THelicalTrack hel = gen.GenerateHelix(pt, cosmin, cosmax, 
               	                            phimin, phimax,
			                    TVector3(0.,0.,150.));
      gen.Swim(hel);

      TVTrackHit * thp=0;
      TIter next(&kalhits); 
      while((thp=static_cast<TVTrackHit *>(next())))
      { 
        const EXTPCMeasLayer & ml = static_cast<const EXTPCMeasLayer &>(thp->GetMeasLayer());
        TVector3 xv=ml.HitToXv(*thp);

		double position[3];

		position[0] = xv.X();
		position[1] = xv.Y();
		position[2] = xv.Z();

	    TrackerHitImpl* theHit = new TrackerHitImpl();
	    theHit->setPosition(position);
		tpcHitCollection->addElement(theHit);


/*
		//produce noise
		double noiseHitPhi = CLHEP::RandFlat::shoot(0., 6.28);
		double noiseHitZ = CLHEP::RandFlat::shoot(0., 500.);
		double noisePosition[3];

		double layerR = ml.GetR();
		noisePosition[0] = layerR * cos(noiseHitPhi);
		noisePosition[1] = layerR * sin(noiseHitPhi);
		noisePosition[2] = noiseHitZ;

		TrackerHitImpl* noiseHit = new TrackerHitImpl();
	    noiseHit->setPosition(noisePosition);
		tpcHitCollection->addElement(noiseHit);
*/
      }


/*
	  //generate the second track
	  kalhits.Delete(); // clear hit data

      hel = gen.GenerateHelix(-pt, cosmin, cosmax, 
               	                            phimin, phimax,
			                    TVector3(0.,0.,160.));
      gen.Swim(hel);

      thp=0;
      TIter next1(&kalhits); 
      while ((thp=static_cast<TVTrackHit *>(next1())))
      { 
        const EXTPCMeasLayer & ml = static_cast<const EXTPCMeasLayer &>(thp->GetMeasLayer());
        TVector3 xv=ml.HitToXv(*thp);

		double position[3];

		position[0] = xv.X();
		position[1] = xv.Y();
		position[2] = xv.Z();

	    TrackerHitImpl* theHit = new TrackerHitImpl();
	    theHit->setPosition(position);
		tpcHitCollection->addElement(theHit);
      }
*/

/*
	  //generate the third track
	  kalhits.Delete(); // clear hit data

      hel = gen.GenerateHelix(-pt, cosmin, cosmax, 
               	                            phimin, phimax,
			                    TVector3(0.,0.,170.));
			                    //TVector3(50.,50.,200.));
      gen.Swim(hel);

      thp=0;
      TIter next2(&kalhits); 
      while ((thp=static_cast<TVTrackHit *>(next2())))
      { 
        const EXTPCMeasLayer & ml = static_cast<const EXTPCMeasLayer &>(thp->GetMeasLayer());
        TVector3 xv=ml.HitToXv(*thp);

		double position[3];

		position[0] = xv.X();
		position[1] = xv.Y();
		position[2] = xv.Z();

	    TrackerHitImpl* theHit = new TrackerHitImpl();
	    theHit->setPosition(position);
		tpcHitCollection->addElement(theHit);
      }

*/

/*
   //generate curler
   //set detector layer
   //full end plate
   const int nlayers = 240;
   Circle layers[nlayers];

   double radius = 408.;

   for(int n = 0; n < nlayers; n++)
   {
      if( (n>=30) && (n%30==0) ) radius += 21;
      layers[n].setR(radius);
      cout << "layer:" << n << "  ridius:" << radius << endl;

      radius += 5.;
  
      //center is (0, 0) by default 
   }

	  const double SIGMA = 50;

      //setting helix
	  double   phi =  CLHEP::RandFlat::shoot(0., 6.28);
	  double   hc_x   = 1000*cos(phi) + gRandom->Gaus(0., SIGMA);
      double   hc_y   = 1000*sin(phi) + gRandom->Gaus(0., SIGMA);
      double   h_r    = 500;
      double   lambda = 0.1;
      int      sign   = -1;
      TVector3 X0(1,-1000,800);

      Helix h1(hc_x, hc_y, h_r, X0, lambda, sign);

      for(int n = 0; n < nlayers; n++)
      {
         cout << "--------------" << endl;

         vector<TVector3> xp;
         //h1.calcXPWithLayer(layers[n], xp);

         if(h1.calcXPWithLayer(layers[n], xp))
         {
            for(unsigned int i = 0; i < xp.size(); i++)
            {
               double position[3];
  
               TVector3 xx = xp.at(i);

			   //if(xx.X()<0.||xx.Z()>1000.) continue;
			   if(xx.Z()>2000.) continue;
               position[0] = xx.X();
               position[1] = xx.Y();
               position[2] = xx.Z();


               TrackerHitImpl* theHit = new TrackerHitImpl();
               theHit->setPosition(position);

               //add hit to collection
               tpcHitCollection->addElement(theHit);
            }//for 2 intersections
         }//if has intersections
      }//for layers
*/

      // add the rawData collection to the event
	  evt->addCollection( tpcHitCollection, "TPCHits" );

      // write the event to the file
      lcWrt->writeEvent( evt ) ;
	
#ifdef DEBUG
      LCTOOLS::dumpEvent( evt ) ;
#endif

      // ------------ IMPORTANT ------------- !
      // we created the event so we need to delete it ...
      delete evt ;
      // -------------------------------------

    } // for eventNr

    delete runHdr ;
      
    lcWrt->close() ;
      
  } catch (std::exception& ex) {
    
    cout << " an excpetion occured: " << endl ;
    cout << "  " << ex.what() << endl ;
    return 1 ;
  }

  return 0;
}
