#include "helix.h"

#include "lcio.h"
#include "IO/LCWriter.h"
#include "EVENT/LCIO.h"

#include "IMPL/LCEventImpl.h"
#include "IMPL/LCRunHeaderImpl.h"
#include "IMPL/LCCollectionVec.h"
#include "IMPL/LCTOOLS.h"
#include "IMPL/TrackerRawDataImpl.h"
#include "IMPL/LCFlagImpl.h"
#include "IMPL/TrackerHitImpl.h"

#include "gearimpl/Util.h"
#include "gearxml/GearXML.h"
#include "gear/GearMgr.h"
#include "gear/GEAR.h"
#include <marlin/Global.h>

#include <iostream>
#include <sstream>
#include <string>
#include <cstdio>
#include <typeinfo>
#include <CLHEP/Random/RandGauss.h>

#include "TRandom1.h"

using namespace std ;
using namespace lcio ;
using namespace gear;

int main(int argc, char **argv)
{
#if 0
   //read pad layout from gear
   GearXML gearXML( argv[1] ) ;
   GearMgr* gearMgr = gearXML.createGearMgr() ;
   marlin::Global::GEAR = gearMgr;
   TPCParameters const &TPCParams=gearMgr->getTPCParameters();
#endif

   string outFileName;
   if (argc==3)
     outFileName=argv[2];
   else
     outFileName="TPCHits.slcio";

   try {
      LCWriter* lcWrt = LCFactory::getInstance()->createLCWriter() ;

      try {
          lcWrt->open( outFileName, LCIO::WRITE_NEW ) ;
      }
      catch (IOException& e) {
      cout << e.what() << endl ;
      return 0 ;
    }

   //create runheader and write it
   LCRunHeaderImpl* runHdr = new LCRunHeaderImpl ;
   int runNr=0;
   runHdr->setRunNumber( runNr) ;

   string detName("tpc")  ;
   runHdr->setDetectorName( detName ) ;

   stringstream description ;
   description << "tpc hits";
   runHdr->setDescription( description.str()  ) ;

   string tpcName("tpc") ;
   runHdr->addActiveSubdetector( tpcName ) ;
   lcWrt->writeRunHeader( runHdr );

   //set detector layer
   //full end plate
   const int nlayers = 240;
   Circle layers[nlayers];

   double radius = 408.;

   for(int n = 0; n < nlayers; n++)
   {
      if( (n>=30) && (n%30==0) ) radius += 21;
      layers[n].setR(radius);
      cout << "layer:" << n << "  ridius:" << radius << endl;

      radius += 5.;
  
      //center is (0, 0) by default 
   }

   const int maxEvents = 100;
   for(int eventNr=0; eventNr<maxEvents; eventNr++)
   {
      LCEventImpl*  evt = new LCEventImpl();

      evt->setRunNumber   (  runNr  );
      evt->setEventNumber ( eventNr );
      evt->setDetectorName( detName );

      LCCollectionVec* tpcHitCollection = new LCCollectionVec(LCIO::TRACKERHIT);
      tpcHitCollection->setTransient(0);
      
      //setting helix
#if 0
      double   hc_x   = -270;
      double   hc_y   = 750; 
      double   h_r    = 825; 
      double   lambda = 0.3;
      int      sign   = -1;
      TVector3 X0(1,-1000,600);
#endif

	  const double SIGMA = 50;
	  double   hc_x   = 0. + gRandom->Gaus(0., SIGMA);
      double   hc_y   = 1000 + gRandom->Gaus(0., SIGMA);
      double   h_r    = 500;
      double   lambda = 0.1;
      int      sign   = -1;
      TVector3 X0(1,-1000,800);

      Helix h1(hc_x, hc_y, h_r, X0, lambda, sign);

      for(int n = 0; n < nlayers; n++)
      {
         cout << "--------------" << endl;

         vector<TVector3> xp;
         //h1.calcXPWithLayer(layers[n], xp);

         if(h1.calcXPWithLayer(layers[n], xp))
         {
            for(unsigned int i = 0; i < xp.size(); i++)
            {
               double position[3];
  
               TVector3 xx = xp.at(i);

			   //if(xx.X()<0.||xx.Z()>1000.) continue;
			   if(xx.Z()>2000.) continue;
               position[0] = xx.X();
               position[1] = xx.Y();
               position[2] = xx.Z();


               TrackerHitImpl* theHit = new TrackerHitImpl();
               theHit->setPosition(position);

               //add hit to collection
               tpcHitCollection->addElement(theHit);
            }//for 2 intersections
         }//if has intersections
      }//for layers

      //add hits collection to event
      evt->addCollection( tpcHitCollection, "TPCHits" );
      lcWrt->writeEvent( evt ) ;

      delete evt;
    }

    delete runHdr ;
    lcWrt->close() ;
  }
  catch (std::exception& ex) {
    cout << " an excpetion occured: " << endl ;
    cout << "  " << ex.what() << endl ;
    return 1 ;
  }

  return 0;
}
