#ifndef HISTOGRAMSIMTRACKERHITCHARGESPECTRUMPROCESSOR_H
#define HISTOGRAMSIMTRACKERHITCHARGESPECTRUMPROCESSOR_H

#include <marlin/Processor.h>
#include <lcio.h>
#include <string>

// In case you want AIDA histograms
namespace AIDA
{
        class IHistogram1D;
}

namespace marlintpc
{

/**  Histogram the charge of SimTrackerHits in units of e 
 *   (hardcoded value 26 eV for Argon).
 * 
 * @author M. Killenberg, CERN
 *
 */

class HistogramSimTrackerHitChargeSpectrumProcessor : public marlin::Processor
{
 public:
  
  virtual Processor*  newProcessor() { return new HistogramSimTrackerHitChargeSpectrumProcessor; }
  
  HistogramSimTrackerHitChargeSpectrumProcessor();
  
  virtual void init();
  
  virtual void processRunHeader(EVENT::LCRunHeader* run);
  
  virtual void processEvent(EVENT::LCEvent* evt);
  
  virtual void check(EVENT::LCEvent* evt);
  
  virtual void end();
  
  
 protected:
  /* the place for protected and private member data and functions */
  std::string _inputColName; ///< Name of the input collection

  AIDA::IHistogram1D * _myAidaHistogram; ///< Pointer to the histogram
  int _histoNBins; ///< Number of bins in the histogram
  float _histoMin; ///< Minimum of the histogram
  float _histoMax; ///< Maximum of the histogram

};
} // namespace marlintpc
#endif // HISTOGRAMSIMTRACKERHITCHARGESPECTRUMPROCESSOR_H
