#include "PrintEventNumberProcessor.h"

// C++
#include <iostream>


// Aida
#ifdef MARLIN_USE_AIDA
//AIDA
#include <AIDA/AIDA.h>
#include <marlin/AIDAProcessor.h>
#endif

// LCIO
#include "UTIL/LCTOOLS.h"

using namespace lcio ;
using namespace marlin ;

namespace marlintpc{


    PrintEventNumberProcessor aPrintEventNumberProcessor ;
    

    PrintEventNumberProcessor::PrintEventNumberProcessor() : 
      Processor("PrintEventNumberProcessor") {
	/* the constructor call first the super constructor which the name 
	 * of the processor
	 * this must by the same as the class name.
	 */
	
	/* description of the processor which will be displayed in the
	 * steering file autometicly generated by 
	 * marlin (use MarlinTPC -x (XML-format) or MarlinTPXC -l (text format))
	 */
	_description = "PrintEventNumberProcessor prints event number to screen"
	  " depending on the verbosity level" ;	
	
	/* register steering parameters: name, description, class-variable, default value
	 * the type will by definded by the type of the default value
	 * string, double, float and int are possible
	 * plaese note to get a float you must cast in C++: (float) 1.3 
	 */
	registerProcessorParameter( "EveryNEvents" , 
				    "Print event number every n Events (default 100)"  ,
				    _everyNEvents ,
				    100 ) ;
	
    }
    
    
    void PrintEventNumberProcessor::init() { 
	
	
    }
    
    void PrintEventNumberProcessor::processRunHeader( LCRunHeader* run) { 
	
      
      run->parameters().setValue( _processorName + "_revision", "$Rev: 699 $");

      for ( ProcParamMap::iterator i = _map.begin(); i != _map.end(); i++ ) {
	if ( ! i->second->isOptional() || i->second->valueSet() ) {
	  run->parameters().setValue( _processorName + "_" + i->second->name(), 
				      i->second->value() );
	}
      }

    } 
    
    void PrintEventNumberProcessor::processEvent( LCEvent * evt ) { 

      
      if ( evt->getEventNumber()%_everyNEvents == 0 )
      {
	  std::cout <<"Event "<<evt->getEventNumber()<<std::endl;
      }
    }
    
    
    void PrintEventNumberProcessor::check( LCEvent *  /*evt*/ ) { 



    }

    
    
    void PrintEventNumberProcessor::end(){ 
      /*	
      streamlog_out(MESSAGE4) << "PrintEventNumberProcessor::end()  " << name() 
		<< " processed " << evt->getEventNumber() 
		<< " events in " << _nRun << " runs "
		<< std::endl ;
      */	

    }
    
}
