#include "lcio.h"

#include "IO/LCReader.h"
#include "IMPL/LCTOOLS.h"
#include "EVENT/LCRunHeader.h"
#include "IMPL/LCRunHeaderImpl.h"
 #include "IMPL/LCGenericObjectImpl.h"
#include "IMPL/LCCollectionVec.h"

#include <cstdio>
#include <string>


int main(int  /*argc*/, char*  /*argv*/[] )
{
    lcio::LCReader* lcReader = lcio::LCFactory::getInstance()->createLCReader() ;
    lcReader->open( "PedestalSimpleFile_0.slcio" );

    EVENT::LCEvent* evt=lcReader->readNextEvent();
    lcio::LCCollection* inputCol = evt->getCollection("TPCPedestals");
    lcio::LCGenericObjectImpl* pedestalObject;
    
    bool valuesCorrect=true;
    
    int nEntries = inputCol->getNumberOfElements();
    
    std::cout<<"Testing pedestal entries:"<<std::endl;
    std::cout<<"Entry# module pad  value (expected)  width  (expected)  status"<<std::endl;
    std::cout.precision(4);
    std::cout.setf(std::ios::fixed,std::ios::floatfield); 
    // loop over the number of elements of the input collection
    for (int iEntry = 0; iEntry < nEntries ; iEntry++)    
    {
        // get the pedestals (stored in LCGenericObjects) from the collection
        pedestalObject = dynamic_cast<lcio::LCGenericObjectImpl*>(inputCol->getElementAt(iEntry));
        // read out the stored values from the generic object
        int pad=pedestalObject->getIntVal(0);
        int module=pedestalObject->getIntVal(1);
        float pedestalValue=pedestalObject->getFloatVal(0);
        float pedestalWidth=pedestalObject->getFloatVal(1);
        // since input data was simulated with random generator
        // calculate pedestal values differ slightly from input values
        // so they are checked, if they are inside a certain range
        float rangeValue=0.010;
        float rangeWidth=0.10; //width usually differs a bit more 
        if (iEntry<10) std::cout<<" ";
        std::cout<<" "<<iEntry<<"     "<<module<<"     ";
        if (pad<10) std::cout<<" ";
        std::cout<<pad<<"   "<<pedestalValue<<" ("<<(3.0*(float)module+5.0)<<")   "<<pedestalWidth<<"  ("<<1.0+(float)module<<")    ";
        if (   ( (pedestalValue>(3.0*(float)module+5.0-rangeValue)) && (pedestalValue<(3.0*(float)module+5.0+rangeValue)) )   
            && ( (pedestalWidth>((float)module+1.0-rangeWidth)) && (pedestalWidth<((float)module+1.0+rangeWidth))  )     )           
        {
            std::cout<<"ok";
        }
        else
        {
            std::cout<<"failed";
            valuesCorrect=false;
        }
        std::cout<<std::endl;
        
    }
    std::cout<<"Test result: ";
    if (valuesCorrect) 
        std::cout<<"ok";
    else
        std::cout<<"error";
    std::cout<<std::endl;

    if (valuesCorrect) return 0;
    else return 1;
}
