#ifndef KALTRACKINGHELIXTESTER_H
#define KALTRACKINGHELIXTESTER_H

#include <utility> // for the std::pair

namespace gear
{
  class TPCParameters;
  class BField;
  class GearMgr;
  class TPCModule;
  class PadRowLayout2D;
}

namespace marlintpc
{
  class KalTrackingHelix;

  /**
   * Class to test KalTrackingHelix. This class is a friend of KalTrackingHelix and
   * has access to the protected members, so it can prepare the internal variables to a certain
   * state and test the member functions.
   * 
   * The tests always a return a pair of unsigned ints. The first value is tne number of
   * failed tests, the second value the total number of tests executed.
   */
  class KalTrackingHelixTester
  {
  public:
    KalTrackingHelixTester();
    KalTrackingHelixTester(KalTrackingHelixTester const &);
    KalTrackingHelixTester & operator=(KalTrackingHelixTester const &);
    virtual ~KalTrackingHelixTester();

    std::pair<unsigned int, unsigned int> testGetTrack();
    std::pair<unsigned int, unsigned int> testSearchTrack();
    std::pair<unsigned int, unsigned int> testSearchTrackHit();
    std::pair<unsigned int, unsigned int> testDoAddAndFilter();
    std::pair<unsigned int, unsigned int> testCalculateDeltaChi2();/* trivial */
    std::pair<unsigned int, unsigned int> testReset();
    std::pair<unsigned int, unsigned int> testLCIOParameters();
    std::pair<unsigned int, unsigned int> testCreateGearHit(); /*implemented*/
    std::pair<unsigned int, unsigned int> testExtrapolateIntoNextLayer();
    std::pair<unsigned int, unsigned int> testCalculateStartHelix();
    std::pair<unsigned int, unsigned int> testToBaseRange(); /* implemented */
    std::pair<unsigned int, unsigned int> getNearestPointInNextLayer();/* implemented */

    std::pair<unsigned int, unsigned int> runAllTests(); /* implemented */

  protected:
    gear::GearMgr * gearMgr; //< the gear manager
    gear::TPCParameters *tpcParameters; //< the TPCParameters
    gear::TPCModule * module; //< the (only) module
    gear::PadRowLayout2D * localPadLayout; //< the local pad plane
    gear::BField *bField; //< the magnetic field

    KalTrackingHelix * kalTrackingHelix;
 };
}//namespace

#endif// KALTRACKINGHELIXTESTER_H
