#include "SignalSplitter.h"

#include <iostream>
#include <cmath>
#include <cstdlib>
using namespace std;
using namespace marlintpc;

int main(int argc, char* argv[])
{
  if (argc!=7)
  {
    cout << "usage: testSignalSplitter binWidth sigma1 sigma2 offset amplitude1 amplitude2" << std::endl;
    return 1;
  }

  std::vector<float> signal1;
  signal1.push_back( 12 );
  signal1.push_back( 24 );
  signal1.push_back( 48 );
  signal1.push_back( 36 );
  signal1.push_back( 48 );
  signal1.push_back( 24 );
  signal1.push_back( 12 );

  SignalSplitter splitter;
  std::vector< std::pair< size_t, std::vector<float> > > splitSignal = 
    splitter.splitSignal( signal1 );

//  for ( std::vector< std::pair< size_t, std::vector<float> > >::iterator it = splitSignal.begin();  it < splitSignal.end(); it++ )
//  {
//    cout << "offset "<< it->first << " ; signal ";
//    for ( std::vector<float>::iterator it2 = it->second.begin(); 
//	  it2 < it->second.end(); it2++ )
//    {
//      cout << *it2<< " ";
//    }
//    cout << std::endl;
//  }
  
  // create two pulses being real gaussians
  // the middle of the first pulse is 3*sigma1, the middle of the second pulse shiftet 
  // by offset wrt. the first pulse, so it's 3*sigma1+offset
  std::vector<float> inputPulse1, inputPulse2, sumPulse;;
  double binWidth=atof(argv[1]);
  double sigma1=atof(argv[2]);
  double sigma2=atof(argv[3]);
  double offset=atof(argv[4]);
  double signalLength= ( 3*sigma1 > offset+3*sigma2 ? 6*sigma1 :  3*sigma1+offset+3*sigma2 ) ;
  double A1=atof(argv[5]);
  double A2=atof(argv[6]);
  double factor1a=A1*sigma1*sqrt(M_PI_2);//cout << "factor1a="<< factor1a<<endl;
  double factor1b=sigma1* M_SQRT2;//cout << "factor1b="<< factor1b<<endl;
  double factor2a=A2*sigma2*sqrt(M_PI_2);//cout << "factor2a="<< factor2a<<endl;
  double factor2b=sigma2* M_SQRT2;//cout << "factor2b="<< factor2b<<endl;
  for ( double pos = 0 ; pos < signalLength; pos+=binWidth )
  {
    //    cout << "pos="<< pos << " , erf( " <<(pos)/factor2 <<" ) - erf( " <<(pos+binWidth)/factor2 << " )"<<std::endl;
    // the integral of a normalised gaussian is 1/2 * [ 1 + erf( (x-mu)/sigma/sqrt(2) ) ] , 
    // so the bin content of a gaussian with amplitule A  is
    // A* sigma * sqrt( pi / 2 ) * [ erf( (x_stop-mu)/sigma/sqrt(2) ) -  erf( (x_start -mu)/sigma/sqrt(2) )
    double binContent1 = factor1a* ( erf( (pos-3*sigma1+binWidth)/factor1b ) - erf( (pos-3*sigma1)/factor1b ) );
    double binContent2 = factor2a* ( erf( (pos-3*sigma1-offset+binWidth)/factor2b ) - erf( (pos-3*sigma1-offset)/factor2b ) );

    inputPulse1.push_back(binContent1);
    inputPulse2.push_back(binContent2);
    sumPulse.push_back(binContent1+binContent2);
    
  }

  double sum=0; double sumPos=0;
  cout << "#inputPulse1: ";
  for ( vector<float>::iterator it=inputPulse1.begin(); it<inputPulse1.end(); it++)
  {
    cout <<*it << " " ;
    sum += *it;
    sumPos += *it* (distance(inputPulse1.begin(),it)+0.5) *binWidth;
  }
  cout << " , sum="<<sum << " , cog = " << sumPos/sum << std::endl;

  sum=0; sumPos=0;
  cout << "#inputPulse2: ";
  for ( vector<float>::iterator it=inputPulse2.begin(); it<inputPulse2.end(); it++)
  {
    cout <<*it << " " ;
    sum += *it;
    sumPos += *it* (distance(inputPulse2.begin(),it)+0.5) *binWidth;
  }
  cout << " , sum="<<sum << " , cog = " << sumPos/sum << std::endl;
  cout << std::endl;

  sum=0; sumPos=0;
  cout << "#sumPulse: ";
  for ( vector<float>::iterator it=sumPulse.begin(); it<sumPulse.end(); it++)
  {
    sum += *it;
    sumPos += *it* (distance(sumPulse.begin(),it)+0.5) *binWidth;
    cout <<*it << " " ;
  }
  cout << " , sum="<<sum << " , cog = " << sumPos/sum << std::endl;
  cout << std::endl;

  splitSignal = splitter.splitSignal( sumPulse );
  for ( std::vector< std::pair< size_t, std::vector<float> > >::iterator it = splitSignal.begin();  it < splitSignal.end(); it++ )
  {
    sum=0; sumPos=0;
    cout << "#offset "<< it->first << " ; signal ";
    for ( std::vector<float>::iterator it2 = it->second.begin(); 
	  it2 < it->second.end(); it2++ )
    {
      sum += *it2;
      sumPos += *it2* (distance(it->second.begin(),it2)+0.5) *binWidth;
      cout << *it2<< " ";
    }
   cout << " , sum="<<sum << " , cog = " << sumPos/sum + it->first*binWidth<< std::endl;
  }

  // gnuplottable output
  std::cout << "# i in1 in2 sumPulse split1 split2" <<std::endl;
  for (int i = 0; i <  static_cast<int>(sumPulse.size()); i++ )
  {
    cout << i <<"\t" << inputPulse1[i]
	 <<"\t" << inputPulse2[i] 
	 <<"\t" << sumPulse[i] 
	 <<"\t" << (i<  static_cast<int>(splitSignal[0].second.size()) ? splitSignal[0].second[i] : 0 )
	 <<"\t" << ( (i- static_cast<int>(splitSignal[1].first)) >=0 ?  splitSignal[1].second[i- splitSignal[1].first] :0 )<< std::endl;
  }

  return 0;
}
