#include <gearimpl/TPCParametersImpl.h>
#include <gearimpl/FixedPadSizeDiskLayout.h>
#include <gearimpl/TPCModuleImpl.h>
#include <IMPL/LCCollectionVec.h>
#include <IMPL/TrackerHitImpl.h>
#include <EVENT/LCIO.h>
#include <iostream>

#include "TPCHitMap.h"

using namespace gear;
using namespace marlintpc;
using namespace IMPL;
using namespace EVENT;
using namespace std;

LCCollectionVec hitCollection(LCIO::TRACKERHIT);

int main()
{
  // initialise the TPCParameters with 600 mm drift length
  // and kartesian coordinates
  TPCParametersImpl tpcParameters(600, PadRowLayout2D::CARTESIAN);
  
  // 10 cm rows with 1 cm wide pads from 10 cm to 1 m radius
  PadRowLayout2D *padLayout = new FixedPadSizeDiskLayout( 100 /*rMin*/, 1000 /*rMax*/,
							  100 /*padHeight*/, 10 /*PadWidth*/);
  
  int moduleID = 0;
  TPCModule *module = new TPCModuleImpl(moduleID, padLayout,  
					tpcParameters.getCoordinateType(), 40. /*readoutFrequency*/);
  tpcParameters.addModule(module);

  //FIXME::
  TPCHitMap hitMap(&tpcParameters, 0);

  // create two hits for each pad row

  for (int row = 0; row < padLayout->getNRows() ; ++row)
  {
    //FIXME: The interface does not quarantee that there is a pad 0 in the row
    double r = padLayout->getPadCenter( padLayout->getPadIndex(row, 0) )[0];
    
    // just create two straight line. one at phi = 0.1 rad, the other one at phi = -0.2 rad
    double phi = 0.1;
    gear::Vector2D globalPositionVector = module->localToGlobal(r, phi);

    double position[3];
    position[0]=globalPositionVector[0];
    position[1]=globalPositionVector[1];
    position[2]=0;

    TrackerHitImpl * hit = new TrackerHitImpl;
    hit->setPosition(position);
    hitCollection.addElement(hit);

    phi = -0.1;
    globalPositionVector = module->localToGlobal(r, phi);

    position[0]=globalPositionVector[0];
    position[1]=globalPositionVector[1];
    position[2]=0;

    hit = new TrackerHitImpl;
    hit->setPosition(position);
    hitCollection.addElement(hit);
  }

  // start the testing and keep track how many tests failed.
  unsigned int nTestsFailed=0;
  unsigned int nTestsTotal=0;

  // the map should be empty
  ++nTestsTotal;
  if (!hitMap.isEmpty())
  {
    cout << "hit map is not empty, but should be" << endl;
    ++nTestsFailed;
  }
  
  hitMap.fillEventHits(&hitCollection);

  // the map should be not empty
  ++nTestsTotal;
  if (hitMap.isEmpty())
  {
    cout << "hit map is empty, but should not be" << endl;
    ++nTestsFailed;
  }

  //check isUsed flag
  for(int row = 0; row < padLayout->getNRows() ; ++row)
  {
     std::set< std::pair<int, int> >  moduleRowsInDomain;
	 moduleRowsInDomain.insert( pair<int, int>(0, row) );

     std::list<ModuleRowHit>  hitList = hitMap.getHitsInDomain(moduleRowsInDomain);

	 const unsigned int HITSIZE = 2; 

	 for(unsigned int n = 0; n < HITSIZE; n++)
	 {
		++nTestsTotal;
		ModuleRowHit hit = hitList.front();
		if( hit.isUsed() ) 
		{
			cout << "Test failed:  the isUsed flag should be false"
				 << hit.module << "  " << hit.row << "  "
				 << endl;

			++nTestsFailed;
		}

		hitList.pop_front();
	 }

  }


  //set isUsed flag true
  for(int row = 0; row < padLayout->getNRows() ; ++row)
  {
     std::set< std::pair<int, int> >  moduleRowsInDomain;
	 moduleRowsInDomain.insert( pair<int, int>(0, row) );

     std::list<ModuleRowHit>  hitList = hitMap.getHitsInDomain(moduleRowsInDomain);

	 const unsigned int HITSIZE = 2; 

	 for(unsigned int n = 0; n < HITSIZE; n++)
	 {
		++nTestsTotal;
		ModuleRowHit hit = hitList.front();
		hitMap.setHitUsed(hit, true);
		hitList.pop_front();
	 }
  }

  //check the content we get now
  for(int row = 0; row < padLayout->getNRows() ; ++row)
  {
     std::set< std::pair<int, int> >  moduleRowsInDomain;
	 moduleRowsInDomain.insert( pair<int, int>(0, row) );

     std::list<ModuleRowHit> hitList = hitMap.getHitsInDomain(moduleRowsInDomain);
     ++nTestsTotal;

	 if(hitList.size())
	 {
		 cout << "size is " << hitList.size() 
			  << ", but the content in the hitList should be 0" << endl;

		 ++nTestsFailed;
	 }
  }

  // let's remove all the hits:
  for(int i=0; i<9; i++)
  {
	  ModuleRowHit hit1(moduleID, i, dynamic_cast<TrackerHit*>(hitCollection[2*i]));
	  ModuleRowHit hit2(moduleID, i, dynamic_cast<TrackerHit*>(hitCollection[2*i]));

	  hitMap.removeHit(hit1);
	  hitMap.removeHit(hit2);
  }

  ++nTestsTotal;
  if(hitMap.size())
  {
	  cout << "The size of hit map should be 0" << endl;
	  ++nTestsFailed;
  }

  cout << nTestsFailed << " out of "<<nTestsTotal <<" tests failed." <<endl;
  return nTestsFailed;
}
