#define LP1
//#define DEBUG

#include <lcio.h>

#include <IO/LCWriter.h>
#include <EVENT/LCIO.h>

#include <IMPL/LCEventImpl.h>
#include <IMPL/LCRunHeaderImpl.h> 
#include <IMPL/LCCollectionVec.h>
#include <IMPL/LCTOOLS.h>
#include <IMPL/TrackerRawDataImpl.h>
#include <IMPL/LCFlagImpl.h>

#ifdef LP1
#include <gearimpl/VersatileDiskRowLayout.h>
#else
#include <gearimpl/RectangularPadRowLayout.h>
#endif

#include <gearimpl/Util.h>
#include <gearxml/GearXML.h>
#include <gear/GearMgr.h>
#include <gear/GEAR.h>

// global constants from Marlin, used for the global pointer to the GearMgr
#include <marlin/Global.h>

#include "ADCChannelMapping.h"
#include "Pedestal.h"

#include <iostream>
#include <sstream>
#include <string>
#include <cstdio>
#include <typeinfo>
#include <CLHEP/Random/RandGauss.h>

#include <TRandom1.h>
#include <TNtupleD.h>         // from ROOT
#include <TFile.h>            // from ROOT
#include <TKalTrack.h>
#include <TKalDetCradle.h>
#include <TKalTrackState.h>
#include <TKalTrackSite.h>
#include <TVTrackHit.h>
#include <EXTPCKalDetector.h>
#include <GearTPCCylinderHit.h>
#include <EXEventGen.h>
#include <TObjArray.h>

using namespace std ;
using namespace lcio ;
using namespace gear;
using namespace tpcconddata;
using namespace kaldet;

//static const Bool_t gkDir = kIterBackward;
static const Bool_t gkDir = kIterForward;
int main(int argc, char **argv)
{
   TFile hfile("h.root","RECREATE","KalTest");
   TNtupleD *hTrackMonitor = new TNtupleD("track", "", "ndf:chi2:cl:cpa");

  if (argc!=2)
  {
	cout << "Input parameters error, need pad layout file!" << endl;
    return(1);
  }
  
  // read pad layout from gear
  GearXML gearXML( argv[1] ) ;
  GearMgr* gearMgr = gearXML.createGearMgr() ;
  marlin::Global::GEAR = gearMgr;
 
  //TKalDetCradle    lp1; // LP1 detector
  EXTPCKalDetector &tpcdet=*EXTPCKalDetector::GetInstance(); // TPC (tpc)
  TKalDetCradle    &lp1= const_cast<TKalDetCradle &>(dynamic_cast<const TKalDetCradle &>(tpcdet.GetParent()));

  //lp1.SwitchOffMS(); 

  TObjArray kalhits;                // array to store hits
  EXEventGen gen(lp1, kalhits);  // create event generator


  /////////FIXME//////////////////
  double pt=5.;
  double cosmin=-0.05;
  double cosmax=+0.05;
  double phimin=-TMath::Pi()/2-0.01;
  double phimax=-TMath::Pi()/2+0.01;
  /////////FIXME//////////////////
     
  int maxEvents=1;
  for (int eventNr=0; eventNr<maxEvents; eventNr++)
  {
    // create new lcio event
    kalhits.Delete(); // clear hit data
    THelicalTrack hel = gen.GenerateHelix(pt, cosmin, cosmax, phimin, phimax, TVector3(0.,0.,150.));
    gen.Swim(hel);

      // ============================================================
      //  Do Kalman Filter
      // ============================================================

      Int_t i1, i2, i3;
      if (gkDir == kIterBackward) {
         i3 = 0;
         i1 = kalhits.GetEntries() - 1;
         i2 = i1 / 2;
      } else {
         i1 = 0;
         i3 = kalhits.GetEntries() - 1;
         i2 = i3 / 2;
      }

      // ---------------------------
      //  Create a dummy site: sited
      // ---------------------------

	  cout << "create dummy site..." << endl;
      GearTPCHit * hitd = dynamic_cast<GearTPCHit *>(kalhits.At(i1));
      (*hitd)(0,1) = 1.e6;   // give a huge error to d
      (*hitd)(1,1) = 1.e6;   // give a huge error to z

      TKalTrackSite &sited = *new TKalTrackSite(*hitd);
      sited.SetOwner();   // site owns states

      // ---------------------------
      // Create initial helix
      // ---------------------------
      cout << "create initial helix..." << endl;
      GearTPCHit   *h1 = dynamic_cast<GearTPCHit *>(kalhits.At(i1));   // first hit
      GearTPCHit   *h2 = dynamic_cast<GearTPCHit *>(kalhits.At(i2));   // last hit
      GearTPCHit   *h3 = dynamic_cast<GearTPCHit *>(kalhits.At(i3));   // middle hit
      TVector3 x1 = h1->GetMeasLayer().HitToXv(*h1);
      TVector3 x2 = h2->GetMeasLayer().HitToXv(*h2);
      TVector3 x3 = h3->GetMeasLayer().HitToXv(*h3);
      THelicalTrack helstart(x1, x2, x3, h1->GetBfield(), gkDir); // initial helix 
	  x1.Print();
	  x2.Print();
	  x3.Print();

      // ---------------------------
      //  Set dummy state to sited
      // ---------------------------

      static TKalMatrix svd(kSdim,1);
      svd(0,0) = 0.;
      svd(1,0) = helstart.GetPhi0();
      svd(2,0) = helstart.GetKappa();
      svd(3,0) = 0.;
      svd(4,0) = helstart.GetTanLambda();
      if (kSdim == 6) svd(5,0) = 0.;

	  svd.Print();

      static TKalMatrix C(kSdim,kSdim);
      for (Int_t i=0; i<kSdim; i++) {
         C(i,i) = 1.e4;   // dummy error matrix
      }

      sited.Add(new TKalTrackState(svd,C,sited,TVKalSite::kPredicted));
      sited.Add(new TKalTrackState(svd,C,sited,TVKalSite::kFiltered));

      // ---------------------------
      //  Add sited to the kaltrack
      // ---------------------------

      TKalTrack kaltrack;    // a track is a kal system
      kaltrack.SetOwner();   // kaltrack owns sites
      kaltrack.Add(&sited);  // add the dummy site to the track

      // ---------------------------
      //  Prepare hit iterrator
      // ---------------------------

      TIter next(&kalhits, gkDir);   // come in to IP

      // ---------------------------
      //  Start Kalman Filter
      // ---------------------------

	  cout << "start Kalman filter..." << endl;
      GearTPCHit *hitp = 0;
      while ((hitp = dynamic_cast<GearTPCHit *>(next()))) {     // loop over hits
         TKalTrackSite  * site = new TKalTrackSite(*hitp); // create a site for this hit
		 TVector3 pivot = site->GetPivot();
		 cout << "site: x = " << pivot.X() << ", y = " << pivot.Y() << ", z = " << pivot.Z() << endl;

         if (!kaltrack.AddAndFilter(*site)) {               // add and filter this site
            cerr << " site discarded!" << endl;           
            delete site;                                  // delete this site, if failed
         }
		 else
		 {
			 cout << "DChi2:" << site->GetDeltaChi2() << endl;
		 }
      }
      //kaltrack.SmoothBackTo(1);                          // smooth back.

      // ============================================================
      //  Monitor Fit Result
      // ============================================================

      Int_t    ndf  = kaltrack.GetNDF();
      Double_t chi2 = kaltrack.GetChi2();
      Double_t cl   = TMath::Prob(chi2, ndf);
      Double_t cpa  = kaltrack.GetCurSite().GetCurState()(2, 0);
      hTrackMonitor->Fill(ndf, chi2, cl, cpa);

  } // for eventNr

  hfile.Write();

  return 0;
}
